@@ -22,7 +22,6 @@ let server = {
     "eliom_common";
     "eliom_config";
     "eliom_content";
-    "eliom_cookie";
     "eliom_extension";
     "eliom_lib";
     "eliom_mkreg";
@@ -59,6 +58,7 @@ let server = {
     "eliom_route_base";
     "eliom_shared_content";
     "eliom_types_base";
+    "eliom_client_main";
     "eliommod";
     "eliommod_cli";
     "eliommod_cookies";
