@@ -17,6 +17,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
+open Js_of_ocaml
+
 type param = Form.form_elt
 type field = [`String of Js.js_string Js.t | `File of File.file Js.t ]
 
@@ -33,3 +35,7 @@ let inject_param_table t : (string * param) list Eliom_lib.String.Table.t  =
   Eliom_lib.String.Table.map
     (fun v -> inject_param_list v)
     t
+
+let string_of_param = function
+  | `String s -> Js.to_string s
+  | `File   _ -> failwith "is a file"
