@@ -68,10 +68,15 @@ val is_client_app : unit -> bool
     the [replace] flag is set, the new page will replace the current
     page in the browser history if the service belongs to the same
     application. The last two parameters are respectively the GET and
-    POST parameters to send to the service. *)
+    POST parameters to send to the service.
+    If [window_name] is provided and not "_self" (for example ["_blank"]),
+    will behave as [exit_to].
+*)
 val change_page :
   ?ignore_client_fun:bool ->
   ?replace:bool ->
+  ?window_name:string ->
+  ?window_features:string ->
   ?absolute:bool ->
   ?absolute_path:bool ->
   ?https:bool ->
@@ -119,8 +124,17 @@ val call_ocaml_service :
 
 (** Stop current program and load a new page.  Note that for string arguments,
     sole line feed or sole carriage return characters are substituted by the
-    string ["\r\n"]. *)
+    string ["\r\n"].
+    If [window_name] is specified (for example ["_blank"]), open in new window
+    by calling Javascript function [window.open].
+    In that case, you can add [window_features] parameter
+    (as in Javascript function [window.open]).
+    Warning: opening in other window may not work on mobile apps
+    if [window.open] is not implemented.
+*)
 val exit_to :
+  ?window_name:string ->
+  ?window_features:string ->
   ?absolute:bool ->
   ?absolute_path:bool ->
   ?https:bool ->
@@ -135,9 +149,10 @@ val exit_to :
   ?keep_get_na_params:bool ->
   'a -> 'b -> unit
 
-(** Loads an Eliom service in a window (cf. Javascript's [window.open]). *)
+(** Loads an Eliom service in a window (cf. Javascript's [window.open]).
+*)
 val window_open :
-  window_name:Js.js_string Js.t ->
+  window_name:Js.js_string Js.t  ->
   ?window_features:Js.js_string Js.t ->
   ?absolute:bool ->
   ?absolute_path:bool ->
@@ -404,6 +419,16 @@ val init : unit -> unit
 
 val set_reload_function : (unit -> unit -> Eliom_service.result Lwt.t) -> unit
 
+module History : sig
+
+  (** get the URLs of the last pages visited (within the app) in reverse temporal order. *)
+  val past : unit -> string list (*tmp*)
+
+  (** get the URLs of the pages visited before having navigated backwards in history. *)
+  val future : unit -> string list (*tmp*)
+
+end
+
 (** [push_history_dom] stores the document/body of the current page so
     that the next time when we encounter the page while navigating through the
     history, the DOM will be recovered from the cache instead of recharging or
@@ -442,14 +467,16 @@ val push_history_dom : unit -> unit
    previous and the next page are kept. *)
 val set_max_dist_history_doms : int option -> unit
 
-(** Lwt_log section for this module.
-    Default level is [Lwt_log.Info].
-    Use [Lwt_log.Section.set_level Eliom_client.log_section Lwt_log.Debug]
-    to see debug messages.
-*)
-val log_section : Lwt_log.section
+(** Lwt_log section for this module. *)
+val section : Lwt_log.section
 
 (** Is it a middle-click event? *)
 val middleClick : Dom_html.mouseEvent Js.t -> bool
 
 type client_form_handler = Dom_html.event Js.t -> bool Lwt.t
+
+(** headers to add to each of Eliom's HTTP requests. *)
+module Additional_headers : sig
+  val add : string -> string -> unit
+  val remove : string -> unit
+end
