@@ -235,17 +235,17 @@ struct
     | Eliom_lib.Left (channel, position) ->
       match position with
       | Eliom_comet_base.Newest i when i > channel.ch_index -> false
-      | Eliom_comet_base.Newest i -> true
+      | Eliom_comet_base.Newest _ -> true
       | Eliom_comet_base.After i when i > channel.ch_index -> false
-      | Eliom_comet_base.After i -> true
-      | Eliom_comet_base.Last n when (Dlist.size channel.ch_content) > 0 -> true
-      | Eliom_comet_base.Last n -> false
+      | Eliom_comet_base.After _ -> true
+      | Eliom_comet_base.Last _ when (Dlist.size channel.ch_content) > 0 -> true
+      | Eliom_comet_base.Last _ -> false
 
   let really_wait_data requests =
     let rec make_list = function
       | [] -> []
       | (Eliom_lib.Left (channel,_))::q -> (Lwt_condition.wait channel.ch_wakeup)::(make_list q)
-      | Eliom_lib.Right _ :: q ->
+      | Eliom_lib.Right _ :: _ ->
         assert false (* closed channels are considered to have data *)
     in
     Lwt.pick (make_list requests)
@@ -462,9 +462,10 @@ end = struct
              Lwt_stream.get_available_up_to n stream
            in
            if l <> [] then s.waiter <- stream_waiter stream;
-           take (n - List.length l) (List.map (fun v -> (id, v)) l @ acc) rem
+           take
+             (n - List.length l) (List.rev_map (fun v -> (id, v)) l @ acc) rem
     in
-    take n [] handler.hd_active_channels
+    List.rev (take n [] handler.hd_active_channels)
 
   let wait_channels handler =
     List.fold_left
@@ -515,8 +516,10 @@ end = struct
     handler.hd_registered_chan_id <- List.filter ((<>) chan_id) handler.hd_registered_chan_id
 
   let wait_closed_connection () =
-    let ri = Eliom_request_info.get_ri () in
-    let%lwt () = Ocsigen_extensions.Ocsigen_request_info.connection_closed ri in
+    let%lwt () =
+      Ocsigen_request.connection_closed
+        (Eliom_request_info.get_ri ())
+    in
     Lwt.fail Connection_closed
 
   (* register the service handler.hd_service *)
@@ -606,7 +609,7 @@ end = struct
         failwith "Eliom_comet: accessing channel references should not be blocking: this is an eliom bug"
 
   let get_handler_eref scope =
-    let scope_hierarchy = Eliom_common_base.scope_hierarchy_of_scope scope in
+    let scope_hierarchy = Eliom_common_base.scope_hierarchy_of_user_scope scope in
     try
       Hashtbl.find handler_ref_table scope_hierarchy
     with
@@ -846,7 +849,7 @@ end = struct
   let create_from_events ?scope ?name ?(size=1000) events =
     match scope with
       | None -> create_stateful ?name ~size events
-      | Some ((`Client_process n) as scope) ->
+      | Some ((`Client_process _) as scope) ->
          create_stateful ~scope ?name ~size events
       | Some `Site ->
          create_stateless ?name ~size (Lwt_react.E.to_stream events)
@@ -854,7 +857,7 @@ end = struct
   let create ?scope ?name ?(size=1000) stream =
     match scope with
       | None -> create_stateful ?name ~size (Lwt_react.E.of_stream stream)
-      | Some ((`Client_process n) as scope) ->
+      | Some ((`Client_process _) as scope) ->
          create_stateful ~scope ?name ~size (Lwt_react.E.of_stream stream)
       | Some `Site ->
          create_stateless ?name ~size stream
