@@ -19,8 +19,6 @@
 
 open Eliom_lib
 
-open Ocsigen_cookies
-
 include Eliom_common_base
 
 exception Eliom_Session_expired
@@ -46,16 +44,6 @@ let tenable_value ~name v = object
 end
 
 (*****************************************************************************)
-(*VVV Do not forget to change the version number
-  when the internal format change!!! *)
-let persistent_cookie_table_version = "_v5"
-(* v2 introduces session groups *)
-(* v3 introduces tab sessions *)
-(* v4 introduces group tables *)
-(* v5 removes secure scopes *)
-
-let eliom_persistent_cookie_table =
-  "eliom_persist_cookies"^persistent_cookie_table_version
 
 let datacookiename = "eliomdatasession|"
 let servicecookiename = "eliomservicesession|"
@@ -106,7 +94,7 @@ type timeout =
 
 
 
-(* The table of tables for each session. Keys are cookies *)
+(* The table of tables for each session. Keys are hashes of cookies or group names *)
 module SessionCookies =
   Hashtbl.Make(struct
     type t = string
@@ -114,6 +102,28 @@ module SessionCookies =
     let hash = Hashtbl.hash
   end)
 
+(* keys in tables are hashes of cookie values *)
+module Hashed_cookies : sig
+    type t
+    val sha256 : string -> string
+    val hash : string -> t
+    val to_string : t -> string
+end = struct
+  type t = string
+
+  let sha256 c =
+    let to_b64 = Cryptokit.Base64.encode_compact () in
+    Cryptokit.transform_string to_b64 @@
+      Cryptokit.(hash_string (Hash.sha256 ()) c)
+
+  let hash c =
+    (* To preserve compatibility, we only hash cookies that ends with an
+       'H'.  This is the case for all new cookies (see Eliommod_cookies). *)
+    if c <> "" &&  c.[String.length c - 1] = 'H' then sha256 c else c
+
+  let to_string x = x
+end
+
 (* session groups *)
 type 'a sessgrp =
   (string * cookie_level * (string, Ipaddr.t) leftright)
@@ -130,7 +140,7 @@ let make_persistent_full_group_name ~cookie_level site_dir_string = function
     Some (Marshal.to_string
             (site_dir_string, cookie_level, Left g) [])
 
-let getperssessgrp a = Marshal.from_string a 0
+let getperssessgrp a : 'a sessgrp = Marshal.from_string a 0
 
 let string_of_perssessgrp = id
 
@@ -138,7 +148,8 @@ let string_of_perssessgrp = id
 
 type 'a one_service_cookie_info =
   (* service sessions: *)
-  {sc_value:string             (* current value *);
+  {sc_hvalue:Hashed_cookies.t  (* hash of current value *);
+   sc_set_value:string option  (* new value to set *);
    sc_table:'a ref             (* service session table
                                   ref towards cookie table
                                *);
@@ -159,7 +170,8 @@ type 'a one_service_cookie_info =
 
 type one_data_cookie_info =
   (* in memory data sessions: *)
-  {dc_value:string                    (* current value *);
+  {dc_hvalue:Hashed_cookies.t         (* hash of current value *);
+   dc_set_value:string option         (* new value to set *);
    dc_timeout:timeout ref             (* user timeout -
                                          ref towards cookie table
                                       *);
@@ -173,7 +185,8 @@ type one_data_cookie_info =
   }
 
 type one_persistent_cookie_info =
-  {pc_value:string                    (* current value *);
+  {pc_hvalue:Hashed_cookies.t         (* hash of current value *);
+   pc_set_value:string option         (* new value to set *);
    pc_timeout:timeout ref             (* user timeout *);
    pc_cookie_exp:cookie_exp ref       (* cookie expiration date to set *);
    pc_session_group:perssessgrp option ref (* session group *)
@@ -226,7 +239,6 @@ type 'a cookie_info1 =
    (* None = new cookie
       (not sent by the browser) *)
    *
-
    one_persistent_cookie_info session_cookie ref
    (* SCNo_data = the session has been closed
       SCData_session_expired = the cookie has not been found in the table.
@@ -240,40 +252,38 @@ type 'a cookie_info =
   'a cookie_info1 (* unsecure *) *
   'a cookie_info1 (* secure *)
 
+module Service_cookie = struct
+  (* non persistent cookies for services *)
+  type 'a t = {
+    full_state_name : full_state_name;
+    session_table : 'a;
+    expiry : float option ref;
+    timeout : timeout ref;
+    session_group : cookie_level sessgrp ref;
+    session_group_node : string Ocsigen_cache.Dlist.node
+  }
+  type 'a table = 'a t SessionCookies.t
+  (* the table contains:
+     - the table of services
+     - the expiration date (by timeout), changed at each access to the table
+       (float option) None -> no expiration
+     - the timeout for the user (float option option) None -> see global config
+       Some None -> no timeout
+     - the group to which belongs the session
+  *)
+end
 
-
-(* non persistent cookies for services *)
-type 'a servicecookiestablecontent =
-  (full_state_name *
-   'a                  (* session table *) *
-   float option ref    (* expiration date by timeout
-                          (server side) *) *
-   timeout ref         (* user timeout *) *
-   cookie_level sessgrp ref   (* session group *) *
-   string Ocsigen_cache.Dlist.node (* session group node *))
-
-type 'a servicecookiestable = 'a servicecookiestablecontent SessionCookies.t
-(* the table contains:
-   - the table of services
-   - the expiration date (by timeout), changed at each access to the table
-     (float option) None -> no expiration
-   - the timeout for the user (float option option) None -> see global config
-     Some None -> no timeout
-   - the group to which belongs the session
-*)
-
-(* non persistent cookies for in memory data *)
-type datacookiestablecontent =
-  (full_state_name *
-   float option ref        (* expiration date by timeout
-                              (server side) *) *
-   timeout ref             (* user timeout *) *
-   cookie_level sessgrp ref   (* session group *) *
-   string Ocsigen_cache.Dlist.node (* session group node *))
-
-type datacookiestable = datacookiestablecontent SessionCookies.t
-
-
+module Data_cookie = struct
+  (* non persistent cookies for in-memory data *)
+  type t = {
+    full_state_name : full_state_name;
+    expiry : float option ref;
+    timeout : timeout ref;
+    session_group : cookie_level sessgrp ref;
+    session_group_node : string Ocsigen_cache.Dlist.node
+  }
+  type table = t SessionCookies.t
+end
 
 
 (*****************************************************************************)
@@ -339,15 +349,17 @@ type node_info = {
 
 module Hier_set = String.Set
 
+type omitpersistentstorage_rule = HeaderRule of Ocsigen_header.Name.t * Re.re
+
 type server_params =
   {sp_request: Ocsigen_extensions.request;
    sp_si: sess_info;
    sp_sitedata: sitedata (* data for the whole site *);
    sp_cookie_info: tables cookie_info;
    sp_tab_cookie_info: tables cookie_info;
-   mutable sp_user_cookies: Ocsigen_cookies.cookieset;
+   mutable sp_user_cookies: Ocsigen_cookie_map.t;
    (* cookies (un)set by the user during service *)
-   mutable sp_user_tab_cookies: Ocsigen_cookies.cookieset;
+   mutable sp_user_tab_cookies: Ocsigen_cookie_map.t;
    mutable sp_client_appl_name: string option; (* The application name,
                                                   as sent by the browser *)
    sp_suffix: Url.path option (* suffix *);
@@ -364,18 +376,18 @@ and page_table_content = [
     `Ptc of
       (page_table ref * page_table_key, na_key_serv) leftright
         Ocsigen_cache.Dlist.node option *
-      (server_params, Ocsigen_http_frame.result) service list ]
+      (server_params, Ocsigen_response.t) service list ]
 
 and naservice_table_content =
-    (int (* generation (= number of reloads of sites
-            after which that service has been created) *) *
-       int ref option (* max_use *) *
-       (float * float ref) option (* timeout and expiration date *) *
-       (server_params -> Ocsigen_http_frame.result Lwt.t) *
-       (page_table ref * page_table_key, na_key_serv) leftright
-       Ocsigen_cache.Dlist.node option
-       (* for limitation of number of dynamic coservices *)
-    )
+  (int (* generation (= number of reloads of sites
+          after which that service has been created) *) *
+   int ref option (* max_use *) *
+   (float * float ref) option (* timeout and expiration date *) *
+   (server_params -> Ocsigen_response.t Lwt.t) *
+   (page_table ref * page_table_key, na_key_serv) leftright
+     Ocsigen_cache.Dlist.node option
+     (* for limitation of number of dynamic coservices *)
+  )
 
 and naservice_table =
   | AVide
@@ -453,18 +465,17 @@ and sitedata =
       for scopes session and client process.
       The scope is registered in the full session name. *)
    global_services: tables; (* global service table *)
-   session_services: tables servicecookiestable;
+   session_services: tables Service_cookie.table;
    (* cookie table for services (tab and browser sessions) *)
-   session_data: datacookiestable; (* cookie table for in memory session data
+   session_data: Data_cookie.table; (* cookie table for in memory session data
                                       (tab and browser sessions)
                                       contains the information about the cookie
-                                      (expiration, group ...).
-                                   *)
+                                      (expiration, group ...). *)
    group_of_groups: [ `Session_group ] sessgrp Ocsigen_cache.Dlist.t;
    (* Limitation of the number of groups per site *)
    mutable remove_session_data: string -> unit;
    mutable not_bound_in_data_tables: string -> bool;
-   mutable exn_handler: exn -> Ocsigen_http_frame.result Lwt.t;
+   mutable exn_handler: exn -> Ocsigen_response.t Lwt.t;
    mutable unregistered_services: Url.path list;
    mutable unregistered_na_services: na_key_serv list;
    mutable max_volatile_data_sessions_per_group : int * bool;
@@ -484,6 +495,9 @@ and sitedata =
    mutable application_script : bool (* defer *) * bool; (* async *)
    mutable cache_global_data : (string list * int) option;
    mutable html_content_type : string option;
+   mutable ignored_get_params : (string * Re.re) list;
+   mutable ignored_post_params : (string * Re.re) list;
+   mutable omitpersistentstorage : omitpersistentstorage_rule list option;
   }
 
 and dlist_ip_table = (page_table ref * page_table_key, na_key_serv)
@@ -502,25 +516,20 @@ let find_dlist_ip_table :
 
 (*****************************************************************************)
 
-let make_full_cookie_name cookieprefix (cookie_scope, secure, site_dir_string) =
-  let scope_hier = scope_hierarchy_of_scope cookie_scope in
+let make_full_cookie_name cookieprefix {user_scope; secure; site_dir_str} =
+  let scope_hier = scope_hierarchy_of_user_scope user_scope in
   let secure = if secure then "S|" else "|" in
   let hier1, hiername = match scope_hier with
     | User_hier hiername -> "||", hiername
     | Default_ref_hier -> "|ref|", ""
     | Default_comet_hier -> "|comet|", ""
   in
-  let s = String.concat ""
-    [cookieprefix; secure; site_dir_string; hier1; hiername]
-  in
-  s
+  String.concat "" [cookieprefix; secure; site_dir_str; hier1; hiername]
 
 let make_full_state_name2
-    site_dir_string secure ~(scope:[< user_scope ]) : full_state_name =
+    site_dir_str secure ~(scope:[< user_scope ]) : full_state_name =
   (* The information in the cookie name, without the kind of session *)
-  ((scope :> user_scope),
-   secure,
-   site_dir_string)
+  {user_scope = (scope :> user_scope); secure; site_dir_str}
 
 let make_full_state_name ~sp ~secure ~(scope:[< user_scope ]) =
   make_full_state_name2 sp.sp_sitedata.site_dir_string secure scope
@@ -530,18 +539,26 @@ let get_cookie_info sp = function
   | `Client_process -> sp.sp_tab_cookie_info
 
 
+type info = {
+  request : Ocsigen_extensions.request;
+  session_info : sess_info;
+  all_cookie_info : tables cookie_info;
+  tab_cookie_info : tables cookie_info;
+  user_tab_cookies : Ocsigen_cookie_map.t
+}
 
 (*****************************************************************************)
 (** Create server parameters record *)
 let make_server_params
     sitedata
-    (ri, si, all_cookie_info, all_tab_cookie_info, user_tab_cookies)
+    ({request = ri; session_info = si} as info)
     suffix
     full_state_name =
   let appl_name =
     try
       Some
-        (CookiesTable.find appl_name_cookie_name si.si_tab_cookies)
+        (Ocsigen_cookie_map.Map_inner.find
+           appl_name_cookie_name si.si_tab_cookies)
     (* It is an XHR from the client application, or an internal form *)
     with Not_found -> None
   in
@@ -550,20 +567,20 @@ let make_server_params
     | Some cpi -> cpi
     | None ->
       let request_info = ri.Ocsigen_extensions.request_info in
-      { cpi_ssl = Ocsigen_extensions.Ocsigen_request_info.ssl request_info;
+      { cpi_ssl = Ocsigen_request.ssl request_info;
         cpi_hostname = Ocsigen_extensions.get_hostname ri;
         cpi_server_port = Ocsigen_extensions.get_port ri;
         cpi_original_full_path =
-          Ocsigen_extensions.Ocsigen_request_info.original_full_path request_info;
+          Ocsigen_request.original_full_path request_info;
       }
   in
   { sp_request = ri;
     sp_si = si;
     sp_sitedata = sitedata;
-    sp_cookie_info = all_cookie_info;
-    sp_tab_cookie_info = all_tab_cookie_info;
-    sp_user_cookies = Ocsigen_cookies.empty_cookieset;
-    sp_user_tab_cookies = user_tab_cookies;
+    sp_cookie_info = info.all_cookie_info;
+    sp_tab_cookie_info = info.tab_cookie_info;
+    sp_user_cookies = Ocsigen_cookie_map.empty;
+    sp_user_tab_cookies = info.user_tab_cookies;
     sp_client_appl_name = appl_name;
     sp_suffix = suffix;
     sp_full_state_name = full_state_name;
@@ -577,7 +594,9 @@ let get_sp_option () = Lwt.get sp_key
 let get_sp () =
   match Lwt.get sp_key with
   | Some sp -> sp
-  | None -> failwith "This function cannot be called here because it needs information about the request or the site."
+  | None ->
+      let msg = "This function cannot be called here because it needs information about the request or the site." in
+      failwith @@ String.concat "\n" @@ msg :: Eliom_common_base.backtrace_lwt 2
 
 let sp_of_option sp =
   match sp with
@@ -588,15 +607,15 @@ let sp_of_option sp =
 (* Scope registration                                                        *)
 (*****************************************************************************)
 
-let global_scope : global_scope = `Global
-let site_scope : site_scope = `Site
+let global_scope : [> global_scope ] = `Global
+let site_scope : [> site_scope ] = `Site
 
-let default_group_scope : session_group_scope = `Session_group Default_ref_hier
-let default_session_scope : session_scope = `Session Default_ref_hier
-let default_process_scope : client_process_scope = `Client_process Default_ref_hier
+let default_group_scope : [> session_group_scope ] = `Session_group Default_ref_hier
+let default_session_scope : [> session_scope ] = `Session Default_ref_hier
+let default_process_scope : [> client_process_scope ] = `Client_process Default_ref_hier
 
-let comet_client_process_scope : client_process_scope = `Client_process Default_comet_hier
-let request_scope : request_scope = `Request
+let comet_client_process_scope : [> client_process_scope ] = `Client_process Default_comet_hier
+let request_scope : [> request_scope ] = `Request
 
 let registered_scope_hierarchies = ref Hier_set.empty
 
@@ -794,7 +813,7 @@ let empty_tables max forsession =
       then
         let dlist = Ocsigen_cache.Dlist.create max in
         Ocsigen_cache.Dlist.set_finaliser_before (dlist_finaliser t2) dlist;
-        fun ?sp v -> add_dlist_ dlist v
+        fun ?sp:_ v -> add_dlist_ dlist v
       else
         fun ?sp v ->
           let ip, max, sitedata =
@@ -806,8 +825,8 @@ let empty_tables max forsession =
                          failwith "global tables created outside initialisation"
                      | Some get -> get ())
               | Some sp ->
-                  ((Lazy.force
-                      (Ocsigen_extensions.Ocsigen_request_info.remote_ip_parsed sp.sp_request.Ocsigen_extensions.request_info)),
+                  ((Ocsigen_request.remote_ip_parsed
+                      sp.sp_request.Ocsigen_extensions.request_info),
                    (fst sp.sp_sitedata.max_anonymous_services_per_subnet),
                    sp.sp_sitedata
                   )
@@ -846,9 +865,9 @@ sessionkind|S?|sitedirstring|"ref" ou "comet" ou ""|hiername
 *)
 
 let full_state_name_of_cookie_name cookie_level cookiename =
-  let pref, cookiename = Ocsigen_lib.String.sep '|' cookiename in
+  let _pref, cookiename = Ocsigen_lib.String.sep '|' cookiename in
   let secure, cookiename = Ocsigen_lib.String.sep '|' cookiename in
-  let sitedirstring, cookiename = Ocsigen_lib.String.sep '|' cookiename in
+  let site_dir_str, cookiename = Ocsigen_lib.String.sep '|' cookiename in
   let hier1, hiername = Ocsigen_lib.String.sep '|' cookiename in
   let secure = secure = "S" in
   let sc_hier = match hier1 with
@@ -857,21 +876,23 @@ let full_state_name_of_cookie_name cookie_level cookiename =
     | "comet" -> Eliom_common_base.Default_comet_hier
     | _ -> raise Not_found
   in
-  match cookie_level with
-    | `Session -> (`Session sc_hier, secure, sitedirstring)
-    | `Client_process -> (`Client_process sc_hier, secure, sitedirstring)
+  let user_scope =
+    match cookie_level with
+    | `Session -> `Session sc_hier
+    | `Client_process -> `Client_process sc_hier
+  in
+  {user_scope; secure; site_dir_str}
 
 let getcookies secure cookie_level cookienamepref cookies =
   let length = String.length cookienamepref in
   let last = length - 1 in
-  CookiesTable.fold
+  Ocsigen_cookie_map.Map_inner.fold
     (fun name value beg ->
       if String.first_diff cookienamepref name 0 last = length
       then
         try
-          let (_, sec, _) as expcn =
-            full_state_name_of_cookie_name cookie_level name in
-          if sec = secure
+          let expcn = full_state_name_of_cookie_name cookie_level name in
+          if expcn.secure = secure
           then Full_state_name_table.add expcn value beg
           else beg
         with Not_found -> beg
@@ -894,22 +915,42 @@ type cpi = client_process_info =  {
   cpi_original_full_path : string list;
 } [@@deriving json]
 
-let get_session_info req previous_extension_err =
+let matches_regexp name (_, re) =
+  try
+    let _ = Re.exec re name in
+    true
+  with Not_found -> false
+
+let matches_regexps regexps (name, _) =
+  List.exists (matches_regexp name) regexps
+
+let get_session_info ~sitedata ~req previous_extension_err =
   let req_whole = req
   and ri = req.Ocsigen_extensions.request_info
   and ci = req.Ocsigen_extensions.request_config in
-  (* *)
 
-  let rc = Ocsigen_extensions.Ocsigen_request_info.request_cache ri in
+  let rc = Ocsigen_request.request_cache ri in
   let no_post_param, p =
-    match Ocsigen_extensions.Ocsigen_request_info.post_params ri with
-      | None -> true, Lwt.return_nil
-      | Some f -> false, f (ci.Ocsigen_extensions.uploaddir, ci.Ocsigen_extensions.maxuploadfilesize)
+    match
+      Ocsigen_request.post_params ri
+        ci.Ocsigen_extensions.uploaddir
+        ci.Ocsigen_extensions.maxuploadfilesize
+    with
+    | None ->
+      true, Lwt.return []
+    | Some v ->
+      false, v
   in
   let no_file_param, file_params =
-    match Ocsigen_extensions.Ocsigen_request_info.files ri with
-      | None -> true, Lwt.return_nil
-      | Some f -> false, f (ci.Ocsigen_extensions.uploaddir, ci.Ocsigen_extensions.maxuploadfilesize)
+    match
+      Ocsigen_request.files ri
+        ci.Ocsigen_extensions.uploaddir
+        ci.Ocsigen_extensions.maxuploadfilesize
+    with
+    | None ->
+      true, Lwt.return []
+    | Some v ->
+      false, v
   in
 
   let%lwt post_params = p in
@@ -933,52 +974,61 @@ let get_session_info req previous_extension_err =
             List.assoc_remove tab_cookies_param_name post_params
           in
           let tc = [%of_json: (string * string) list] tc in
-          (List.fold_left (fun t (k,v) -> CookiesTable.add k v t) CookiesTable.empty tc, pp)
+          (List.fold_left
+             (fun t (k,v) -> Ocsigen_cookie_map.Map_inner.add k v t)
+             Ocsigen_cookie_map.Map_inner.empty tc, pp)
           (*Marshal.from_string (Ocsigen_lib.decode tc) 0, pp*)
         with Not_found ->
-          try (* looking for tab cookies in headers *)
-            let tc = Ocsigen_headers.find tab_cookies_header_name
-              (Ocsigen_extensions.Ocsigen_request_info.http_frame ri)
-            in
-            let tc = [%of_json: (string * string) list] tc in
-            (List.fold_left (fun t (k,v) -> CookiesTable.add k v t) CookiesTable.empty tc,
-             post_params)
-          with Not_found -> CookiesTable.empty, post_params
+          (match
+             Ocsigen_request.header ri
+               (Ocsigen_header.Name.of_string tab_cookies_header_name)
+           with
+           | Some tc ->
+             let tc = [%of_json: (string * string) list] tc in
+             List.fold_left
+               (fun t (k,v) -> Ocsigen_cookie_map.Map_inner.add k v t)
+               Ocsigen_cookie_map.Map_inner.empty
+               tc,
+             post_params
+           | None ->
+             Ocsigen_cookie_map.Map_inner.empty, post_params)
       in
       (None, tab_cookies, post_params)
       end
   in
 
   let cpi =
-    try (* looking for client process info in headers *)
-      let cpi =
-        Ocsigen_headers.find
-          tab_cpi_header_name
-          (Ocsigen_extensions.Ocsigen_request_info.http_frame ri) in
+    match
+      Ocsigen_request.header ri
+        (Ocsigen_header.Name.of_string tab_cpi_header_name)
+    with
+    | Some cpi ->
       Some ([%of_json: cpi] cpi)
-    with Not_found -> None
+    | None ->
+      None
   in
 
-  let epd =
-    lazy (try (* looking in headers *)
-            let epd = Ocsigen_headers.find
-              expecting_process_page_name
-              (Ocsigen_extensions.Ocsigen_request_info.http_frame ri)
-            in
-            [%of_json: bool] epd
-      with Not_found -> false)
-  in
+  let epd = lazy (
+    match
+      Ocsigen_request.header ri
+        (Ocsigen_header.Name.of_string expecting_process_page_name)
+    with
+    | Some epd ->
+      [%of_json: bool] epd
+    | None ->
+      false
+  ) in
 
   let post_params, get_params, to_be_considered_as_get =
+    let g = Ocsigen_request.get_params_flat ri in
     try
-      ([],
-       Lazy.force (Ocsigen_extensions.Ocsigen_request_info.get_params ri)
-       @snd (List.assoc_remove
-               to_be_considered_as_get_param_name post_params),
-       true)
+      [],
+      g @ snd (List.assoc_remove
+                 to_be_considered_as_get_param_name post_params),
+      true
     (* It was a POST request to be considered as GET *)
     with Not_found ->
-      (post_params, Lazy.force (Ocsigen_extensions.Ocsigen_request_info.get_params ri), false)
+      post_params, g, false
   in
 
 
@@ -998,36 +1048,47 @@ let get_session_info req previous_extension_err =
   let get_params, post_params, file_params,
       (all_get_params, all_post_params, all_file_params,
        nl_get_params, nl_post_params, nl_file_params,
-     all_get_but_nl (*204FORMS*, internal_form *)) =
+       all_get_but_nl (*204FORMS*, internal_form *),
+       ignored_get, ignored_post) =
     try
       (get_params,
        post_params,
        file_params0,
        Polytables.get
-         ~table:(Ocsigen_extensions.Ocsigen_request_info.request_cache ri)
+         ~table:(Ocsigen_request.request_cache ri)
          ~key:eliom_params_after_action)
     with Not_found ->
       let nl_get_params, get_params = split_nl_prefix_param get_params0 in
       let nl_post_params, post_params = split_nl_prefix_param post_params0 in
       let nl_file_params, file_params = split_nl_prefix_param file_params0 in
+      let ignored_get, get_params =
+        List.partition (matches_regexps sitedata.ignored_get_params) get_params
+      in
+      let ignored_post, post_params =
+        List.partition (matches_regexps sitedata.ignored_post_params) post_params
+      in
       let all_get_but_nl = get_params in
       get_params, post_params, file_params,
       (get_params0,
        (if no_post_param then None else Some post_params0),
        (if no_file_param then None else Some file_params0),
        nl_get_params, nl_post_params, nl_file_params,
-       all_get_but_nl (*204FORMS*, internal_form *))
+       all_get_but_nl (*204FORMS*, internal_form *),
+      ignored_get, ignored_post)
   in
 
   let browser_cookies =
-    try (* Cookie substitutes for iOS WKWebView *)
-      let tc = Ocsigen_headers.find cookie_substitutes_header_name
-          (Ocsigen_extensions.Ocsigen_request_info.http_frame ri) in
-      let tc = [%of_json: (string * string) list] tc in
-        List.fold_left (fun t (k,v) -> CookiesTable.add k v t)
-          CookiesTable.empty tc
-    with Not_found ->
-      Lazy.force (Ocsigen_extensions.Ocsigen_request_info.cookies ri) in
+    match
+      Ocsigen_request.header ri
+        (Ocsigen_header.Name.of_string cookie_substitutes_header_name)
+    with
+    | Some tc ->
+      List.fold_left (fun t (k,v) -> Ocsigen_cookie_map.Map_inner.add k v t)
+        Ocsigen_cookie_map.Map_inner.empty
+        ([%of_json: (string * string) list] tc)
+    | None ->
+      Ocsigen_request.cookies ri
+  in
 
   let data_cookies = getcookies false `Session datacookiename browser_cookies in
   let service_cookies = getcookies false `Session servicecookiename browser_cookies in
@@ -1174,41 +1235,27 @@ let get_session_info req previous_extension_err =
     (sservice_cookies, sdata_cookies, spersistent_cookies)
   in
 
-  let get_params_string, url_string =
-(*204FORMS* old implementation of forms with 204 and change_page_event
-    if internal_form
-    then
-      let gps = Url.make_encoded_parameters all_get_params in
-      let uri = ri.Ocsigen_extensions.ri_full_path_string in
-      ((if gps = "" then None else Some gps),
-       String.may_append uri ~sep:"?" gps)
-    else *)
-    (Ocsigen_extensions.Ocsigen_request_info.get_params_string ri,
-     Ocsigen_extensions.Ocsigen_request_info.url_string ri)
-  in
-
-  let ri', sess =
+  let ri, sess =
 (*VVV 2011/02/15 TODO: I think we'd better not change ri here.
   Keep ri for original values and use si for Eliom's values?
 *)
-    Ocsigen_extensions.Ocsigen_request_info.update ri
-      ~url_string
-      ~get_params_string
-      ~meth:(if (Ocsigen_extensions.Ocsigen_request_info.meth ri) = Ocsigen_http_frame.Http_header.HEAD
-                || to_be_considered_as_get
-             then Ocsigen_http_frame.Http_header.GET
-             else Ocsigen_extensions.Ocsigen_request_info.meth ri)
-       (* Here we modify ri, instead of putting service parameters in si.
-          Thus it works better after actions:
-          the request can be taken by other extensions, with new parameters.
-          Initial parameters are kept in si.
-       *)
-      ~get_params:(lazy get_params)
-      ~post_params:(if no_post_param then None
-                    else Some (fun _ -> Lwt.return post_params))
-      ~files:(if no_file_param then None
-              else Some (fun _ -> Lwt.return file_params))
-      (),
+    Ocsigen_request.update ri
+      ?meth:
+        (if Ocsigen_request.meth ri = `HEAD || to_be_considered_as_get then
+           Some `GET
+         else
+           None)
+      (* Here we modify ri, instead of putting service parameters in
+         si.  Thus it works better after actions: the request can be
+         taken by other extensions, with new parameters.  Initial
+         parameters are kept in si.  *)
+      ~get_params_flat:get_params
+      ?post_data:(
+        if no_post_param then
+          None
+        else
+          Some (Some (post_params, file_params))
+      ),
     {si_service_session_cookies= service_cookies;
      si_data_session_cookies= data_cookies;
      si_persistent_session_cookies= persistent_cookies;
@@ -1235,65 +1282,122 @@ let get_session_info req previous_extension_err =
      si_persistent_nl_get_params= persistent_nl_get_params;
      si_all_get_but_nl= all_get_but_nl;
      si_all_get_but_na_nl= lazy (remove_na_prefix_params all_get_but_nl);
+     si_ignored_get_params= ignored_get;
+     si_ignored_post_params= ignored_post;
      si_client_process_info= cpi;
      si_expect_process_data= epd;
 (*204FORMS*     si_internal_form= internal_form; *)
     }
   in
   Lwt.return
-    ({ req_whole with Ocsigen_extensions.request_info = ri' }, sess,
+    ({ req_whole with Ocsigen_extensions.request_info = ri }, sess,
      previous_tab_cookies_info)
 
-type info =
-    (Ocsigen_extensions.request *
-       sess_info *
-       tables cookie_info (* current browser cookie info *) *
-       tables cookie_info (* current tab cookie info *) *
-       Ocsigen_cookies.cookieset (* current user tab cookies *))
-
 exception Eliom_retry_with of info
 
 (*****************************************************************************)
-(* Each persistent table created by sites correspond to a file on the disk.
-   We save the names of the currently opened tables in this table: *)
 
-module Perstables =
-  struct
-    let empty = []
-    let add v t = v::t
-    let fold = List.fold_left
+module Omit_persistent_storage = struct
+  let check_if_omitting_storage () =
+    match get_sp_option () with
+    | Some {sp_request; sp_sitedata = {omitpersistentstorage = Some rules}} ->
+      let apply_rule = function
+        HeaderRule (header_name, regexp) ->
+          match Ocsigen_request.header sp_request.Ocsigen_extensions.request_info header_name with
+          | None -> false (* no User-Agent header *)
+          | Some header_value -> Re.execp regexp header_value
+      in
+      List.for_all apply_rule rules
+    | _ -> false
+
+  let not_if_omitting_storage f =
+    if check_if_omitting_storage () then Lwt.return_unit else f ()
+end
+
+module Ocsipersist = struct
+  include Ocsipersist
+  module Polymorphic = struct
+    include Ocsipersist.Polymorphic
+    let add table key value =
+      Omit_persistent_storage.not_if_omitting_storage
+        (fun () -> add table key value)
+    let remove table key =
+      Omit_persistent_storage.not_if_omitting_storage
+        (fun () -> remove table key)
+    let replace_if_exists table key value =
+      Omit_persistent_storage.not_if_omitting_storage
+        (fun () -> replace_if_exists table key value)
   end
+  module Store = struct
+    include Ocsipersist.Store
+    let set pv value =
+      Omit_persistent_storage.not_if_omitting_storage
+        (fun () -> set pv value)
+  end
+  module Functorial = struct
+    include Ocsipersist.Functorial
+    module Table (T : sig val name : string end)
+                 (Key : COLUMN)
+                 (Value : COLUMN) =
+      struct
+        include Table (T) (Key) (Value)
+        let add key value =
+          Omit_persistent_storage.not_if_omitting_storage
+            (fun () -> add key value)
+        let remove key =
+          Omit_persistent_storage.not_if_omitting_storage
+            (fun () -> remove key)
+        let replace_if_exists key value =
+          Omit_persistent_storage.not_if_omitting_storage
+            (fun () -> replace_if_exists key value)
+        let modify_opt key f =
+          Omit_persistent_storage.not_if_omitting_storage
+            (fun () -> modify_opt key f)
+      end
+  end
+end
 
-let perstables = ref Perstables.empty
+(* keeping track of all the persistent tables *)
+module Persistent_tables = struct
+  let polymorphic_tables = ref []
+  let functorial_tables = ref []
 
-let create_persistent_table name =
-  perstables := Perstables.add name !perstables;
-  Ocsipersist.open_table name
+  let create name =
+    polymorphic_tables := name :: !polymorphic_tables;
+    Ocsipersist.Polymorphic.open_table name
 
-let persistent_cookies_table :
-    (full_state_name * float option * timeout * perssessgrp option)
-    Ocsipersist.table Lwt.t Lazy.t =
-  lazy (create_persistent_table eliom_persistent_cookie_table)
-(* Another tables, containing the session info for each cookie *)
-(* the table contains:
-   - the expiration date (by timeout), changed at each access to the table
-     (float option) None -> no expiration
-   - the timeout for the user (float option option) None -> see global config
-     Some None -> no timeout
- *)
-(* It is lazy, because we must delay the creation of the table until
-   the initialization of eliom in case we use static linking with
-   sqlite backend ... *)
+  let add_functorial_table t = functorial_tables := t :: !functorial_tables
 
+  (** removes the entry from all opened tables *)
+  let remove_key_from_all_tables key =
+    (* doesn't remove entry from Persistent_cookies_expiry_dates; not a problem *)
+    Lwt_list.iter_s
+      (fun (module T : Ocsipersist.TABLE with type key = string) -> T.remove key)
+      !functorial_tables
+    >>= fun () ->
+    Lwt_list.iter_s (* could be replaced by iter_p *)
+      (fun t ->
+        Ocsipersist.Polymorphic.open_table t >>= fun table ->
+        Ocsipersist.Polymorphic.remove table key >>= Lwt.pause)
+      !polymorphic_tables
 
-(** removes the entry from all opened tables *)
-let remove_from_all_persistent_tables key =
-  Perstables.fold (* could be replaced by a parallel map *)
-    (fun thr t -> thr >>= fun () ->
-      Ocsipersist.open_table t >>= fun table ->
-      Ocsipersist.remove table key >>= Lwt_unix.yield)
-    return_unit
-    !perstables
+  let number_of_tables () = List.length !polymorphic_tables + List.length !functorial_tables
+
+  let number_of_table_elements () =
+    Lwt_list.map_s
+      (fun t ->
+        Ocsipersist.Polymorphic.open_table t >>= fun table ->
+        Ocsipersist.Polymorphic.length table >>= fun e ->
+        Lwt.return (t, e))
+      !polymorphic_tables
+    >>= fun polymorphic_counts ->
+    Lwt_list.map_s
+      (fun (module T : Ocsipersist.TABLE with type key = string) ->
+         T.length () >>= fun n -> Lwt.return (T.name, n)
+      ) !functorial_tables
+    >>= fun functorial_counts ->
+    Lwt.return @@ polymorphic_counts @ functorial_counts
+end
 
 
 (**** Wrapper type shared by client/server side ***)
@@ -1317,24 +1421,20 @@ let bus_unwrap_id : unwrap_id = Eliom_wrap.id_of_int bus_unwrap_id_int
 
 (* HACK: Remove the 'nl_get_appl_parameter' used to avoid confusion
    between XHR and classical request in App. *)
-let patch_request_info req =
-  if List.mem_assoc nl_get_appl_parameter
-    (Lazy.force (Ocsigen_extensions.Ocsigen_request_info.get_params req.Ocsigen_extensions.request_info))
-  then
-    { req with
+let patch_request_info ({Ocsigen_extensions.request_info} as r) =
+  let u = Ocsigen_request.uri request_info in
+  match Uri.get_query_param u nl_get_appl_parameter with
+  | Some _ ->
+    { r with
       Ocsigen_extensions.request_info =
-        let get_params =
+        let get_params_flat =
           List.remove_assoc nl_get_appl_parameter
-            (Lazy.force (Ocsigen_extensions.Ocsigen_request_info.get_params req.Ocsigen_extensions.request_info))
+            (Ocsigen_request.get_params_flat request_info)
         in
-        Ocsigen_extensions.Ocsigen_request_info.update req.Ocsigen_extensions.request_info
-          ~get_params:(lazy get_params)
-          ~get_params_string:
-            (if get_params = [] then None
-             else Some (Url.make_encoded_parameters get_params)) ()
-        }
-  else
-    req
+        Ocsigen_request.update ~get_params_flat request_info
+    }
+  | None ->
+    r
 
 
 let get_site_dir sitedata = sitedata.site_dir
