@@ -22,6 +22,8 @@
 
 open Eliom_lib
 
+module Ocsipersist : module type of Ocsipersist
+
 (** {2 Scopes} *)
 (* those types are not available to the user, a scope must be created using
    create_..._scope functions *)
@@ -47,6 +49,7 @@ type user_level = [ `Session_group | cookie_level ]
 val cookie_scope_of_user_scope : [< user_scope ] -> [> cookie_scope ]
 val cookie_level_of_user_scope : [< user_scope ] -> [> cookie_level ]
 val level_of_user_scope : [< user_scope ] -> [> user_level ]
+val scope_hierarchy_of_user_scope : [< user_scope ] -> scope_hierarchy
 
 (** Eliom is using regular (browser) cookies but can also use its own
     browser tab cookies (only if you are using a client side Eliom application).
@@ -83,13 +86,13 @@ type session_scope = [`Session of scope_hierarchy]
 type client_process_scope = [`Client_process of scope_hierarchy]
 type request_scope = [`Request]
 
-val global_scope : global_scope
-val site_scope : site_scope
-val default_group_scope : session_group_scope
-val default_session_scope : session_scope
-val default_process_scope : client_process_scope
-val comet_client_process_scope : client_process_scope
-val request_scope : request_scope
+val global_scope : [> global_scope ]
+val site_scope : [> site_scope ]
+val default_group_scope : [> session_group_scope ]
+val default_session_scope : [> session_scope ]
+val default_process_scope : [> client_process_scope ]
+val comet_client_process_scope : [> client_process_scope ]
+val request_scope : [> request_scope ]
 
 val create_scope_hierarchy : string -> scope_hierarchy
 
@@ -120,8 +123,11 @@ exception Eliom_Typing_Error of (string * exn) list
 *)
 exception Eliom_site_information_not_available of string
 
-type full_state_name =
-    user_scope * bool (* secure *) * string (* site_dir_string *)
+type full_state_name = {
+  user_scope : user_scope;
+  secure : bool;
+  site_dir_str : string
+}
 module Full_state_name_table : Map.S with type key = full_state_name
 
 (** If present and true in request data, it means that
@@ -241,8 +247,6 @@ val to_be_considered_as_get_param_name : string
 val full_xhr_redir_header : string
 val half_xhr_redir_header : string
 
-val default_group_name : string
-
 type client_process_info =  {
   cpi_ssl : bool;
   cpi_hostname : string;
@@ -272,7 +276,7 @@ type sess_info = {
      string Full_state_name_table.t *
      string Full_state_name_table.t);
 
-  si_tab_cookies: string Ocsigen_cookies.CookiesTable.t;
+  si_tab_cookies: string Ocsigen_cookie_map.Map_inner.t;
 
   si_nonatt_info : na_key_req;
   si_state_info: (att_key_req * att_key_req);
@@ -287,6 +291,9 @@ type sess_info = {
   si_all_get_but_na_nl: (string * string) list Lazy.t;
   si_all_get_but_nl: (string * string) list;
 
+  si_ignored_get_params: (string * string) list;
+  si_ignored_post_params: (string * string) list;
+
   si_client_process_info: client_process_info option;
   si_expect_process_data : bool Lazy.t;
 
@@ -309,9 +316,7 @@ type perssessgrp (* the same triple, marshaled *)
 val make_persistent_full_group_name :
   cookie_level:cookie_level -> string -> string option -> perssessgrp option
 
-val getperssessgrp : perssessgrp ->
-  (string * cookie_level *
-   (string, Ipaddr.t) leftright)
+val getperssessgrp : perssessgrp -> 'a sessgrp
 
 val string_of_perssessgrp : perssessgrp -> string
 
@@ -325,9 +330,21 @@ type cookie_exp =
 
 val default_client_cookie_exp : unit -> cookie_exp
 
+module Hashed_cookies : sig
+  type t
+
+  val sha256 : string -> string
+
+  (** NOTE: this function is the identity function for strings that do not with an 'H' *)
+  val hash : string -> t
+
+  val to_string : t -> string
+end
+
 type timeout = TGlobal | TNone | TSome of float
 type 'a one_service_cookie_info = {
-  sc_value : string;
+  sc_hvalue : Hashed_cookies.t;
+  sc_set_value : string option;
   sc_table : 'a ref;
   sc_timeout : timeout ref;
   sc_exp : float option ref;
@@ -336,7 +353,8 @@ type 'a one_service_cookie_info = {
   mutable sc_session_group_node:string Ocsigen_cache.Dlist.node;
 }
 type one_data_cookie_info = {
-  dc_value : string;
+  dc_hvalue : Hashed_cookies.t;
+  dc_set_value : string option;
   dc_timeout : timeout ref;
   dc_exp : float option ref;
   dc_cookie_exp : cookie_exp ref;
@@ -344,7 +362,8 @@ type one_data_cookie_info = {
   mutable dc_session_group_node:string Ocsigen_cache.Dlist.node;
 }
 type one_persistent_cookie_info = {
-  pc_value : string;
+  pc_hvalue : Hashed_cookies.t;
+  pc_set_value : string option;
   pc_timeout : timeout ref;
   pc_cookie_exp : cookie_exp ref;
   pc_session_group : perssessgrp option ref;
@@ -364,18 +383,29 @@ type 'a cookie_info =
     'a cookie_info1 (* unsecure *) *
       'a cookie_info1  (* secure *)
 
-type 'a servicecookiestablecontent =
-    full_state_name * 'a * float option ref * timeout ref *
-      cookie_level sessgrp ref *
-      string Ocsigen_cache.Dlist.node
-type 'a servicecookiestable =
-    'a servicecookiestablecontent SessionCookies.t
-type datacookiestablecontent =
-    full_state_name * float option ref * timeout ref *
-      cookie_level sessgrp ref *
-      string Ocsigen_cache.Dlist.node
-type datacookiestable =
-    datacookiestablecontent SessionCookies.t
+module Service_cookie : sig
+  type 'a t = {
+    full_state_name : full_state_name;
+    session_table : 'a;
+    expiry : float option ref;
+    timeout : timeout ref;
+    session_group : cookie_level sessgrp ref;
+    session_group_node : string Ocsigen_cache.Dlist.node
+  }
+  type 'a table = 'a t SessionCookies.t
+end
+
+module Data_cookie : sig
+  (* non-persistent cookies for in-memory data *)
+  type t = {
+    full_state_name : full_state_name;
+    expiry : float option ref;
+    timeout : timeout ref;
+    session_group : cookie_level sessgrp ref;
+    session_group_node : string Ocsigen_cache.Dlist.node
+  }
+  type table = t SessionCookies.t
+end
 
 type meth = [`Get | `Post | `Put | `Delete | `Other]
 
@@ -400,6 +430,8 @@ type node_info = {
 
 module Hier_set : Set.S
 
+type omitpersistentstorage_rule = HeaderRule of Ocsigen_header.Name.t * Re.re
+
 type 'a dircontent = Vide | Table of 'a direlt ref String.Table.t
 and 'a direlt = Dir of 'a dircontent ref | File of 'a ref
 
@@ -416,9 +448,9 @@ type server_params = {
   sp_sitedata : sitedata;
   sp_cookie_info : tables cookie_info;
   sp_tab_cookie_info : tables cookie_info;
-  mutable sp_user_cookies: Ocsigen_cookies.cookieset;
+  mutable sp_user_cookies: Ocsigen_cookie_map.t;
   (* cookies (un)set by the user during service *)
-  mutable sp_user_tab_cookies: Ocsigen_cookies.cookieset;
+  mutable sp_user_tab_cookies: Ocsigen_cookie_map.t;
   mutable sp_client_appl_name: string option; (* The application name,
                                                  as sent by the browser *)
   sp_suffix : Url.path option;
@@ -438,14 +470,14 @@ and page_table_content = [
     `Ptc of
       (page_table ref * page_table_key, na_key_serv) leftright
         Ocsigen_cache.Dlist.node option *
-      (server_params, Ocsigen_http_frame.result) service list ]
+      (server_params, Ocsigen_response.t) service list ]
 
 and naservice_table_content =
     (int (* generation (= number of reloads of sites
             after which that service has been created) *) *
        int ref option (* max_use *) *
        (float * float ref) option (* timeout and expiration date *) *
-       (server_params -> Ocsigen_http_frame.result Lwt.t) *
+       (server_params -> Ocsigen_response.t Lwt.t) *
        (page_table ref * page_table_key, na_key_serv) leftright
        Ocsigen_cache.Dlist.node option
        (* for limitation of number of dynamic coservices *)
@@ -524,13 +556,13 @@ and sitedata = {
   mutable registered_scope_hierarchies: Hier_set.t;
 
   global_services : tables;
-  session_services : tables servicecookiestable;
-  session_data : datacookiestable;
+  session_services : tables Service_cookie.table;
+  session_data : Data_cookie.table;
   group_of_groups: [ `Session_group ] sessgrp Ocsigen_cache.Dlist.t;
   (* Limitation of the number of groups per site *)
   mutable remove_session_data : string -> unit;
   mutable not_bound_in_data_tables : string -> bool;
-  mutable exn_handler : exn -> Ocsigen_http_frame.result Lwt.t;
+  mutable exn_handler : exn -> Ocsigen_response.t Lwt.t;
   mutable unregistered_services : Url.path list;
   mutable unregistered_na_services : na_key_serv list;
   mutable max_volatile_data_sessions_per_group : int * bool;
@@ -550,6 +582,9 @@ and sitedata = {
   mutable application_script : bool (* defer *) * bool; (* async *)
   mutable cache_global_data : (string list * int) option;
   mutable html_content_type : string option;
+  mutable ignored_get_params : (string * Re.re) list;
+  mutable ignored_post_params : (string * Re.re) list;
+  mutable omitpersistentstorage : omitpersistentstorage_rule list option;
 }
 
 type 'a lazy_site_value (** lazy site values, are lazy values with
@@ -560,10 +595,13 @@ type 'a lazy_site_value (** lazy site values, are lazy values with
 val force_lazy_site_value : 'a lazy_site_value -> 'a
 val lazy_site_value_from_fun : ( unit -> 'a ) -> 'a lazy_site_value
 
-
-type info =
-    (Ocsigen_extensions.request * sess_info *
-       tables cookie_info * tables cookie_info * Ocsigen_cookies.cookieset)
+type info = {
+  request : Ocsigen_extensions.request;
+  session_info : sess_info;
+  all_cookie_info : tables cookie_info;
+  tab_cookie_info : tables cookie_info;
+  user_tab_cookies : Ocsigen_cookie_map.t
+}
 
 exception Eliom_retry_with of info
 
@@ -581,9 +619,10 @@ val new_service_session_tables : sitedata -> tables
 val split_prefix_param :
   string -> (string * 'a) list -> (string * 'a) list * (string * 'a) list
 val get_session_info :
-  Ocsigen_extensions.request ->
+  sitedata:sitedata ->
+  req:Ocsigen_extensions.request ->
   int -> (Ocsigen_extensions.request * sess_info *
-            (tables cookie_info * Ocsigen_cookies.cookieset) option) Lwt.t
+          (tables cookie_info * Ocsigen_cookie_map.t) option) Lwt.t
 type ('a, 'b) foundornot = Found of 'a | Notfound of 'b
 
 val make_full_cookie_name : string -> full_state_name -> string
@@ -592,20 +631,14 @@ val make_full_state_name :
 val make_full_state_name2 :
   string -> bool -> scope:[< user_scope ] -> full_state_name
 
+module Persistent_tables : sig
+  val create : string -> 'a Ocsipersist.table Lwt.t
+  val add_functorial_table : (module Ocsipersist.TABLE with type key = string) -> unit
+  val remove_key_from_all_tables : string -> unit Lwt.t
+  val number_of_tables : unit -> int
+  val number_of_table_elements : unit -> (string * int) list Lwt.t
+end
 
-
-module Perstables :
-  sig
-    val empty : 'a list
-    val add : 'a -> 'a list -> 'a list
-    val fold : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
-  end
-val perstables : string list ref
-val create_persistent_table : string -> 'a Ocsipersist.table Lwt.t
-val persistent_cookies_table :
-  (full_state_name * float option * timeout * perssessgrp option)
-  Ocsipersist.table Lwt.t Lazy.t
-val remove_from_all_persistent_tables : string -> unit Lwt.t
 val absolute_change_sitedata : sitedata -> unit
 val get_current_sitedata : unit -> sitedata
 val end_current_sitedata : unit -> unit
@@ -631,7 +664,9 @@ val eliom_params_after_action :
      (string * string) list String.Table.t *
      (string * string) list String.Table.t *
    (string * file_info) list String.Table.t *
-     (string * string) list (*204FORMS* * bool *))
+     (string * string) list (*204FORMS* * bool *) *
+     (string * string) list *
+     (string * string) list)
   Polytables.key
 
 val att_key_serv_of_req : att_key_req -> att_key_serv
@@ -658,8 +693,8 @@ val get_cookie_info : server_params -> [< cookie_level ] -> tables cookie_info
 
 val tab_cookie_action_info_key :
   (tables cookie_info *
-   Ocsigen_cookies.cookieset *
-   string Ocsigen_cookies.CookiesTable.t) Polytables.key
+   Ocsigen_cookie_map.t *
+   string Ocsigen_cookie_map.Map_inner.t) Polytables.key
 
 val sp_key : server_params Lwt.key
 val get_sp_option : unit -> server_params option
