@@ -19,8 +19,6 @@
 
 open Eliom_lib
 
-open Ocsigen_cookies
-
 exception Eliom_site_information_not_available of string
 
 (******************************************************************)
@@ -55,34 +53,34 @@ type cookie_scope = [ `Session of scope_hierarchy
                     | `Client_process of scope_hierarchy ]
 
 let level_of_user_scope : [< user_scope ] -> [> user_level ] = function
-  | `Session n -> `Session
-  | `Session_group n -> `Session_group
-  | `Client_process n -> `Client_process
+  | `Session _ -> `Session
+  | `Session_group _ -> `Session_group
+  | `Client_process _ -> `Client_process
 
 let cookie_level_of_user_scope : [< user_scope ] -> [> cookie_level ] = function
-  | `Session n
-  | `Session_group n -> `Session
-  | `Client_process n -> `Client_process
+  | `Session _
+  | `Session_group _ -> `Session
+  | `Client_process _ -> `Client_process
 
 let cookie_scope_of_user_scope : [< user_scope ] -> [> cookie_scope ] = function
   | `Session n
   | `Session_group n -> `Session n
   | `Client_process n -> `Client_process n
 
-let scope_hierarchy_of_scope : [< user_scope ] -> scope_hierarchy =
+let scope_hierarchy_of_user_scope : [< user_scope ] -> scope_hierarchy =
   function
     | `Session n
     | `Session_group n
     | `Client_process n -> n
 
-type full_state_name = (* WAS: fullsessionname *)
-    (* The key in the table of states.
-       For cookies scopes,
-       it is also the information in the cookie name,
-       without the kind of session,
-       and with the scope level (that is not in the cookie name).
-    *)
-    user_scope * bool (* secure*) * string (* site_dir_string *)
+(* The key in the table of states. For cookies scopes, it is also the
+   information in the cookie name, without the kind of session, and with the
+   scope level (that is not in the cookie name). *)
+type full_state_name = {
+  user_scope : user_scope;
+  secure : bool;
+  site_dir_str : string
+}
 
 module Full_state_name_table = Map.Make(struct
   type t = full_state_name
@@ -176,8 +174,6 @@ let full_xhr_redir_header = "X-Eliom-Location-Full"
 let half_xhr_redir_header = "X-Eliom-Location-Half"
 let response_url_header = "X-Eliom-Url"
 
-let default_group_name = "__eliom$%@default_group"
-
 let set_tab_cookies_header_name = "X-Eliom-Set-Process-Cookies"
 let tab_cookies_header_name = "X-Eliom-Process-Cookies"
 
@@ -234,7 +230,7 @@ type sess_info =
         string Full_state_name_table.t *
         string Full_state_name_table.t);
 
-     si_tab_cookies: string CookiesTable.t;
+     si_tab_cookies: string Ocsigen_cookie_map.Map_inner.t;
 
      si_nonatt_info: na_key_req;
      si_state_info: (att_key_req * att_key_req);
@@ -250,6 +246,9 @@ type sess_info =
      si_all_get_but_na_nl: (string * string) list Lazy.t;
      si_all_get_but_nl: (string * string) list;
 
+     si_ignored_get_params: (string * string) list;
+     si_ignored_post_params: (string * string) list;
+
      si_client_process_info : client_process_info option;
      si_expect_process_data : bool Lazy.t;
 
@@ -314,7 +313,7 @@ let prefixlengthminusone = prefixlength - 1
 let split_nl_prefix_param l =
     let rec aux other map = function
       | [] -> (map, other)
-      | ((n, v) as a)::l ->
+      | ((n, _) as a)::l ->
           if String.first_diff
             n nl_param_prefix 0 prefixlengthminusone = prefixlength
           then
@@ -349,7 +348,7 @@ let remove_prefixed_param pref l =
   let len = String.length pref in
   let rec aux = function
     | [] -> []
-    | ((n,v) as a)::l ->
+    | ((n,_) as a)::l ->
         try
           if (String.sub n 0 len) = pref then
             aux l
@@ -414,3 +413,27 @@ type 'a to_and_of = {
   of_string : string -> 'a;
   to_string : 'a -> string
 }
+
+(* gets backtrace up until the first slot in the backtrace which mentions
+     Lwt, which is usually where the backtrace is no longer informative *)
+let backtrace_lwt =
+  let lwt_slot_re = Re.Str.regexp "Called from Lwt." in
+  fun skip ->
+    let stack = Printexc.get_callstack 16 in
+    let stack_length = Printexc.raw_backtrace_length stack in
+    let rec loop acc i =
+      if i >= stack_length
+      then acc
+      else
+        match
+          let open Printexc in
+          Slot.format i @@ convert_raw_backtrace_slot
+          @@ get_raw_backtrace_slot stack i
+        with
+        | Some s ->
+            if Re.Str.string_match lwt_slot_re s 0
+            then acc
+            else loop (s :: acc) (i + 1)
+        | None -> loop acc (i + 1)
+    in
+    List.rev @@ loop [] skip
