@@ -52,7 +52,7 @@ module Xml : sig
         ( [ `A | `Form_get | `Form_post] *
           ((bool * string list) option) *
           string option *
-          Ocsigen_lib.poly (* (unit -> bool) client_value *)
+          Ocsigen_lib_base.poly (* (unit -> bool) client_value *)
         ) option Eliom_lazy.request
 
   type internal_event_handler = Raw of string | Caml of caml_event_handler
@@ -406,7 +406,7 @@ module Html : sig
     val attrib : 'a t -> 'a -> [> | `User_data ] attrib
 
     (** [get_dom element custom_data] gets the [custom_data] from a JavaScript [element]
-        ({% <<a_api project="js_of_ocaml"|class type Dom_html.element>> %}).
+        ({% <<a_api project="js_of_ocaml"|class type Js_of_ocaml.Dom_html.element>> %}).
         @return The value encoded in the respective custom data attribute of [element], or the default value, if any.
         @raise Not_found if the element does not contain the respective custom
           data attribute and the [custom_data] was created without [default].
@@ -415,7 +415,7 @@ module Html : sig
 
     (** [set_dom element custom_data value] sets the custom data attribute for
         [custom_data] of an JavaScript [element]
-        ({% <<a_api project="js_of_ocaml"|class type Dom_html.element>> %})
+        ({% <<a_api project="js_of_ocaml"|class type Js_of_ocaml.Dom_html.element>> %})
         to [value]. *)
     val set_dom : Dom_html.element Js.t -> 'a t -> 'a -> unit
   end
