@@ -35,6 +35,6 @@ let register_eliom_extension f =
 let get_eliom_extension () = !module_action
 
 
-let run_eliom_extension (fext : eliom_extension_sig) now info sitedata  =
+let run_eliom_extension (fext : eliom_extension_sig) _now info sitedata  =
   let sp = Eliom_common.make_server_params sitedata info None None in
   Lwt.with_value Eliom_common.sp_key (Some sp) fext
