@@ -152,8 +152,8 @@ end
 (* We do not use the deriving (un)marshaling even if typ is available
    because direct jsn (un)marshaling is very fast client side
 *)
-let to_json ?typ s = Js.to_string (Json.output s)
-let of_json ?typ v = Json.unsafe_input (Js.string v)
+let to_json ?typ:_ s = Js.to_string (Json.output s)
+let of_json ?typ:_ v = Json.unsafe_input (Js.string v)
 
 (* to marshal data and put it in a form *)
 let encode_form_value x = to_json x
@@ -170,5 +170,5 @@ let unmarshal_js var =
 
 type file_info = File.file Js.t
 
-let make_cryptographic_safe_string ?len () =
+let make_cryptographic_safe_string ?len:_ () =
   failwith "make_cryptographic_safe_string not implemented client-side"
