@@ -20,7 +20,7 @@
 include Eliom_parameter_sigs.S
   with type raw_post_data =
     ((string * string) * (string * string) list) option *
-    string Ocsigen_stream.t option
+    Cohttp_lwt.Body.t
 
 (** [user_type ~of_string ~to_string s] construct a parameter, labeled
     [s], such that the server will have to use [of_string] and
@@ -59,11 +59,12 @@ val type_checker :
   ('a, 'b, 'c) params_type
 
 (** [regexp r d s] tells that the service takes a string that matches
-   the regular expression [r] as parameter, labeled [s], and that will
-   be rewritten in d.  The syntax of regexp is PCRE's one.  For
-   example: [regexp (Pcre.regexp "\\[(.* )\\]") "($1)" "myparam"] will
-   match the parameter [myparam=[hello]] and send the string
-   ["(hello)"] to the service handler.  *)
+    the regular expression [r] as parameter, labeled [s], and that will
+    be rewritten in d.  The syntax of regexp is PCRE's one (uses then
+    [Pcre] bindings).  For example: [regexp
+    (Pcre.regexp "\\[(.* )\\]") "($1)" "myparam"] will match
+    the parameter [myparam=[hello]] and send the string ["(hello)"] to
+    the service handler.  *)
 val regexp :
   Pcre.regexp ->
   string ->
