@@ -19,13 +19,10 @@
 
 open Js_of_ocaml
 open Eliom_lib
-open Ocsigen_cookies
 
 (* Logs *)
 let section = Lwt_log.Section.make "eliom:process"
 let log_section = section
-let _ = Lwt_log.Section.set_level log_section Lwt_log.Info
-(* *)
 
 let history_api = Dom_html.hasPushState ()
 
@@ -51,16 +48,27 @@ let set_sitedata, is_set_sitedata,
     reset_sitedata =
   get_set_js_serverside_value Eliom_common.sitedata "__eliom_appl_sitedata"
 
+let ignored_get_params = ref []
+let ignored_post_params = ref []
+
+let set_ignored_params get post =
+  let compile =
+    List.map
+      (fun s -> Re.seq [Re.start; Re.Pcre.re s; Re.stop] |> Re.compile)
+  in
+  ignored_get_params := compile get;
+  ignored_post_params := compile post
+
 let set_info, is_set_info,
     (get_info : unit -> Eliom_common.client_process_info),
     reset_info
   =
   get_set_js_serverside_value (ref None) "__eliom_appl_process_info"
 
-let set_request_cookies, is_set_request_cookies,
-    (get_request_cookies : unit -> Eliommod_cookies.cookie
-       Ocsigen_cookies.CookiesTable.t Ocsigen_cookies.Cookies.t),
-  reset_request_cookies =
+let set_request_cookies,
+    is_set_request_cookies,
+    (get_request_cookies : unit -> Ocsigen_cookie_map.t),
+    reset_request_cookies =
   get_set_js_serverside_value (ref None) "__eliom_request_cookies"
 
 let set_request_template, is_set_request_template,
@@ -75,9 +83,9 @@ let appl_name =
           and not a JS variable,
           because we want to send it back with each request.
           For mobile apps, we set the cookie from JS variable. *)
-       (CookiesTable.find
+       (Ocsigen_cookie_map.Map_inner.find
           Eliom_common.appl_name_cookie_name
-          (Cookies.find
+          (Ocsigen_cookie_map.Map_path.find
              (get_sitedata ()).Eliom_types.site_dir
              (Eliommod_cookies.get_table
                 (Some (get_info ()).cpi_hostname))))
