@@ -24,6 +24,11 @@ open Eliom_state
 
 let (>>=) = Lwt.bind
 
+module Ocsipersist = struct
+  include Eliom_common.Ocsipersist.Store
+  include Eliom_common.Ocsipersist.Polymorphic
+end
+
 let pers_ref_store = Ocsipersist.open_store "eliom__persistent_refs"
 
 type 'a eref_kind =
@@ -154,7 +159,7 @@ module Volatile = struct
   let modify state eref f =
     set state eref (f (get state eref))
 
-  let unset state (f, _, table : _ eref) =
+  let unset state (_, _, table : _ eref) =
     match table with
       | Vol t -> Eliom_state.Ext.Low_level.remove_volatile_data
         ~state ~table:(Lazy.force t);
@@ -245,7 +250,7 @@ let set (_, _, table as eref) value =
 let modify eref f =
   get eref >>= fun x -> set eref (f x)
 
-let unset (f, _, table as eref) =
+let unset (_, _, table as eref) =
   match table with
     | Per t ->
       t >>= fun t ->
