@@ -25,7 +25,6 @@
 
 *)
 
-
 (** Eliom references come in two flavors: they may be stored persistently or
     the may be volatile.  The module [Volatile] allows creation of
     references which can be, get, set, modify, and unset volatile references
@@ -89,8 +88,7 @@ val eref :
 
     In each scope, the function [f] is called for creating the value of the
     reference the first time the reference is read (by {!Eliom_reference.get}),
-    if the value has not been set explicitly before (by {!Eliom_reference.set});
-    or if the reference was reset (by {!Eliom_reference.reset}) before.
+    if the value has not been set explicitly before (by {!Eliom_reference.set}).
   *)
 val eref_from_fun :
   scope:[< Eliom_common.all_scope ] ->
