@@ -147,6 +147,9 @@ module type APP = sig
 
 end
 
+(** Function to modify the global_app URI for some obscure reason *)
+val transform_global_app_uri : (Eliom_content.Html.uri -> Eliom_content.Html.uri) ref
+
 (** Functor for application creation.
     See {% <<a_manual chapter="clientserver-applications" | the chapter on applications >> %}
     in the Eliom manual for details. *)
@@ -179,6 +182,7 @@ module Flow5 : Eliom_registration_sigs.S_with_create
    and type return = Eliom_service.non_ocaml
    and type result = block_content kind
 
+(*
 (** Eliom service registration for services that return fragments of
     TyXML's tree. *)
 module Make_typed_xml_registration
@@ -190,6 +194,7 @@ module Make_typed_xml_registration
    and type options = unit
    and type return = Eliom_service.non_ocaml
    and type result = block_content kind
+*)
 
 (** {2 Untyped pages} *)
 
@@ -275,7 +280,7 @@ module Redirection : sig
     ?charset      : string ->
     ?code         : int ->
     ?content_type : string ->
-    ?headers      : Http_headers.t ->
+    ?headers      : Ocsigen_header.t ->
     _ page ->
     _ kind Lwt.t
 
@@ -389,6 +394,7 @@ module String : Eliom_registration_sigs.S_with_create
    and type return = Eliom_service.non_ocaml
    and type result = unknown_content kind
 
+(*
 (** Eliom service registration for services that returns "byte"
     contents with {% <<a_api project="ocsigenserver" text="Ocsigen's
     streams"| module Ocsigen_stream>>%}. The page content is a pair
@@ -406,6 +412,7 @@ module Streamlist : Eliom_registration_sigs.S_with_create
    and type options = unit
    and type return = Eliom_service.non_ocaml
    and type result = unknown_content kind
+*)
 
 (** {2 Customizing registration} *)
 
@@ -455,4 +462,8 @@ val cast_unknown_content_kind :
 
 (** [cast_http_result] should only be used to register new output
     modules *)
-val cast_http_result : Ocsigen_http_frame.result -> 'a kind
+val cast_http_result : Ocsigen_response.t -> 'a kind
+
+val extension : Ocsigen_server.Site.extension
+
+val end_init : unit -> unit
