@@ -75,12 +75,21 @@ let xml_result x =
 let string_result x = x.XmlHttpRequest.content
 
 (*TODO: use Url.Current.set *)
-let redirect_get url = Dom_html.window##.location##.href := Js.string url
+let redirect_get ?window_name ?window_features url =
+  match window_name with
+  | None ->
+    Dom_html.window##.location##.href := Js.string url
+  | Some window_name ->
+    ignore (Dom_html.window##(
+      open_ (Js.string url) (Js.string window_name)
+        (Js.Opt.map (Js.Opt.option window_features) Js.string)
+    ))
 
-let redirect_post url params =
+let redirect_post ?window_name url params =
   let f = Dom_html.createForm Dom_html.document in
   f##.action := Js.string url;
   f##._method := Js.string "post";
+  (match window_name with None -> () | Some wn -> f##.target := Js.string wn);
   List.iter
     (fun (n, v) ->
       match v with
@@ -89,7 +98,7 @@ let redirect_post url params =
           in
           i##.value := v;
           Dom.appendChild f i
-        | `File i ->
+        | `File _ ->
           Lwt_log.raise_error ~section "redirect_post not implemented for files")
     params;
   f##.style##.display := (Js.string "none");
@@ -98,11 +107,11 @@ let redirect_post url params =
   f##submit
 
 (* Forms cannot use PUT http method: do not redirect *)
-let redirect_put _url _params =
+let redirect_put ?window_name:_ _url _params =
   Lwt_log.raise_error ~section "redirect_put not implemented"
 
 (* Forms cannot use DELETE http method: do not redirect *)
-let redirect_delete _url _params =
+let redirect_delete ?window_name:_ _url _params =
   Lwt_log.raise_error ~section "redirect_delete not implemented"
 
 let nl_template =
@@ -112,6 +121,15 @@ let nl_template =
 (* Warning: it must correspond to [nl_template]. *)
 let nl_template_string = "__nl_n_eliom-template.name"
 
+module Additional_headers = struct
+  module Headers = Map.Make (String)
+  let headers = ref Headers.empty
+  let add header value =
+    headers :=
+    Headers.update (String.lowercase_ascii header) (fun _ -> Some value) !headers
+  let remove header = headers := Headers.remove header !headers
+  let to_list () = Headers.bindings !headers
+end
 
 let locked, set_locked = React.S.create false
 
@@ -171,6 +189,7 @@ let send
       else
         headers
     in
+    let headers = Additional_headers.to_list () @ headers in
     (* CCC *
        For now we assume that an eliom application is not distributed
        among different server with different hostnames:
@@ -269,7 +288,7 @@ let send
             (match r.XmlHttpRequest.headers Eliom_common.half_xhr_redir_header
              with
                | None | Some "" -> Lwt.return (r.XmlHttpRequest.url, None)
-               | Some uri ->
+               | Some _uri ->
                  redirect_post url
                    (match post_args with
                     | Some post_args -> post_args
