@@ -57,11 +57,34 @@ let set_session_info ~uri si f =
   default_ri := ri;
   Lwt.with_value ri_key ri f
 
+let matches_regexp name re =
+  try
+    let _ = Re.exec re name in
+    true
+  with Not_found -> false
+
+let matches_regexps regexps (name, _) =
+  List.exists (matches_regexp name) regexps
+
 let update_session_info
     ~path
     ~all_get_params
     ~all_post_params
     cont =
+  let ignored_get, all_get_params =
+    List.partition
+      (matches_regexps !Eliom_process.ignored_get_params)
+      all_get_params
+  in
+  let ignored_post, all_post_params =
+    match all_post_params with
+    | None -> [], None
+    | Some p ->
+       List.partition
+        (matches_regexps !Eliom_process.ignored_post_params)
+        p
+       |> fun (a, b) -> (a, Some b)
+  in
   let nl_get_params, all_get_but_nl =
     Eliom_common.split_nl_prefix_param all_get_params
   in
@@ -81,7 +104,9 @@ let update_session_info
     si_nl_post_params = Eliom_lib.String.Table.empty;
     si_all_post_params = all_post_params;
     si_all_get_but_nl = all_get_but_nl;
-    si_all_get_but_na_nl = all_get_but_na_nl
+    si_all_get_but_na_nl = all_get_but_na_nl;
+    si_ignored_get_params = ignored_get;
+    si_ignored_post_params = ignored_post
     } in
   let ri = Some {path; si} in
   default_ri := ri;
@@ -92,7 +117,7 @@ let remove_first_slash path =
   | ""::l -> l
   | l -> l
 
-let get_original_full_path_sp sp =
+let get_original_full_path_sp _sp =
   (* returns current path, not the one when application started *)
   if not (Eliom_process.history_api || !client_app_initialised) then
     match Url.Current.get () with
@@ -109,8 +134,6 @@ let get_original_full_path_string () =
 
 let get_original_full_path_string_sp = get_original_full_path_string
 
-let get_other_get_params () =
-  (get_sess_info ()).Eliom_common.si_other_get_params
 let get_nl_get_params () = (get_sess_info ()).Eliom_common.si_nl_get_params
 let get_nl_get_params_sp = get_nl_get_params
 
@@ -118,17 +141,9 @@ let get_persistent_nl_get_params () =
   Lazy.force (get_sess_info ()).Eliom_common.si_persistent_nl_get_params
 let get_persistent_nl_get_params_sp = get_persistent_nl_get_params
 
-let get_nl_post_params () =
-  (get_sess_info ()).Eliom_common.si_nl_post_params
-
 let get_si () = get_sess_info ()
 
 let get_site_dir () = (Eliom_process.get_sitedata ()).site_dir
-let get_site_dir_sp () = (Eliom_process.get_sitedata ()).site_dir
-let get_site_dir_string () =
-  (Eliom_process.get_sitedata ()).site_dir_string
-
-let get_sp_appl_name = Eliom_process.get_application_name
 
 let ssl_ = match Url.Current.get () with
   | Some (Url.Https _) -> true
@@ -197,7 +212,7 @@ let default_request_data =
                                     Eliom_common.Full_state_name_table.empty,
                                     Eliom_common.Full_state_name_table.empty);
 
-       si_tab_cookies = Ocsigen_cookies.CookiesTable.empty;
+       si_tab_cookies = Ocsigen_cookie_map.Map_inner.empty;
 
        si_nonatt_info = Eliom_common.RNa_no;
        si_state_info = (Eliom_common.RAtt_no,
@@ -213,6 +228,9 @@ let default_request_data =
        si_all_get_but_na_nl = lazy [];
        si_all_get_but_nl = [];
 
+       si_ignored_get_params = [];
+       si_ignored_post_params = [];
+
        si_client_process_info = None;
        si_expect_process_data = lazy false;
       }
@@ -230,3 +248,6 @@ exception Eliom_no_raw_post_data_on_client
 let raw_post_data _ = Lwt.fail Eliom_no_raw_post_data_on_client
 
 type raw_post_data = unit
+
+let get_ignored_get_params () = (get_sess_info ()).si_ignored_get_params
+let get_ignored_post_params () = (get_sess_info ()).si_ignored_post_params
