@@ -33,21 +33,22 @@ let find_sitedata fun_name =
 (*****************************************************************************)
 let get_http_method () =
   let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.meth sp.Eliom_common.sp_request.request_info
-
-let get_http_header () =
-  let sp = Eliom_common.get_sp () in
-  let http_frame =
-    Ocsigen_request_info.http_frame sp.sp_request.request_info
-  in
-  http_frame.frame_header
+  Ocsigen_request.meth sp.Eliom_common.sp_request.request_info
 
 let get_user_agent () =
   let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.user_agent sp.Eliom_common.sp_request.request_info
+  match
+    Ocsigen_request.header
+      sp.Eliom_common.sp_request.request_info
+      Ocsigen_header.Name.user_agent
+  with
+  | Some ua ->
+    ua
+  | None ->
+    ""
 
 let get_full_url_sp sp =
-  Ocsigen_request_info.url_string sp.Eliom_common.sp_request.request_info
+  Uri.to_string (Ocsigen_request.uri sp.Eliom_common.sp_request.request_info)
 
 let get_full_url () =
   let sp = Eliom_common.get_sp () in
@@ -55,19 +56,11 @@ let get_full_url () =
 
 let get_remote_ip () =
   let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.remote_ip sp.Eliom_common.sp_request.request_info
-
-let get_remote_port () =
-  let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.remote_port sp.Eliom_common.sp_request.request_info
-
-let get_remote_inet_addr () =
-  let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.remote_inet_addr sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.remote_ip sp.Eliom_common.sp_request.request_info
 
 let get_get_params () =
   let sp = Eliom_common.get_sp () in
-  Lazy.force (Ocsigen_request_info.get_params sp.Eliom_common.sp_request.request_info)
+  Ocsigen_request.get_params sp.Eliom_common.sp_request.request_info
 
 let get_all_current_get_params_sp sp =
   sp.Eliom_common.sp_si.Eliom_common.si_all_get_params
@@ -76,29 +69,21 @@ let get_all_current_get_params () =
   let sp = Eliom_common.get_sp () in
   get_all_current_get_params_sp sp
 
-let get_initial_get_params () =
-  let sp = Eliom_common.get_sp () in
-  Lazy.force (Ocsigen_request_info.initial_get_params sp.Eliom_common.sp_request.request_info)
-
 let get_post_params_sp sp =
-  match Ocsigen_request_info.post_params sp.Eliom_common.sp_request.request_info with
-    | None -> None
-    | Some f ->
-      Some (f
-        (sp.Eliom_common.sp_request.request_config.uploaddir,
-         sp.Eliom_common.sp_request.request_config.maxuploadfilesize))
+  Ocsigen_request.post_params
+    sp.Eliom_common.sp_request.request_info
+    sp.Eliom_common.sp_request.request_config.uploaddir
+    sp.Eliom_common.sp_request.request_config.maxuploadfilesize
 
 let get_post_params () =
   let sp = Eliom_common.get_sp () in
   get_post_params_sp sp
 
 let get_files_sp sp =
-  match Ocsigen_request_info.files sp.Eliom_common.sp_request.request_info with
-    | None -> None
-    | Some f ->
-      Some (f
-        (sp.Eliom_common.sp_request.request_config.uploaddir,
-         sp.Eliom_common.sp_request.request_config.maxuploadfilesize))
+  Ocsigen_request.files
+    sp.Eliom_common.sp_request.request_info
+    sp.Eliom_common.sp_request.request_config.uploaddir
+    sp.Eliom_common.sp_request.request_config.maxuploadfilesize
 
 let get_all_files () =
   let sp = Eliom_common.get_sp () in
@@ -108,51 +93,47 @@ let get_all_post_params () =
   let sp = Eliom_common.get_sp () in
   sp.Eliom_common.sp_si.Eliom_common.si_all_post_params
 
+let get_ignored_get_params () =
+  let sp = Eliom_common.get_sp () in
+  sp.Eliom_common.sp_si.Eliom_common.si_ignored_get_params
+
+let get_ignored_post_params () =
+  let sp = Eliom_common.get_sp () in
+  sp.Eliom_common.sp_si.Eliom_common.si_ignored_post_params
+
 let get_original_full_path_string_sp sp =
-  Ocsigen_request_info.original_full_path_string sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.original_full_path_string sp.Eliom_common.sp_request.request_info
 
 let get_original_full_path_string () =
   let sp = Eliom_common.get_sp () in
   get_original_full_path_string_sp sp
 
 let get_original_full_path_sp sp =
-  Ocsigen_request_info.original_full_path sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.original_full_path sp.Eliom_common.sp_request.request_info
 
 let get_original_full_path () =
   let sp = Eliom_common.get_sp () in
   get_original_full_path_sp sp
 
-let get_current_full_path () =
-  let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.full_path sp.Eliom_common.sp_request.request_info
-
-let get_current_full_path_string () =
-  let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.full_path_string sp.Eliom_common.sp_request.request_info
-
 let get_current_sub_path () =
   let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.sub_path sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.sub_path sp.Eliom_common.sp_request.request_info
 
 let get_current_sub_path_string () =
   let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.sub_path_string sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.sub_path_string sp.Eliom_common.sp_request.request_info
 
 let get_header_hostname () =
   let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.host sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.host sp.Eliom_common.sp_request.request_info
 
 let get_timeofday_sp sp =
-  Ocsigen_request_info.timeofday sp.Eliom_common.sp_request.request_info
-
-let get_request_id_sp sp = Int64.bits_of_float (get_timeofday_sp sp)
+  Ocsigen_request.timeofday sp.Eliom_common.sp_request.request_info
 
 let get_timeofday () =
   let sp = Eliom_common.get_sp () in
   get_timeofday_sp sp
 
-let get_request_id () = Int64.bits_of_float (get_timeofday ())
-
 let get_hostname_sp sp =
   Ocsigen_extensions.get_hostname sp.Eliom_common.sp_request
 
@@ -168,15 +149,21 @@ let get_server_port () =
   get_server_port_sp sp
 
 let get_ssl_sp sp =
-  Ocsigen_request_info.ssl sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.ssl sp.Eliom_common.sp_request.request_info
 
 let get_ssl () =
   let sp = Eliom_common.get_sp () in
   get_ssl_sp sp
 
+let get_accept_language_sp sp =
+  Ocsigen_header.Accept_language.parse
+    (Ocsigen_request.header_multi
+       sp.Eliom_common.sp_request.request_info
+       Ocsigen_header.Name.accept_language)
+
 let get_accept_language () =
   let sp = Eliom_common.get_sp () in
-  Lazy.force (Ocsigen_request_info.accept_language sp.Eliom_common.sp_request.request_info)
+  get_accept_language_sp sp
 
 let get_other_get_params () =
   let sp = Eliom_common.get_sp () in
@@ -212,26 +199,22 @@ let get_state_name () =
   let sp = Eliom_common.get_sp () in
   sp.Eliom_common.sp_full_state_name
 let get_request_cache_sp sp =
-  Ocsigen_request_info.request_cache sp.Eliom_common.sp_request.request_info
+  Ocsigen_request.request_cache sp.Eliom_common.sp_request.request_info
 let get_request_cache () =
   let sp = Eliom_common.get_sp () in
   get_request_cache_sp sp
-let clean_request_cache () =
-  let sp = Eliom_common.get_sp () in
-  Ocsigen_request_info.update_request_cache sp.Eliom_common.sp_request.request_info
-    (Polytables.create ())
 let get_link_too_old () =
   let sp = Eliom_common.get_sp () in
   try
     Polytables.get
-      ~table:(Ocsigen_request_info.request_cache sp.Eliom_common.sp_request.request_info)
+      ~table:(Ocsigen_request.request_cache sp.Eliom_common.sp_request.request_info)
       ~key:Eliom_common.eliom_link_too_old
   with Not_found -> false
 let get_expired_service_sessions () =
   let sp = Eliom_common.get_sp () in
   try
     Polytables.get
-      ~table:(Ocsigen_request_info.request_cache sp.Eliom_common.sp_request.request_info)
+      ~table:(Ocsigen_request.request_cache sp.Eliom_common.sp_request.request_info)
       ~key:Eliom_common.eliom_service_session_expired
   with Not_found -> ([], [])
 
@@ -239,7 +222,7 @@ let get_cookies ?(cookie_level = `Session) () =
   let sp = Eliom_common.get_sp () in
   match cookie_level with
     | `Session ->
-      Lazy.force (Ocsigen_request_info.cookies sp.Eliom_common.sp_request.request_info)
+      Ocsigen_request.cookies sp.Eliom_common.sp_request.request_info
     | `Client_process ->
       sp.Eliom_common.sp_si.Eliom_common.si_tab_cookies
 
@@ -339,9 +322,9 @@ let get_ri () =
   let sp = Eliom_common.get_sp () in
   get_ri_sp sp
 
-let get_tmp_filename fi = fi.tmp_filename
-let get_filesize fi = fi.filesize
-let get_original_filename fi = fi.original_basename
+let get_tmp_filename fi = fi.Ocsigen_extensions.tmp_filename
+let get_filesize fi = fi.Ocsigen_extensions.filesize
+let get_original_filename fi = fi.Ocsigen_extensions.raw_original_filename
 let get_file_content_type fi = fi.file_content_type
 
 let get_sitedata () =
@@ -357,11 +340,8 @@ let set_site_handler sitedata handler =
 
 type raw_post_data =
   ((string * string) * (string * string) list) option *
-  string Ocsigen_stream.t option
+  Cohttp_lwt.Body.t
 
 let raw_post_data sp =
   let ri = get_ri_sp sp in
-  Lwt.return
-    ((Ocsigen_request_info.content_type ri,
-      (Ocsigen_request_info.http_frame ri).
-        Ocsigen_http_frame.frame_content))
+  Lwt.return (Ocsigen_request.content_type ri, Ocsigen_request.body ri)
