@@ -19,13 +19,7 @@
  *)
 
 (** This module contains the functions you need to get (or set)
-    information about current request.
- *)
-
-open Eliom_lib
-
-open Ocsigen_extensions
-open Ocsigen_cookies
+    information about current request. *)
 
 (** {2 Getting information about the request} *)
 
@@ -33,10 +27,7 @@ open Ocsigen_cookies
 val in_request_handler : unit -> bool
 
 (** returns the HTTP method used for the request (usually GET or POST). *)
-val get_http_method : unit -> Ocsigen_http_frame.Http_header.http_method
-
-(** returns the HTTP header of the request *)
-val get_http_header : unit -> Ocsigen_http_frame.Http_header.http_header
+val get_http_method : unit -> Cohttp.Code.meth
 
 (** returns the name of the user agent that did the request
    (usually the name of the browser). *)
@@ -48,26 +39,13 @@ val get_full_url : unit -> string
 (** returns the internet address of the client as a string *)
 val get_remote_ip : unit -> string
 
-(** returns client's port *)
-val get_remote_port : unit -> int
-
-(** returns the internet address of the client,
-   using the type [Unix.inet_addr] (defined in OCaml's standard library). *)
-val get_remote_inet_addr : unit -> Unix.inet_addr
-
-(** returns the full path of the URL as a string. *)
-val get_current_full_path_string : unit -> string
-
-(** returns the full path of the URL using the type [Url.path] *)
-val get_current_full_path : unit -> Url.path
-
 (** returns the full path of the URL as first sent by the browser
     (not changed by previous extensions like rewritemod) *)
 val get_original_full_path_string : unit -> string
 
 (** returns the full path of the URL as first sent by the browser
     (not changed by previous extensions like rewritemod) *)
-val get_original_full_path : unit -> Url.path
+val get_original_full_path : unit -> Eliom_lib.Url.path
 
 (** returns the sub path of the URL as a string.
     The sub-path is the full path without the path of the site (set in the
@@ -75,11 +53,11 @@ val get_original_full_path : unit -> Url.path
  *)
 val get_current_sub_path_string : unit -> string
 
-(** returns the sub path of the URL using the type [Url.path].
+(** returns the sub path of the URL using the type [Eliom_lib.Url.path].
     The sub-path is the full path without the path of the site (set in the
     configuration file).
  *)
-val get_current_sub_path : unit -> Url.path
+val get_current_sub_path : unit -> Eliom_lib.Url.path
 
 (** returns the hostname that has been sent by the user agent.
     For HTTP/1.0, the Host field is not mandatory in the request.
@@ -109,22 +87,15 @@ val get_ssl : unit -> bool
 val get_accept_language : unit -> (string * float option) list
 
 (** returns the suffix of the current URL *)
-val get_suffix : unit -> Url.path option
+val get_suffix : unit -> Eliom_lib.Url.path option
 
 (** returns the cookies sent by the browser *)
 val get_cookies : ?cookie_level:Eliom_common.cookie_level ->
-  unit -> string CookiesTable.t
+  unit -> string Ocsigen_cookie_map.Map_inner.t
 
 (** returns an Unix timestamp associated to the request *)
 val get_timeofday : unit -> float
 
-(** returns an unique id associated to the request *)
-val get_request_id : unit -> int64
-
-
-
-
-
 (** {3 Exceptions and fallbacks} *)
 
 (** returns a table in which you can store all the data you want during a
@@ -136,9 +107,6 @@ val get_request_id : unit -> int64
  *)
 val get_request_cache : unit -> Polytables.t
 
-(** Remove all data from the request cache *)
-val clean_request_cache : unit -> unit
-
 (** returns [true] if the coservice called has not been found.
     In that case, the current service is the fallback.
 *)
@@ -150,30 +118,29 @@ val get_expired_service_sessions :
   unit -> (Eliom_common.full_state_name list *
                        Eliom_common.full_state_name list)
 
-(** {2 Getting information about files uploaded} *)
-
-(** Warning: The files uploaded are automatically erased by Ocsigen
-   just after the request has been fulfilled.
-   If you want to keep them, create a new hard link yourself during
-   the service (or make a copy).
+(** returns the HTTP error code sent by the Ocsigen extension
+   that tried to answer to the request before Eliom.
+   It is 404 by default.
  *)
+val get_previous_extension_error_code : unit -> int
 
 (** returns the filename used by Ocsigen for the uploaded file. *)
-val get_tmp_filename : file_info -> string
+val get_tmp_filename : Ocsigen_extensions.file_info -> string
 
-(** returns the size of the file. *)
-val get_filesize : file_info -> int64
+(** returns the size of the uploaded file. *)
+val get_filesize : Ocsigen_extensions.file_info -> int64
 
 (** returns the name the file had on the client when it has been sent. *)
-val get_original_filename : file_info -> string
+val get_original_filename : Ocsigen_extensions.file_info -> string
 
 (** returns the content type sent by the browser with the file
     (if any). *)
 val get_file_content_type :
-  file_info -> ((string * string) * (string * string) list) option
+  Ocsigen_extensions.file_info -> ((string * string) * (string * string) list) option
+
 
 (** returns the root of the site. *)
-val get_site_dir : unit -> Url.path
+val get_site_dir : unit -> Eliom_lib.Url.path
 
 
 
@@ -195,31 +162,27 @@ val get_site_dir : unit -> Url.path
    a page with GET parameters, only the parameters of that non-attached
    coservice are returned (even if the other are still in the URL).
  *)
-val get_get_params : unit -> (string * string) list
+val get_get_params : unit -> (string * string list) list
 
 (** returns current parameters of the URL (GET parameters)
    (even those that are for subsequent services, but not previous actions) *)
 val get_all_current_get_params : unit -> (string * string) list
 
-(** returns all parameters of the URL (GET parameters)
-    as sent initially by the browser *)
-val get_initial_get_params : unit -> (string * string) list
-
 (** returns the parameters of the URL (GET parameters)
    that do not concern the running service. *)
 val get_other_get_params : unit -> (string * string) list
 
 (** returns non localized parameters in the URL. *)
 val get_nl_get_params :
-  unit -> (string * string) list String.Table.t
+  unit -> (string * string) list Eliom_lib.String.Table.t
 
 (** returns persistent non localized parameters in the URL. *)
 val get_persistent_nl_get_params :
-  unit -> (string * string) list String.Table.t
+  unit -> (string * string) list Eliom_lib.String.Table.t
 
 (** returns non localized POST parameters. *)
 val get_nl_post_params :
-  unit -> (string * string) list String.Table.t
+  unit -> (string * string) list Eliom_lib.String.Table.t
 
 (** returns the parameters in the body of the HTTP request (POST parameters)
     that concern the running service. None means that POST data where
@@ -232,8 +195,15 @@ val get_all_post_params : unit -> (string * string) list option
 
 (** returns all files in he HTTP request
    (even those that are for another service) *)
-val get_all_files : unit -> (string * file_info) list option
+val get_all_files : unit -> (string * Ocsigen_extensions.file_info) list option
 
+(** returns the GET parameters that have been ignored using
+    <ignoredgetparams/> in config file. *)
+val get_ignored_get_params : unit -> (string * string) list
+
+(** returns the POST parameters that have been ignored using
+    <ignoredpostparams/> in config file. *)
+val get_ignored_post_params : unit -> (string * string) list
 
 
 (*****************************************************************************)
@@ -242,10 +212,10 @@ val get_all_files : unit -> (string * file_info) list option
 (** You probably don't need these functions. *)
 
 (** returns all the information about the request. *)
-val get_ri : unit -> Ocsigen_extensions.request_info
+val get_ri : unit -> Ocsigen_request.t
 
 (** returns all the information about the request and config. *)
-val get_request : unit -> request
+val get_request : unit -> Ocsigen_extensions.request
 
 (** returns the name of the sessions to which belongs the running service
     ([None] if it is not a session service)
@@ -263,10 +233,6 @@ val get_persistent_cookies :
 val get_data_cookies :
   unit -> string Eliom_common.Full_state_name_table.t
 
-
-(** Returns the http error code of the request before Eliom was called *)
-val get_previous_extension_error_code :unit -> int
-
 (** Returns [true] if the request was done by a client side Eliom program,
     which was expecting to receive a new HTML page to display inside
     the process. *)
@@ -283,7 +249,7 @@ val expecting_process_page : unit -> bool
     If there is no client side process, same as
     {!get_original_full_path}.
 *)
-val get_csp_original_full_path : unit -> Url.path
+val get_csp_original_full_path : unit -> Eliom_lib.Url.path
 
 (** returns the hostname used for absolute links, computed
     when launching the client side process for the first time.
@@ -314,7 +280,7 @@ val get_sitedata : unit -> Eliom_common.sitedata
 
 (**/**)
 
-val get_csp_original_full_path_sp : Eliom_common.server_params -> Url.path
+val get_csp_original_full_path_sp : Eliom_common.server_params -> Eliom_lib.Url.path
 val get_csp_hostname_sp : Eliom_common.server_params -> string
 val get_csp_server_port_sp : Eliom_common.server_params -> int
 val get_csp_ssl_sp : Eliom_common.server_params -> bool
@@ -344,44 +310,47 @@ val find_sitedata : string -> Eliom_common.sitedata
 
 val get_si : Eliom_common.server_params -> Eliom_common.sess_info
 
-val get_user_cookies : unit -> Ocsigen_cookies.cookieset
-val get_user_tab_cookies : unit -> Ocsigen_cookies.cookieset
+val get_user_cookies : unit -> Ocsigen_cookie_map.t
+val get_user_tab_cookies : unit -> Ocsigen_cookie_map.t
 
 val get_sp_client_appl_name : unit -> string option
 val get_sp_client_process_info_sp :
   Eliom_common.server_params -> Eliom_common.client_process_info
 val get_sp_client_process_info : unit -> Eliom_common.client_process_info
 
-val set_site_handler : Eliom_common.sitedata ->
-  (exn -> Ocsigen_http_frame.result Lwt.t) -> unit
+val set_site_handler :
+  Eliom_common.sitedata ->
+  (exn -> Ocsigen_response.t Lwt.t) -> unit
 
-val get_request_sp : Eliom_common.server_params -> request
-val get_site_dir_sp : Eliom_common.server_params -> Url.path
+val get_request_sp : Eliom_common.server_params -> Ocsigen_extensions.request
+val get_site_dir_sp : Eliom_common.server_params -> Eliom_lib.Url.path
 val get_hostname_sp : Eliom_common.server_params -> string
 val get_full_url_sp : Eliom_common.server_params -> string
 
 val get_other_get_params_sp : Eliom_common.server_params -> (string * string) list
 val get_nl_get_params_sp :
-  Eliom_common.server_params -> (string * string) list String.Table.t
+  Eliom_common.server_params ->
+  (string * string) list Eliom_lib.String.Table.t
 val get_persistent_nl_get_params_sp :
-  Eliom_common.server_params -> (string * string) list String.Table.t
+  Eliom_common.server_params ->
+  (string * string) list Eliom_lib.String.Table.t
 val get_nl_post_params_sp :
-  Eliom_common.server_params -> (string * string) list String.Table.t
+  Eliom_common.server_params ->
+  (string * string) list Eliom_lib.String.Table.t
 
-val get_original_full_path_sp : Eliom_common.server_params -> Url.path
+val get_original_full_path_sp : Eliom_common.server_params -> Eliom_lib.Url.path
 val get_original_full_path_string_sp : Eliom_common.server_params -> string
 val get_server_port_sp : Eliom_common.server_params -> int
 val get_ssl_sp : Eliom_common.server_params -> bool
-val get_ri_sp : Eliom_common.server_params -> Ocsigen_extensions.request_info
+val get_ri_sp : Eliom_common.server_params -> Ocsigen_request.t
 val get_post_params_sp : Eliom_common.server_params -> (string * string) list Lwt.t option
-val get_files_sp : Eliom_common.server_params -> (string * file_info) list Lwt.t option
+val get_files_sp : Eliom_common.server_params -> (string * Ocsigen_extensions.file_info) list Lwt.t option
 
-val get_suffix_sp : Eliom_common.server_params -> Url.path option
+val get_suffix_sp : Eliom_common.server_params -> Eliom_lib.Url.path option
 val get_request_cache_sp : Eliom_common.server_params -> Polytables.t
-val get_request_id_sp : Eliom_common.server_params -> int64
 
 type raw_post_data =
   ((string * string) * (string * string) list) option *
-  string Ocsigen_stream.t option
+  Cohttp_lwt.Body.t
 
 val raw_post_data : Eliom_common.server_params -> raw_post_data Lwt.t
