@@ -328,7 +328,7 @@ let rec close_service_state_if_empty ~scope ?secure () =
         if
           (Eliommod_sessiongroups.Data.group_size
              (sitedata.Eliom_common.site_dir_string, `Client_process,
-              Left c.Eliom_common.sc_value)
+              Left Eliom_common.(Hashed_cookies.to_string c.sc_hvalue))
            = 0) (* no tab sessions *)
           &&
             (Eliom_common.service_tables_are_empty !(c.Eliom_common.sc_table))
@@ -367,11 +367,11 @@ let rec close_volatile_state_if_empty ~scope ?secure () =
               when
                 (Eliommod_sessiongroups.Data.group_size
                    (sitedata.Eliom_common.site_dir_string, `Client_process,
-                    Left c.Eliom_common.dc_value)
+                    Left Eliom_common.(Hashed_cookies.to_string c.dc_hvalue))
                  = 0) (* no tab sessions *)
                 &&
                   (sitedata.Eliom_common.not_bound_in_data_tables
-                     c.Eliom_common.dc_value)
+                     Eliom_common.(Hashed_cookies.to_string c.dc_hvalue))
                 ->
             Eliommod_sessiongroups.Data.remove
               c.Eliom_common.dc_session_group_node
@@ -380,7 +380,7 @@ let rec close_volatile_state_if_empty ~scope ?secure () =
 (* This should never occur, because we always have tab session data
    when we have a tab session (at least the change_page_event).
         if (sitedata.Eliom_common.not_bound_in_data_tables
-              c.Eliom_common.dc_value)
+              c.Eliom_common.dc_hvalue)
         then Eliommod_sessiongroups.Data.remove
           c.Eliom_common.dc_session_group_node *)
       | `Session_group scope_hierarchy ->
@@ -394,7 +394,7 @@ let rec close_volatile_state_if_empty ~scope ?secure () =
 
 
 
-let close_persistent_state_if_empty ~scope ?secure () =
+let close_persistent_state_if_empty ~scope:_ ?secure:_ () =
   Lwt.return_unit
 (*VVV Can we implement this function? *)
 
@@ -481,7 +481,7 @@ let get_service_session_group_size
     in
     match !(c.Eliom_common.sc_session_group) with
       | _, _, Right _ -> None
-      | _, _, Left v ->
+      | _, _, Left _ ->
         Some (Eliommod_sessiongroups.Serv.group_size !(c.Eliom_common.sc_session_group))
   with
     | Not_found
@@ -564,7 +564,7 @@ let get_volatile_data_session_group_size
     in
     match !(c.Eliom_common.dc_session_group) with
       | _, _, Right _ -> None
-      | _, _, Left v ->
+      | _, _, Left _ ->
         Some (Eliommod_sessiongroups.Data.group_size !(c.Eliom_common.dc_session_group))
   with
     | Not_found
@@ -587,7 +587,7 @@ let set_persistent_data_session_group ?set_max
     sitedata
     ?set_max
     (fst sitedata.Eliom_common.max_persistent_data_sessions_per_group)
-    c.Eliom_common.pc_value !grp n in
+    Eliom_common.(Hashed_cookies.to_string c.pc_hvalue) !grp n in
   let%lwt () = Lwt_list.iter_p
     (Eliommod_persess.close_persistent_state2
        ~scope:(scope:>Eliom_common.user_scope) sitedata None) l in
@@ -604,7 +604,7 @@ let unset_persistent_data_session_group
       ~secure_o:secure ~sp () in
     let grp = c.Eliom_common.pc_session_group in
     let%lwt () = Eliommod_sessiongroups.Pers.remove
-      sitedata c.Eliom_common.pc_value !grp in
+      sitedata Eliom_common.(Hashed_cookies.to_string c.pc_hvalue) !grp in
     grp := None;
 
     close_persistent_state_if_empty
@@ -884,6 +884,8 @@ let get_session_service_table_if_exists ~sp
 (*****************************************************************************)
 (** {2 persistent sessions} *)
 
+module Ocsipersist = Eliom_common.Ocsipersist.Polymorphic
+
 type 'a persistent_table =
     (Eliom_common.user_scope *
        bool *
@@ -892,7 +894,7 @@ type 'a persistent_table =
 let create_persistent_table ~scope ?secure name : 'a persistent_table Lwt.t =
   let sitedata = Eliom_request_info.find_sitedata "create_persistent_table" in
   let secure = Eliom_common.get_secure ~secure_o:secure ~sitedata () in
-  Eliom_common.create_persistent_table name >>= fun t ->
+  Eliom_common.Persistent_tables.create name >>= fun t ->
   Lwt.return (scope, secure, t)
 
 let get_p_table_key_
@@ -901,18 +903,22 @@ let get_p_table_key_
                     secure_o:bool option ->
                     ?sp:Eliom_common.server_params ->
                     unit -> Eliom_common.one_persistent_cookie_info Lwt.t)) =
+  let get_cookie () =
+    let cookie_scope = Eliom_common.cookie_scope_of_user_scope scope in
+    let%lwt c = find_cookie ~cookie_scope ~secure_o:(Some secure) () in
+    Lwt.return Eliom_common.(Hashed_cookies.to_string c.pc_hvalue)
+  in
   let%lwt key = match scope with
     | `Session_group state_name ->
       begin
         match%lwt get_persistent_data_session_group
                     ~scope:(`Session state_name) ~secure () with
           | Some a -> Lwt.return a
-          | _ -> Lwt.return Eliom_common.default_group_name
+          | None ->
+            (* No session group. We use the session cookie as key. *)
+             get_cookie ()
       end
-    | _ ->
-      let cookie_scope = Eliom_common.cookie_scope_of_user_scope scope in
-      let%lwt c = find_cookie ~cookie_scope ~secure_o:(Some secure) () in
-      Lwt.return c.Eliom_common.pc_value
+    | _ -> get_cookie ()
   in
   Lwt.return (table, key)
 
@@ -972,7 +978,6 @@ let create_volatile_table ~scope ?secure () =
             (Eliom_common.Eliom_site_information_not_available
                "create_volatile_table"))
   | Some sp ->
-    let sp = Eliom_common.get_sp () in
     let sitedata = Eliom_request_info.get_sitedata_sp ~sp in
     let secure = Eliom_common.get_secure ~secure_o:secure ~sitedata () in
     create_volatile_table_during_session_ ~scope ~secure sitedata
@@ -985,16 +990,21 @@ let get_table_key_ ~table:(scope, secure, table)
        unit -> Eliom_common.one_data_cookie_info) =
   (* The key in the table is the cookie for client processes and sessions,
      and the group name for groups *)
+  let get_cookie () =
+    let cookie_scope = Eliom_common.cookie_scope_of_user_scope scope in
+      let c = find_cookie ~cookie_scope ~secure_o:(Some secure) () in
+      Eliom_common.(Hashed_cookies.to_string c.dc_hvalue)
+  in
   (table, match scope with
     | `Session_group state_name ->
       (match get_volatile_data_session_group
           ~scope:(`Session state_name) ~secure ()
        with Some a -> a
-         | _ -> Eliom_common.default_group_name)
-    | _ ->
-      let cookie_scope = Eliom_common.cookie_scope_of_user_scope scope in
-      let c = find_cookie ~cookie_scope ~secure_o:(Some secure) () in
-      c.Eliom_common.dc_value)
+          | None ->
+            (* No session group has been set. We use the session instead. *)
+            get_cookie ()
+      )
+    | _ -> get_cookie ())
 
 
 let get_volatile_data ~table () =
@@ -1166,20 +1176,14 @@ module Ext = struct
 
   type service_cookie_info =
       string (* cookie value *)
-      * Eliom_common.tables Eliom_common.servicecookiestablecontent
+      * Eliom_common.tables Eliom_common.Service_cookie.t
 
   type data_cookie_info =
       string (* cookie value *)
-      * Eliom_common.datacookiestablecontent
+      * Eliom_common.Data_cookie.t
 
   type persistent_cookie_info =
-      string (* cookie value *)
-        *
-      (Eliom_common.full_state_name *
-       float option            (* expiration date by timeout
-                                  (server side) *) *
-       Eliom_common.timeout        (* user timeout *) *
-       Eliom_common.perssessgrp option           (* session group *))
+      string (* cookie value *) * Eliommod_cookies.cookie
 
 
   let untype_state state = state
@@ -1210,25 +1214,52 @@ module Ext = struct
       ?(scope = Eliom_common.default_group_scope) group_name =
     ((scope :> Eliom_common.user_scope), `Service, group_name)
 
-  let current_volatile_session_state
-      ?secure ?(scope = Eliom_common.default_session_scope) () =
-    let cookie = Eliommod_datasess.find_or_create_data_cookie
-        ~secure_o:secure ~cookie_scope:scope () in
-    ((scope :> Eliom_common.user_scope), `Data, cookie.Eliom_common.dc_value)
+  let current_volatile_data_state
+      ?secure ?(scope = (Eliom_common.default_session_scope
+                         :> Eliom_common.user_scope)) () =
+    let scope = (scope :> Eliom_common.user_scope) in
+    match scope with
+    | `Session_group h ->
+      (match get_volatile_data_session_group ~scope:(`Session h) ?secure () with
+       | Some g -> volatile_data_group_state ~scope:(`Session_group h) g
+       | None -> raise Not_found)
+    | #Eliom_common.cookie_scope as cookie_scope ->
+      let cookie = Eliommod_datasess.find_or_create_data_cookie
+          ~secure_o:secure ~cookie_scope ()
+      in
+      ((scope, `Data, Eliom_common.(Hashed_cookies.to_string cookie.dc_hvalue))
+       : ('a, 'b) state)
 
-  let current_persistent_session_state
-      ?secure ?(scope = Eliom_common.default_session_scope) () =
-    Eliommod_persess.find_or_create_persistent_cookie
-      ~secure_o:secure ~cookie_scope:scope ()
-    >>= fun cookie ->
-    Lwt.return ((scope :> Eliom_common.user_scope),
-                `Pers, cookie.Eliom_common.pc_value)
+  let current_persistent_data_state
+      ?secure ?(scope = (Eliom_common.default_session_scope
+                         :> Eliom_common.user_scope)) () =
+    let scope = (scope :> Eliom_common.user_scope) in
+    match scope with
+    | `Session_group h ->
+      (match%lwt get_persistent_data_session_group ~scope:(`Session h) ?secure ()
+       with
+       | Some g ->
+         persistent_data_group_state ~scope:(`Session_group h) g |> Lwt.return
+       | None -> Lwt.fail Not_found)
+    | #Eliom_common.cookie_scope as cookie_scope ->
+      Eliommod_persess.find_or_create_persistent_cookie
+        ~secure_o:secure ~cookie_scope () >>= fun cookie ->
+      Lwt.return
+        (scope, `Pers, Eliom_common.(Hashed_cookies.to_string cookie.pc_hvalue))
 
-  let current_service_session_state
-      ?secure ?(scope = Eliom_common.default_session_scope) () =
-    let cookie = Eliommod_sersess.find_or_create_service_cookie
-      ~secure_o:secure ~cookie_scope:scope () in
-    ((scope :> Eliom_common.user_scope), `Service, cookie.Eliom_common.sc_value)
+  let current_service_state
+      ?secure ?(scope = (Eliom_common.default_session_scope
+                         :> Eliom_common.user_scope)) () =
+    let scope = (scope :> Eliom_common.user_scope) in
+    match scope with
+    | `Session_group h ->
+      (match get_service_session_group ~scope:(`Session h) ?secure () with
+       | Some g -> service_group_state ~scope:(`Session_group h) g
+       | None -> raise Not_found)
+    | #Eliom_common.cookie_scope as cookie_scope ->
+      let cookie = Eliommod_sersess.find_or_create_service_cookie
+          ~secure_o:secure ~cookie_scope () in
+      (scope, `Service, Eliom_common.(Hashed_cookies.to_string cookie.sc_hvalue))
 
   let get_service_cookie_info
       ((_, _, cookie) : ([< Eliom_common.cookie_level ], [ `Service ]) state) =
@@ -1244,8 +1275,7 @@ module Ext = struct
 
   let get_persistent_cookie_info
       ((_, _, cookie) : ([< Eliom_common.cookie_level ], [ `Pers ]) state) =
-    Lazy.force Eliommod_persess.persistent_cookies_table >>= fun table ->
-    Ocsipersist.find table cookie >>= fun v ->
+    Eliommod_cookies.Persistent_cookies.Cookies.find cookie >>= fun v ->
     Lwt.return (cookie, v)
 
   let discard_state ~state =
@@ -1276,23 +1306,27 @@ module Ext = struct
         in
         Eliommod_sessiongroups.Pers.remove_group
           ~cookie_level:`Session sitedata sgr_o
-      | (_, `Service, (cookie : string)) ->
-        let (_, (_, _, _, _, _sgr, sgrnode)) = get_service_cookie_info state in
-        Eliommod_sessiongroups.Serv.remove sgrnode;
+      | (_, `Service, (_cookie : string)) ->
+        let (_, {Eliom_common.Service_cookie.session_group_node}) =
+          get_service_cookie_info state
+        in
+        Eliommod_sessiongroups.Serv.remove session_group_node;
         Lwt.return_unit
-      | (_, `Data, cookie) ->
-        let (_, (_, _, _, _sgr, sgrnode)) =
+      | (_, `Data, _cookie) ->
+        let (_, {Eliom_common.Data_cookie.session_group_node}) =
           get_volatile_data_cookie_info state in
-        Eliommod_sessiongroups.Data.remove sgrnode;
+        Eliommod_sessiongroups.Data.remove session_group_node;
         Lwt.return_unit
-      | (_, `Pers, cookie) ->
-        get_persistent_cookie_info state
-        >>= fun (cookie, ((scope, _, _), _, _, sgr_o)) ->
+      | (_, `Pers, _cookie) ->
+        let%lwt cookie, {Eliommod_cookies.full_state_name; session_group} =
+          get_persistent_cookie_info state
+        in
+        let scope = full_state_name.Eliom_common.user_scope in
         let sitedata = get_sitedata () in
         let cookie_level = Eliom_common.cookie_level_of_user_scope scope in
         Eliommod_sessiongroups.Pers.close_persistent_session2
-          ~cookie_level sitedata sgr_o cookie
-(*VVV!!! est-ce que sgr_o est fullsessgrp ? *)
+          ~cookie_level sitedata session_group cookie
+        (*VVV!!! est-ce que session_group est fullsessgrp ? *)
 
 
   let fold_sub_states_aux_aux
@@ -1307,8 +1341,8 @@ module Ext = struct
         | `Client_process _ -> failwith "fold_sub_states"
       in
       let reduce_level = function
-        | `Session_group n -> `Session
-        | `Session n -> `Client_process
+        | `Session_group _ -> `Session
+        | `Session _ -> `Client_process
         | `Client_process _ -> failwith "fold_sub_states"
       in
       let sub_states_level = reduce_level s in
@@ -1338,9 +1372,7 @@ module Ext = struct
       ~(state : Eliom_common.user_scope * [> `Data | `Service ] * string)
       f e =
     let state' = (state :> ('aa, 'bb) state) in
-    let (sitedata, sub_states_level, id, f) as a =
-      fold_sub_states_aux_aux ?sitedata ~state:state' f
-    in
+    let a = fold_sub_states_aux_aux ?sitedata ~state:state' f in
     fold_sub_states_aux Ocsigen_cache.Dlist.fold Ocsigen_lib.id a e state
 
   let fold_sub_states ?sitedata ~state f e =
@@ -1384,7 +1416,7 @@ module Ext = struct
     (*VVV Does not work with volatile group data *)
     let get_volatile_data
         ~state:((state_scope, _, cookie) : ('s, [ `Data ]) state)
-        ~table:(table_scope, secure, t : 'a volatile_table) =
+        ~table:(table_scope, _secure, t : 'a volatile_table) =
       check_scopes table_scope state_scope;
       Eliom_common.SessionCookies.find t cookie
 
@@ -1396,7 +1428,7 @@ module Ext = struct
 
     let set_volatile_data
         ~state:((state_scope, _, cookie) : ('s, [ `Data ]) state)
-        ~table:(table_scope, secure, t : 'a volatile_table)
+        ~table:(table_scope, _secure, t : 'a volatile_table)
         value =
       check_scopes table_scope state_scope;
       Eliom_common.SessionCookies.replace t cookie value
@@ -1422,55 +1454,50 @@ module Ext = struct
 
   end
 
+  let get_service_cookie_scope ~cookie:(_, cookie) =
+    cookie.Eliom_common.Service_cookie.full_state_name.Eliom_common.user_scope
 
-  let get_service_cookie_scope ~cookie:(_, ((ct, _, _), _, _, _, _, _)) =
-    ct
+  let get_volatile_data_cookie_scope ~cookie:(_, data_cookie) =
+    data_cookie.Eliom_common.Data_cookie.full_state_name.Eliom_common.user_scope
 
-  let get_volatile_data_cookie_scope ~cookie:(_, ((ct, _, _), _, _, _, _)) =
-    ct
+  let get_persistent_data_cookie_scope ~cookie:(_, cookie) =
+    cookie.Eliommod_cookies.full_state_name.Eliom_common.user_scope
 
-  let get_persistent_data_cookie_scope ~cookie:(_, ((ct, _, _), _, _, _)) =
-    ct
+  let set_service_cookie_timeout ~cookie:(_, cookie) t =
+    cookie.Eliom_common.Service_cookie.timeout :=
+      match t with None -> TNone | Some t -> TSome t
 
-  let set_service_cookie_timeout ~cookie:(_, (_, _, _, r, _, _)) t =
-    match t with
-    | None -> r := TNone
-    | Some t -> r := TSome t
+  let set_volatile_data_cookie_timeout ~cookie:(_, data_cookie) t =
+    data_cookie.Eliom_common.Data_cookie.timeout :=
+      match t with None -> TNone | Some t -> TSome t
 
-  let set_volatile_data_cookie_timeout ~cookie:(_, (_, _, r, _, _)) t =
-    match t with
-    | None -> r := TNone
-    | Some t -> r := TSome t
-
-  let set_persistent_data_cookie_timeout
-      ~cookie:(cookie, (fullstname, exp, _, sessgrp)) t =
+  let set_persistent_data_cookie_timeout ~cookie:(c, cookie) t =
     let ti = match t with
     | None -> TNone
     | Some t -> TSome t
     in
-    Lazy.force Eliom_common.persistent_cookies_table >>= fun table ->
-    Ocsipersist.add table cookie (fullstname, exp, ti, sessgrp)
+    Eliommod_cookies.Persistent_cookies.add c {cookie with Eliommod_cookies.timeout = ti}
 
-  let get_service_cookie_timeout ~cookie:(_, (_, _, _, r, _, _)) =
-    !r
+  let get_service_cookie_timeout ~cookie:(_, cookie) =
+    !(cookie.Eliom_common.Service_cookie.timeout)
 
-  let get_volatile_data_cookie_timeout ~cookie:(_, (_, _, r, _, _)) =
-    !r
+  let get_volatile_data_cookie_timeout ~cookie:(_, data_cookie) =
+    !(data_cookie.Eliom_common.Data_cookie.timeout)
 
-  let get_persistent_data_cookie_timeout ~cookie:(_, (_, _, r, _)) =
-    r
+  let get_persistent_data_cookie_timeout ~cookie:(_, cookie) =
+    cookie.Eliommod_cookies.timeout
 
+  let unset_service_cookie_timeout ~cookie:(_, cookie) =
+    cookie.Eliom_common.Service_cookie.timeout := TGlobal
 
-  let unset_service_cookie_timeout ~cookie:(_, (_, _, _, r, _, _)) =
-    r := TGlobal
+  let unset_volatile_data_cookie_timeout ~cookie:(_cookie, data_cookie) =
+    data_cookie.Eliom_common.Data_cookie.timeout := TGlobal
 
-  let unset_volatile_data_cookie_timeout ~cookie:(cookie, (_, _, r, _, _)) =
-    r := TGlobal
-
-  let unset_persistent_data_cookie_timeout
-      ~cookie:(cookie, (fullstname, exp, _, sessgrp)) =
-    Lazy.force Eliom_common.persistent_cookies_table >>= fun table ->
-    Ocsipersist.add table cookie (fullstname, exp, TGlobal, sessgrp)
+  let unset_persistent_data_cookie_timeout ~cookie:(c, cookie) =
+    Eliommod_cookies.Persistent_cookies.Cookies.add c
+      {cookie with Eliommod_cookies.timeout = TGlobal} >>= fun () ->
+    let {Eliommod_cookies.expiry} = cookie in
+    Eliommod_cookies.Persistent_cookies.Expiry_dates.remove_cookie expiry c
 
 
   let get_session_group_list () =
@@ -1518,14 +1545,8 @@ let number_of_table_elements = Eliommod_sessexpl.number_of_table_elements
 let number_of_persistent_data_cookies =
   Eliommod_sessexpl.number_of_persistent_cookies
 
-let number_of_persistent_tables = Eliommod_persess.number_of_persistent_tables
-  (* One table is the main table of sessions *)
-
-let number_of_persistent_table_elements () =
-  Eliommod_persess.number_of_persistent_table_elements ()
-
-
-
+let number_of_persistent_tables = Eliom_common.Persistent_tables.number_of_tables
+let number_of_persistent_table_elements = Eliom_common.Persistent_tables.number_of_table_elements
 
 
 (*****************************************************************************)
@@ -1533,7 +1554,7 @@ let get_service_cookie ~cookie_scope ?secure () =
   try
     let c = Eliommod_sersess.find_service_cookie_only
       ~cookie_scope ~secure_o:secure () in
-    Some c.Eliom_common.sc_value
+    Some c.Eliom_common.sc_hvalue
   with Not_found | Eliom_common.Eliom_Session_expired -> None
 
 let get_volatile_data_cookie ~cookie_scope ?secure () =
@@ -1541,14 +1562,14 @@ let get_volatile_data_cookie ~cookie_scope ?secure () =
     let c = Eliommod_datasess.find_data_cookie_only
         ~cookie_scope ~secure_o:secure ()
     in
-    Some c.Eliom_common.dc_value
+    Some c.Eliom_common.dc_hvalue
   with Not_found | Eliom_common.Eliom_Session_expired -> None
 
 let get_persistent_data_cookie ~cookie_scope ?secure () =
   try%lwt
     let%lwt c = Eliommod_persess.find_persistent_cookie_only
       ~cookie_scope ~secure_o:secure () in
-    return_some c.Eliom_common.pc_value
+    return_some c.Eliom_common.pc_hvalue
   with
     | Not_found
     | Eliom_common.Eliom_Session_expired -> return_none
@@ -1569,12 +1590,12 @@ let set_cookie
   let secure = Eliom_common.get_secure ~secure_o:secure ~sitedata () in
   match cookie_level with
     | `Session ->
-      sp.Eliom_common.sp_user_cookies <- Ocsigen_cookies.add_cookie
-        path name (Ocsigen_cookies.OSet (exp, value, secure))
+      sp.Eliom_common.sp_user_cookies <- Ocsigen_cookie_map.add
+        path name (OSet (exp, value, secure))
         sp.Eliom_common.sp_user_cookies
     | `Client_process ->
-      sp.Eliom_common.sp_user_tab_cookies <- Ocsigen_cookies.add_cookie
-        path name (Ocsigen_cookies.OSet (exp, value, secure))
+      sp.Eliom_common.sp_user_tab_cookies <- Ocsigen_cookie_map.add
+        path name (OSet (exp, value, secure))
         sp.Eliom_common.sp_user_tab_cookies
 
 let unset_cookie
@@ -1583,8 +1604,8 @@ let unset_cookie
   let path = change_pathopt_ sp path in
   match cookie_level with
     | `Session ->
-      sp.Eliom_common.sp_user_cookies <- Ocsigen_cookies.add_cookie
-        path name Ocsigen_cookies.OUnset sp.Eliom_common.sp_user_cookies
+      sp.Eliom_common.sp_user_cookies <- Ocsigen_cookie_map.add
+        path name OUnset sp.Eliom_common.sp_user_cookies
     | `Client_process ->
-      sp.Eliom_common.sp_user_tab_cookies <- Ocsigen_cookies.add_cookie
-        path name Ocsigen_cookies.OUnset sp.Eliom_common.sp_user_tab_cookies
+      sp.Eliom_common.sp_user_tab_cookies <- Ocsigen_cookie_map.add
+        path name OUnset sp.Eliom_common.sp_user_tab_cookies
