@@ -32,10 +32,10 @@ open Ocsigen_extensions
 
 
 (****************************************************************************)
-let default_max_persistent_data_sessions_per_group = ref 5
-let default_max_service_sessions_per_group = ref 5
+let default_max_persistent_data_sessions_per_group = ref 50
+let default_max_service_sessions_per_group = ref 50
 let default_max_service_sessions_per_subnet = ref 1000000
-let default_max_volatile_data_sessions_per_group = ref 5
+let default_max_volatile_data_sessions_per_group = ref 50
 let default_max_volatile_data_sessions_per_subnet = ref 1000000
 let default_max_persistent_data_tab_sessions_per_group = ref 50
 let default_max_service_tab_sessions_per_group = ref 50
@@ -44,6 +44,9 @@ let default_secure_cookies = ref false
 let default_application_script = ref (false, false)
 let default_cache_global_data = ref None
 let default_html_content_type = ref None
+let default_ignored_get_params = ref []
+let default_ignored_post_params = ref []
+let default_omitpersistentstorage = ref None
 
 (* Subnet defaults be large enough, because it must work behind a reverse proxy.
 
@@ -75,125 +78,120 @@ let default_max_anonymous_services_per_session = ref 1000
 let default_max_volatile_groups_per_site  = ref 1000000
 (*VVV value ??? *)
 
+module S =
+  Hashtbl.Make(struct
+    type t = Ocsigen_extensions.virtual_hosts * Eliom_lib.Url.path
+    let equal (vh1, u1 : t) (vh2, u2 : t) =
+      Ocsigen_extensions.equal_virtual_hosts vh1 vh2 &&
+      u1 = u2
+    let hash (vh, u : t) =
+      Hashtbl.hash (Ocsigen_extensions.hash_virtual_hosts vh, u)
+  end)
 
-
-
-let new_sitedata =
-  (* We want to keep the old site data even if we reload the server *)
-  (* To do that, we keep the site data in a table *)
-  let module S = Hashtbl.Make(struct
-                                type t =
-                                    Ocsigen_extensions.virtual_hosts * Url.path
-                                let equal (vh1, u1 : t) (vh2, u2 : t) =
-                                  Ocsigen_extensions.equal_virtual_hosts vh1 vh2
-                                  && u1 = u2
-                                let hash (vh, u : t) =
-                                  Hashtbl.hash (
-                                    Ocsigen_extensions.hash_virtual_hosts vh, u)
-                              end)
+let create_sitedata site_dir config_info =
+  let dlist_table = Eliom_common.create_dlist_ip_table 100
+  and group_of_groups =
+    Ocsigen_cache.Dlist.create !default_max_volatile_groups_per_site
   in
+  let sitedata = {
+    (* One dlist for each site? *)
+    Eliom_common.servtimeout = None, None, [];
+    datatimeout =  None, None, [];
+    perstimeout =  None, None, [];
+    site_value_table = Polytables.create ();
+    site_dir;
+    (*VVV encode=false??? *)
+    site_dir_string = Eliom_lib.Url.string_of_url_path ~encode:false site_dir;
+    config_info;
+    default_links_xhr =
+      Eliom_common.tenable_value ~name:"default_links_xhr" true;
+    global_services =
+      Eliom_common.empty_tables
+        !default_max_anonymous_services_per_subnet
+        false;
+    registered_scope_hierarchies = Eliom_common.Hier_set.empty;
+    session_services = Eliommod_cookies.new_service_cookie_table ();
+    session_data = Eliommod_cookies.new_data_cookie_table ();
+    group_of_groups;
+    remove_session_data = (fun _cookie -> ());
+    not_bound_in_data_tables = (fun _cookie -> true);
+    exn_handler = Eliommod_pagegen.def_handler;
+    unregistered_services = [];
+    unregistered_na_services = [];
+    max_service_sessions_per_group =
+      !default_max_service_sessions_per_group, false;
+    max_volatile_data_sessions_per_group =
+      !default_max_volatile_data_sessions_per_group, false;
+    max_persistent_data_sessions_per_group =
+      Some !default_max_persistent_data_sessions_per_group, false;
+    max_service_tab_sessions_per_group =
+      !default_max_service_tab_sessions_per_group, false;
+    max_volatile_data_tab_sessions_per_group =
+      !default_max_volatile_data_tab_sessions_per_group, false;
+    max_persistent_data_tab_sessions_per_group =
+      Some !default_max_persistent_data_tab_sessions_per_group, false;
+    max_service_sessions_per_subnet =
+      !default_max_service_sessions_per_subnet, false;
+    max_volatile_data_sessions_per_subnet =
+      !default_max_volatile_data_sessions_per_subnet, false;
+    max_anonymous_services_per_session =
+      !default_max_anonymous_services_per_session, false;
+    max_anonymous_services_per_subnet =
+      !default_max_anonymous_services_per_subnet, false;
+    secure_cookies = !default_secure_cookies;
+    dlist_ip_table = dlist_table;
+    ipv4mask = None, false;
+    ipv6mask = None, false;
+    application_script = !default_application_script;
+    cache_global_data = !default_cache_global_data;
+    html_content_type = !default_html_content_type;
+    ignored_get_params = !default_ignored_get_params;
+    ignored_post_params = !default_ignored_post_params;
+    omitpersistentstorage = !default_omitpersistentstorage;
+  } in
+  Ocsigen_cache.Dlist.set_finaliser_after
+    (fun node ->
+       (* Finaliser for the session groups *)
+       (* See in eliommod_sessiongroups for the finaliser of sessions *)
+       let fullbrowsersessgrp = Ocsigen_cache.Dlist.value node in
+       (* When removing a group from the dlist, we must close it.
+          Actually, it must be the only way to close a group. *)
+       (* This finaliser is almost identical to the finaliser for
+          other groups, defined in Eliommod_sessiongroups. *)
+       (* First we close all browser sessions in the group, by
+          removing the group from its dlist: *)
+       Eliommod_sessiongroups.Data.remove_group fullbrowsersessgrp;
+       (* Then we remove data from group tables: *)
+       match Tuple3.thd fullbrowsersessgrp with
+       | Left key ->
+         (* iterate on all session data tables: *)
+         sitedata.Eliom_common.remove_session_data key
+       | _ ->
+         (* No group has been set. No group table.
+                   Data associated to default (automatic) groups
+                   is removed when closing associated sessions.
+         *)
+         ()
+    )
+    group_of_groups;
+  Eliommod_gc.service_session_gc sitedata;
+  Eliommod_gc.data_session_gc sitedata;
+  Eliommod_gc.persistent_session_gc sitedata;
+  sitedata
+
+let create_sitedata =
+  (* We want to keep the old site data even if we reload the server.
+     To do that, we keep the site data in a table *)
   let t = S.create 5 in
   fun host site_dir config_info ->
     let key = (host, site_dir) in
     try
       S.find t key
     with
-      | Not_found ->
-        let gog =
-          Ocsigen_cache.Dlist.create !default_max_volatile_groups_per_site
-        in
-        let sitedata =
-          let dlist_table = Eliom_common.create_dlist_ip_table 100 in
-          (* One dlist for each site? *)
-          {Eliom_common.servtimeout = None, None, [];
-           datatimeout =  None, None, [];
-           perstimeout =  None, None, [];
-           site_value_table = Polytables.create ();
-           site_dir = site_dir;
-(*VVV encode=false??? *)
-           site_dir_string = Url.string_of_url_path
-              ~encode:false site_dir;
-           config_info = config_info;
-           default_links_xhr = Eliom_common.tenable_value ~name:"default_links_xhr" true;
-           global_services =
-              Eliom_common.empty_tables
-                !default_max_anonymous_services_per_subnet
-                false;
-           registered_scope_hierarchies = Eliom_common.Hier_set.empty;
-           session_services = Eliommod_cookies.new_service_cookie_table ();
-           session_data = Eliommod_cookies.new_data_cookie_table ();
-           group_of_groups = gog;
-           remove_session_data = (fun cookie -> ());
-           not_bound_in_data_tables = (fun cookie -> true);
-           exn_handler = Eliommod_pagegen.def_handler;
-           unregistered_services = [];
-           unregistered_na_services = [];
-
-           max_service_sessions_per_group =
-  !default_max_service_sessions_per_group, false;
-
-           max_volatile_data_sessions_per_group =
-  !default_max_volatile_data_sessions_per_group, false;
-
-              max_persistent_data_sessions_per_group =
-Some !default_max_persistent_data_sessions_per_group, false;
-
-           max_service_tab_sessions_per_group =
-  !default_max_service_tab_sessions_per_group, false;
-
-           max_volatile_data_tab_sessions_per_group =
-  !default_max_volatile_data_tab_sessions_per_group, false;
-
-              max_persistent_data_tab_sessions_per_group =
-Some !default_max_persistent_data_tab_sessions_per_group, false;
-
-           max_service_sessions_per_subnet =
-  !default_max_service_sessions_per_subnet, false;
-
-           max_volatile_data_sessions_per_subnet =
-  !default_max_volatile_data_sessions_per_subnet, false;
-
-           max_anonymous_services_per_session =
-  !default_max_anonymous_services_per_session, false;
-
-           max_anonymous_services_per_subnet =
-  !default_max_anonymous_services_per_subnet, false;
-
-           secure_cookies = !default_secure_cookies;
-
-           dlist_ip_table = dlist_table;
-           ipv4mask = None, false;
-           ipv6mask = None, false;
-           application_script = !default_application_script;
-           cache_global_data = !default_cache_global_data;
-           html_content_type = !default_html_content_type;
-          }
-        in
-        Ocsigen_cache.Dlist.set_finaliser_after
-          (fun (node : [ `Session ] Eliom_common.sessgrp
-                  Ocsigen_cache.Dlist.node) ->
-            let fullbrowsersessgrp = Ocsigen_cache.Dlist.value node in
-            (* When removing a group from the dlist, we must close it.
-               Actually, it must be the only way to close a group *)
-            (* This finaliser is almost identical to the finaliser for
-               other groups, defined in Eliommod_sessiongroups *)
-            (* First we close all browser sessions in the group,
-               by removing the group from its dlist: *)
-            Eliommod_sessiongroups.Data.remove_group fullbrowsersessgrp;
-            (* Then we close all group tables: *)
-            let key = match Tuple3.thd fullbrowsersessgrp with
-              | Left a -> a
-              | _ -> Eliom_common.default_group_name
-            in
-            (* iterate on all session data tables: *)
-            sitedata.Eliom_common.remove_session_data key
-          )
-          gog;
-        Eliommod_gc.service_session_gc sitedata;
-        Eliommod_gc.data_session_gc sitedata;
-        Eliommod_gc.persistent_session_gc sitedata;
-        S.add t key sitedata;
-        sitedata
+    | Not_found ->
+       let sitedata = create_sitedata site_dir config_info in
+       S.add t key sitedata;
+       sitedata
 
 
 
@@ -223,7 +221,6 @@ Some !default_max_persistent_data_tab_sessions_per_group, false;
 
 (* The following is common to global config and site config *)
 let parse_eliom_option
-    globaloption
     (set_volatile_timeout,
      set_data_timeout,
      set_service_timeout,
@@ -244,7 +241,10 @@ let parse_eliom_option
      set_ipv6mask,
      set_application_script,
      set_global_data_caching,
-     set_html_content_type
+     set_html_content_type,
+     set_ignored_get_params,
+     set_ignored_post_params,
+     set_omitpersistentstorage
     )
     =
   let parse_timeout_attrs tn attrs =
@@ -257,7 +257,7 @@ let parse_eliom_option
       | ("level", "clientprocess")::l
       | ("level", "process")::l
       | ("level", "tab")::l -> aux (v, sn, `Client_process) l
-      | ("level", _)::l ->
+      | ("level", _)::_ ->
           raise
             (Error_in_config_file
                ("Eliom: Wrong attribute value for level in "^tn^" tag"))
@@ -280,17 +280,15 @@ let parse_eliom_option
               (Error_in_config_file
                  ("Eliom: Wrong attribute value for "^tn^" tag"))
     in
-    if (not globaloption) || sn = None
-    then
-      let sn = match sn with
-        | None -> None (* all hierarchies *)
-        | Some "" -> Some None (* default hierarchy *)
-        | c -> Some c
-      in (a, sn, ct)
-    else
-      raise
-        (Error_in_config_file
-           ("Eliom: hierarchyname attribute not allowed for "^tn^" tag in global configuration"))
+    let parse_scope_hierarchy = function
+      | "" -> Eliom_common_base.Default_ref_hier
+      | s when String.lowercase_ascii s = "default" ->
+        Eliom_common_base.Default_ref_hier
+      | s when String.lowercase_ascii s = "comet" ->
+        Eliom_common_base.Default_comet_hier
+      | s -> (Eliom_common_base.User_hier s)
+    in
+    (a, Eliom_lib.Option.map parse_scope_hierarchy sn, ct)
   in
   let convert_attr tag f v =
     try
@@ -492,6 +490,28 @@ let parse_eliom_option
   | (Xml.Element ("htmlcontenttype", [("value", v)], [])) ->
     set_html_content_type v
 
+  | (Xml.Element ("ignoredgetparams", [("regexp", v)], [])) ->
+    let re = Re.seq [Re.start; Re.Pcre.re v; Re.stop] |> Re.compile in
+    set_ignored_get_params (v, re)
+
+  | (Xml.Element ("ignoredpostparams", [("regexp", v)], [])) ->
+    let re = Re.seq [Re.start; Re.Pcre.re v; Re.stop] |> Re.compile in
+    set_ignored_post_params (v, re)
+
+  | (Xml.Element ("omitpersistentstorage", attrs, tags)) ->
+      assert (attrs = []);
+      let parse_rule = function
+        | Xml.Element ("header", attrs, tags) ->
+            assert (tags = []);
+            let attr_name, attr_value = match attrs with [a] -> a | _ -> assert false in
+            let header_name = Ocsigen_header.Name.of_string attr_name in
+            let header_regexp = Re.compile @@ Re.Pcre.re attr_value in
+            Eliom_common.HeaderRule (header_name, header_regexp)
+        | _ -> assert false
+      in
+      let rules = List.map parse_rule tags in
+      set_omitpersistentstorage (Some rules)
+
   | (Xml.Element (s, _, _)) ->
       raise (Error_in_config_file
                ("Unexpected content <"^s^"> inside eliom config"))
@@ -503,7 +523,7 @@ let parse_eliom_options f l =
     | [] -> rest
     | e::l ->
         try
-          parse_eliom_option false f e;
+          parse_eliom_option f e;
           aux rest l
         with Error_in_config_file _ -> aux (e::rest) l
   in List.rev (aux [] l)
@@ -514,7 +534,7 @@ let parse_eliom_options f l =
 
 let rec parse_global_config = function
   | [] -> ()
-  | (Xml.Element ("sessiongcfrequency", [("value", s)], p))::ll ->
+  | (Xml.Element ("sessiongcfrequency", [("value", s)], _))::ll ->
       (try
         let t = float_of_string s in
         Eliommod_gc.set_servicesessiongcfrequency (Some t);
@@ -528,7 +548,7 @@ let rec parse_global_config = function
         else raise (Error_in_config_file
                       "Eliom: Wrong value for <sessiongcfrequency>"));
       parse_global_config ll
-  | (Xml.Element ("servicesessiongcfrequency", [("value", s)], p))::ll ->
+  | (Xml.Element ("servicesessiongcfrequency", [("value", s)], _))::ll ->
       (try
         Eliommod_gc.set_servicesessiongcfrequency (Some (float_of_string s))
       with Failure _ ->
@@ -537,7 +557,7 @@ let rec parse_global_config = function
         else raise (Error_in_config_file
                       "Eliom: Wrong value for <servicesessiongcfrequency>"));
       parse_global_config ll
-  | (Xml.Element ("datasessiongcfrequency", [("value", s)], p))::ll ->
+  | (Xml.Element ("datasessiongcfrequency", [("value", s)], _))::ll ->
       (try
         Eliommod_gc.set_datasessiongcfrequency (Some (float_of_string s))
       with Failure _ ->
@@ -547,7 +567,7 @@ let rec parse_global_config = function
                       "Eliom: Wrong value for <datasessiongcfrequency>"));
       parse_global_config ll
   | (Xml.Element ("persistentsessiongcfrequency",
-              [("value", s)], p))::ll ->
+              [("value", s)], _))::ll ->
                 (try
                   Eliommod_gc.set_persistentsessiongcfrequency
                     (Some (float_of_string s))
@@ -560,13 +580,12 @@ let rec parse_global_config = function
                 parse_global_config ll
   | e::ll ->
       parse_eliom_option
-        true
-        ((fun ct _ m ->
-           Eliommod_timeouts.set_default `Data ct m;
-           Eliommod_timeouts.set_default `Service ct m),
-         (fun ct _ -> Eliommod_timeouts.set_default `Data ct),
-         (fun ct _ -> Eliommod_timeouts.set_default `Service ct),
-         (fun ct _ -> Eliommod_timeouts.set_default `Persistent ct),
+        ((fun ct sh m ->
+           Eliommod_timeouts.set_default ?scope_hierarchy:sh `Data ct m;
+           Eliommod_timeouts.set_default ?scope_hierarchy:sh `Service ct m),
+         (fun ct sh -> Eliommod_timeouts.set_default ?scope_hierarchy:sh `Data ct),
+         (fun ct sh -> Eliommod_timeouts.set_default ?scope_hierarchy:sh `Service ct),
+         (fun ct sh -> Eliommod_timeouts.set_default ?scope_hierarchy:sh `Persistent ct),
          (fun v -> default_max_service_sessions_per_group := v),
          (fun v -> default_max_service_sessions_per_subnet := v),
          (fun v -> default_max_volatile_data_sessions_per_group := v),
@@ -583,7 +602,12 @@ let rec parse_global_config = function
          (fun v -> Eliom_common.ipv6mask := v),
          (fun v -> default_application_script := v),
          (fun v -> default_cache_global_data := v),
-         (fun v -> default_html_content_type := Some v)
+         (fun v -> default_html_content_type := Some v),
+         (fun regexp -> default_ignored_get_params :=
+                          regexp::!default_ignored_get_params),
+         (fun regexp -> default_ignored_post_params :=
+                          regexp::!default_ignored_post_params),
+         (fun v -> default_omitpersistentstorage := v)
         )
         e;
       parse_global_config ll
@@ -698,15 +722,23 @@ let config_in_tag = ref "" (* the parent tag of the currently handled tag *)
 
 type module_to_load = Files of string list | Name of string
 
-let load_eliom_module sitedata cmo_or_name parent_tag content =
+let site_init firstmodule =
+  if !firstmodule
+  then begin
+      Eliom_common.begin_load_eliom_module ();
+      (* I want to be able to define global client values during that phase: *)
+      Eliom_syntax.set_global true;
+      List.iter (fun f -> f ()) !site_init_ref;
+      Eliom_syntax.set_global false;
+      firstmodule := false;
+      Eliom_common.end_load_eliom_module ()
+    end
+
+let load_eliom_module _sitedata cmo_or_name parent_tag content =
   let preload () =
     config := content;
     config_in_tag := parent_tag;
-    Eliom_common.begin_load_eliom_module ();
-    (* I want to be able to define global client values during that phase: *)
-    Eliom_syntax.set_global true;
-    List.iter (fun f -> f ()) !site_init_ref;
-    Eliom_syntax.set_global false
+    Eliom_common.begin_load_eliom_module ()
   in
   let postload () =
     Eliom_common.end_load_eliom_module ();
@@ -739,13 +771,14 @@ let gen_nothing () _ = Lwt.return Ocsigen_extensions.Ext_do_nothing
 let default_module_action _ = failwith "default_module_action"
 
 (** Parsing of config file for each site: *)
-let parse_config hostpattern conf_info site_dir =
+let parse_config _ hostpattern conf_info site_dir =
 (*--- if we put the following line here: *)
-  let sitedata = new_sitedata hostpattern site_dir conf_info in
+  let sitedata = create_sitedata hostpattern site_dir conf_info in
 (*--- then there is one service tree for each <site> *)
 (*--- (mutatis mutandis for the following line:) *)
   Eliom_common.absolute_change_sitedata sitedata;
   let firsteliomtag = ref true in
+  let firstmodule = ref true in
   let eliommodulewarningdisplayed = ref false in
   let rec parse_default_links_xhr atts default_links_xhr = function
     | [] -> default_links_xhr, List.rev atts
@@ -790,13 +823,14 @@ let parse_config hostpattern conf_info site_dir =
     | (s, _)::_ ->
         raise
           (Error_in_config_file ("Wrong attribute for <eliom>: "^s))
-  in fun _ parse_site -> function
+  in fun _ _parse_site -> function
     | Xml.Element ("eliommodule", atts, content) ->
       Eliom_extension.register_eliom_extension
         default_module_action;
       (match parse_module_attrs None atts with
         | Some file_or_name ->
           exception_during_eliommodule_loading := true;
+          site_init firstmodule;
           load_eliom_module sitedata file_or_name "eliommodule" content;
           exception_during_eliommodule_loading := false
         | _ -> ());
@@ -820,17 +854,8 @@ let parse_config hostpattern conf_info site_dir =
                          bool (* from config file *) ->
                          Eliom_common.sitedata ->
                          float option -> unit)
-            cookie_type state_hier_oo v =
+            cookie_type state_hier v =
           let make_full_st_name secure state_hier =
-            let state_hier : Eliom_common.scope_hierarchy =
-              match state_hier with
-              | None -> Eliom_common_base.Default_ref_hier
-              | Some s when String.lowercase_ascii s = "default" ->
-                Eliom_common_base.Default_ref_hier
-              | Some s when String.lowercase_ascii s = "comet" ->
-                Eliom_common_base.Default_comet_hier
-              | Some s -> Eliom_common_base.User_hier s
-            in
             let scope =
               match cookie_type with
               | `Session -> `Session state_hier
@@ -844,7 +869,7 @@ let parse_config hostpattern conf_info site_dir =
           (*VVV We set timeout for both secure and unsecure states.
             Make possible to customize this? *)
           f
-            ?full_st_name:(Option.map (make_full_st_name false) state_hier_oo)
+            ?full_st_name:(Option.map (make_full_st_name false) state_hier)
             ?cookie_level:(Some cookie_type)
             ~recompute_expdates:false
             true
@@ -852,7 +877,7 @@ let parse_config hostpattern conf_info site_dir =
             sitedata
             v;
           f
-            ?full_st_name:(Option.map (make_full_st_name true) state_hier_oo)
+            ?full_st_name:(Option.map (make_full_st_name true) state_hier)
             ?cookie_level:(Some cookie_type)
             ~recompute_expdates:false
             true
@@ -908,7 +933,12 @@ let parse_config hostpattern conf_info site_dir =
              (fun v -> sitedata.Eliom_common.ipv6mask <- Some v, true),
              (fun v -> sitedata.Eliom_common.application_script <- v),
              (fun v -> sitedata.Eliom_common.cache_global_data <- v),
-             (fun v -> sitedata.Eliom_common.html_content_type <- Some v)
+             (fun v -> sitedata.Eliom_common.html_content_type <- Some v),
+             (fun regexp -> sitedata.Eliom_common.ignored_get_params
+                            <- regexp::sitedata.Eliom_common.ignored_get_params),
+             (fun regexp -> sitedata.Eliom_common.ignored_post_params
+                            <- regexp::sitedata.Eliom_common.ignored_post_params),
+             (fun v -> sitedata.Eliom_common.omitpersistentstorage <- v)
             )
             content
         in
@@ -917,11 +947,11 @@ let parse_config hostpattern conf_info site_dir =
            | Some default_links_xhr ->
              sitedata.Eliom_common.default_links_xhr#set ~override_tenable:true default_links_xhr
            | None -> ());
-        Eliom_extension.register_eliom_extension
-          default_module_action;
+        Eliom_extension.register_eliom_extension default_module_action;
         (match parse_module_attrs None atts with
           | Some file_or_name ->
             exception_during_eliommodule_loading := true;
+            site_init firstmodule;
             load_eliom_module sitedata file_or_name "eliom" content;
             exception_during_eliommodule_loading := false
           | _ -> ());
@@ -956,7 +986,7 @@ let parse_config hostpattern conf_info site_dir =
 (*****************************************************************************)
 (** extension registration *)
 let () =
-  register_extension
+  Ocsigen_extensions.register
     ~name:"eliom"
     ~fun_site:parse_config
     ~end_init
