@@ -21,14 +21,102 @@
 (** Cookie management                                                       *)
 open Eliom_lib
 open Lwt
-open Ocsigen_cookies
 
 include Eliom_cookies_base
 
 (*****************************************************************************)
-let make_new_session_id = Ocsigen_lib.make_cryptographic_safe_string
+let make_new_session_id () =
+  Ocsigen_lib.make_cryptographic_safe_string () ^ "H"
 
 
+type date = float
+
+type cookie = {
+  full_state_name : Eliom_common.full_state_name;
+  expiry : date option;
+  timeout : Eliom_common.timeout;
+  session_group : Eliom_common.perssessgrp option
+}
+
+module Persistent_cookies = struct
+  (* Another table, containing the session info for each cookie *)
+  (* the table contains:
+     - the expiration date (by timeout), changed at each access to the table
+       (float option) None -> no expiration
+     - the timeout for the user (float option option) None -> see global config
+       Some None -> no timeout
+   *)
+  (* It is lazy, because we must delay the creation of the table until
+     the initialization of eliom in case we use static linking with
+     sqlite backend ... *)
+
+  module Ocsipersist = Eliom_common.Ocsipersist.Functorial
+
+  (* NOTE: Do not forget to change the version number when the internal format changes! *)
+  let persistent_cookie_table_version = "_v5"
+  (* v2 introduces session groups *)
+  (* v3 introduces tab sessions *)
+  (* v4 introduces group tables *)
+  (* v5 removes secure scopes *)
+  module Cookies =
+    Ocsipersist.Table
+      (struct let name = "eliom_persist_cookies"^persistent_cookie_table_version end)
+      (Ocsipersist.Column.String)
+      (Ocsipersist.Column.Marshal (struct type t = cookie end))
+
+  let () = Eliom_common.Persistent_tables.add_functorial_table (module Cookies)
+
+  (* maps expiry dates to cookie IDs; may have superfluous entries, i.e cookies
+     that will not actually expire on the given date. *)
+  module Expiry_dates = struct
+    include Ocsipersist.Table
+              (struct let name = "eliom_persist_cookies_expiry_dates" end)
+              (Ocsipersist.Column.Float)
+              (Ocsipersist.Column.String)
+
+    let add_cookie exp cookie =
+      modify_opt exp @@
+        function
+          | None -> Some cookie
+          | Some cookies_str ->
+              let cookies = String.split_on_char ',' cookies_str in
+              if List.mem cookie cookies
+              then Some cookies_str
+              else Some (cookies_str ^ "," ^ cookie)
+
+    let remove_cookie exp_o cookie =
+      exp_o |> Eliom_lib.Option.Lwt.iter @@ fun exp ->
+      modify_opt exp @@
+        function
+          | None -> None
+          | Some cookies_str ->
+              let cookies = String.split_on_char ',' cookies_str in
+              let cookies' = List.filter (fun c -> c <> cookie) cookies in
+              if cookies' = []
+                then None
+                else Some (String.concat "," cookies')
+  end
+
+  let add cookie ({expiry} as content) =
+    Eliom_lib.Option.Lwt.iter (fun t -> Expiry_dates.add_cookie t cookie) expiry
+    >>= fun _ ->
+    Cookies.add cookie content
+
+  let replace_if_exists cookie ({expiry} as content) =
+    Eliom_lib.Option.Lwt.iter (fun t -> Expiry_dates.add_cookie t cookie) expiry
+    >>= fun _ ->
+    Cookies.replace_if_exists cookie content
+
+  let garbage_collect ~section gc_cookie =
+    let now = Unix.time () in
+    Expiry_dates.iter ~lt:now @@ fun date cookies_str ->
+      let cookies = String.split_on_char ',' cookies_str in
+      let cookies_log = String.concat "," @@ List.map Eliom_common.Hashed_cookies.sha256 cookies in
+      Lwt_log.ign_info_f ~section "potentially expired cookies %.0f: %s"
+                                  date cookies_log;
+      Lwt_list.iter_s gc_cookie cookies >>= fun _ ->
+      Expiry_dates.remove date
+end
 
 
 (*****************************************************************************)
@@ -50,15 +138,18 @@ let get_cookie_info
     Eliom_common.Full_state_name_table.fold
       (fun name value (oktable, failedlist) ->
         try
-          let full_state_name, ta, expref, timeout_ref, sessgrpref, sessgrpnode =
+          let hvalue = Eliom_common.Hashed_cookies.hash value in
+          let {Eliom_common.Service_cookie.session_table; expiry; timeout;
+                                           session_group; session_group_node} =
             Eliom_common.SessionCookies.find
-              sitedata.Eliom_common.session_services value
+              sitedata.Eliom_common.session_services
+              (Eliom_common.Hashed_cookies.to_string hvalue)
           in
-          Eliommod_sessiongroups.Serv.up sessgrpnode;
-          match !expref with
+          Eliommod_sessiongroups.Serv.up session_group_node;
+          match !expiry with
           | Some t when t < now ->
               (* session expired by timeout *)
-              Eliommod_sessiongroups.Serv.remove sessgrpnode;
+              Eliommod_sessiongroups.Serv.remove session_group_node;
               ((Eliom_common.Full_state_name_table.add
                   name
                   (Some value          (* value sent by the browser *),
@@ -72,24 +163,25 @@ let get_cookie_info
                      (Some value        (* value sent by the browser *),
                       ref
                         (Eliom_common.SC
-                           {Eliom_common.sc_value= value  (* value *);
-                            Eliom_common.sc_table= ref ta (* the table of session services *);
-                            Eliom_common.sc_timeout= timeout_ref (* user timeout ref *);
-                            Eliom_common.sc_exp= expref  (* expiration date (server side) *);
+                           {Eliom_common.sc_hvalue= hvalue  (* value *);
+                            Eliom_common.sc_set_value= None;
+                            Eliom_common.sc_table= ref session_table;
+                            Eliom_common.sc_timeout= timeout;
+                            Eliom_common.sc_exp= expiry;
                             Eliom_common.sc_cookie_exp=
                             ref Eliom_common.CENothing
                               (* cookie expiration date to send
                                  to the browser.
                                  We don't change it *);
-                            Eliom_common.sc_session_group= sessgrpref;
-                            Eliom_common.sc_session_group_node= sessgrpnode;
+                            Eliom_common.sc_session_group= session_group;
+                            Eliom_common.sc_session_group_node= session_group_node;
                            }))
                      oktable),
                   failedlist)
         with Not_found ->
           ((Eliom_common.Full_state_name_table.add
               name
-              (Some value                 (* value sent by the browser *),
+              (Some value              (* value sent by the browser *),
                ref Eliom_common.SCData_session_expired
                                           (* ask the browser
                                              to remove the cookie *))
@@ -106,15 +198,17 @@ let get_cookie_info
       (fun value ->
         lazy
           (try
-            let full_state_name, expref, timeout_ref, sessgrpref, sessgrpnode =
+            let hvalue = Eliom_common.Hashed_cookies.hash value in
+            let {Eliom_common.Data_cookie.expiry; timeout; session_group; session_group_node} =
               Eliom_common.SessionCookies.find
-                sitedata.Eliom_common.session_data value
+                sitedata.Eliom_common.session_data
+                (Eliom_common.Hashed_cookies.to_string hvalue)
             in
-            Eliommod_sessiongroups.Serv.up sessgrpnode;
-            match !expref with
+            Eliommod_sessiongroups.Serv.up session_group_node;
+            match !expiry with
               | Some t when t < now ->
                   (* session expired by timeout *)
-                  Eliommod_sessiongroups.Data.remove sessgrpnode;
+                  Eliommod_sessiongroups.Data.remove session_group_node;
                   (Some value                 (* value sent by the browser *),
                    ref Eliom_common.SCData_session_expired
                                               (* ask the browser
@@ -123,18 +217,19 @@ let get_cookie_info
                   (Some value        (* value sent by the browser *),
                    ref
                      (Eliom_common.SC
-                        {Eliom_common.dc_value= value       (* value *);
-                         Eliom_common.dc_timeout= timeout_ref
+                        {Eliom_common.dc_hvalue= hvalue       (* value *);
+                         Eliom_common.dc_set_value= None;
+                         Eliom_common.dc_timeout= timeout
                           (* user timeout ref *);
-                         Eliom_common.dc_exp= expref (* expiration date
+                         Eliom_common.dc_exp= expiry (* expiration date
                                                         (server side) *);
                          Eliom_common.dc_cookie_exp=
                             ref Eliom_common.CENothing
                               (* cookie expiration date to send
                                  to the browser.
                                  We don't change it *);
-                         Eliom_common.dc_session_group= sessgrpref;
-                         Eliom_common.dc_session_group_node= sessgrpnode}
+                         Eliom_common.dc_session_group= session_group;
+                         Eliom_common.dc_session_group_node= session_group_node}
                      )
                   )
            with Not_found ->
@@ -152,16 +247,21 @@ let get_cookie_info
         lazy
           (catch
              (fun () ->
-               Lazy.force Eliom_common.persistent_cookies_table >>= fun table ->
-               Ocsipersist.find table value >>=
-               fun (full_state_name, persexp, perstimeout, sessgrp) ->
+               let hvalue = Eliom_common.Hashed_cookies.hash value in
+               let hvalue_string =
+                 Eliom_common.Hashed_cookies.to_string hvalue in
+               Persistent_cookies.Cookies.find
+                 (Eliom_common.Hashed_cookies.to_string hvalue)
+               >>=
+               fun {expiry = persexp; timeout = perstimeout; session_group = sessgrp} ->
 
-                 Eliommod_sessiongroups.Pers.up value sessgrp >>= fun () ->
+               Eliommod_sessiongroups.Pers.up hvalue_string sessgrp
+               >>= fun () ->
                  match persexp with
                  | Some t when t < now ->
                      (* session expired by timeout *)
-                     Eliom_common.remove_from_all_persistent_tables
-                       value >>= fun () ->
+                     Eliom_common.Persistent_tables.remove_key_from_all_tables
+                       hvalue_string >>= fun () ->
                        return
                          (Some (value         (* value at the beginning
                                                  of the request *),
@@ -188,8 +288,9 @@ let get_cookie_info
                               sessgrp      (* session group at beginning *)),
                         (ref
                            (Eliom_common.SC
-                              {Eliom_common.pc_value= value
+                              {Eliom_common.pc_hvalue= hvalue
                                  (* value *);
+                               Eliom_common.pc_set_value= None;
                                Eliom_common.pc_timeout= ref perstimeout
                                  (* user persistent timeout ref *);
                                Eliom_common.pc_cookie_exp=
@@ -243,10 +344,10 @@ let get_cookie_info
 
 (* table cookie -> session table *)
 let new_service_cookie_table () :
-    Eliom_common.tables Eliom_common.servicecookiestable =
+    Eliom_common.tables Eliom_common.Service_cookie.table =
   Eliom_common.SessionCookies.create 100
 
-let new_data_cookie_table () : Eliom_common.datacookiestable =
+let new_data_cookie_table () : Eliom_common.Data_cookie.table =
   Eliom_common.SessionCookies.create 100
 
 
@@ -260,7 +361,7 @@ let compute_session_cookies_to_send
     sitedata
     ((service_cookie_info,
       data_cookie_info,
-      pers_cookies_info), secure_ci) (endlist: Ocsigen_cookies.cookieset) =
+      pers_cookies_info), secure_ci) (endlist: Ocsigen_cookie_map.t) =
   let getservvexp (old, newi) =
     return
       (let newinfo =
@@ -268,7 +369,8 @@ let compute_session_cookies_to_send
         | Eliom_common.SCNo_data
         | Eliom_common.SCData_session_expired -> None
         | Eliom_common.SC c ->
-            Some (c.Eliom_common.sc_value,
+            Some (c.Eliom_common.sc_hvalue,
+                  c.Eliom_common.sc_set_value,
                   !(c.Eliom_common.sc_cookie_exp))
       in (old, newinfo))
   in
@@ -282,7 +384,8 @@ let compute_session_cookies_to_send
           | Eliom_common.SCNo_data
           | Eliom_common.SCData_session_expired -> None
           | Eliom_common.SC c ->
-              Some (c.Eliom_common.dc_value,
+              Some (c.Eliom_common.dc_hvalue,
+                    c.Eliom_common.dc_set_value,
                     !(c.Eliom_common.dc_cookie_exp))
         in (old, newinfo))
     else fail Not_found
@@ -302,7 +405,8 @@ let compute_session_cookies_to_send
              | Eliom_common.SCNo_data
              | Eliom_common.SCData_session_expired -> None
              | Eliom_common.SC c ->
-                 Some (c.Eliom_common.pc_value,
+                 Some (c.Eliom_common.pc_hvalue,
+                       c.Eliom_common.pc_set_value,
                        !(c.Eliom_common.pc_cookie_exp))
          in (oldinfo, newinfo))
     else fail Not_found
@@ -324,29 +428,34 @@ let compute_session_cookies_to_send
               (match old, newc with
                 | None, None -> beg
                 | Some _, None ->
-                  Ocsigen_cookies.add_cookie
+                  Ocsigen_cookie_map.add
                     sitedata.Eliom_common.site_dir
-                    (Eliom_common.make_full_cookie_name cookiekind full_st_name)
-                    Ocsigen_cookies.OUnset
+                    (Eliom_common.make_full_cookie_name
+                       cookiekind full_st_name)
+                    OUnset
                     beg
               (* the path is always site_dir because the cookie cannot
                  have been unset by a service outside
                  this site directory *)
-                | None, Some (v, exp) ->
-                  Ocsigen_cookies.add_cookie
+                | _, Some (_, Some v, exp) -> (* New value *)
+                  Ocsigen_cookie_map.add
                     sitedata.Eliom_common.site_dir
-                    (Eliom_common.make_full_cookie_name cookiekind full_st_name)
-                    (Ocsigen_cookies.OSet (ch_exp exp, v, secure))
+                    (Eliom_common.make_full_cookie_name
+                       cookiekind full_st_name)
+                    (OSet (ch_exp exp, v, secure))
                     beg
-                | Some oldv, Some (newv, exp) ->
-                  if exp = Eliom_common.CENothing && oldv = newv
+                | Some oldv, Some (_, None, exp) ->
+                  if exp = Eliom_common.CENothing
                   then beg
                   else
-                    Ocsigen_cookies.add_cookie
+                    Ocsigen_cookie_map.add
                       sitedata.Eliom_common.site_dir
-                      (Eliom_common.make_full_cookie_name cookiekind full_st_name)
-                      (Ocsigen_cookies.OSet (ch_exp exp, newv, secure))
+                      (Eliom_common.make_full_cookie_name
+                         cookiekind full_st_name)
+                      (OSet (ch_exp exp, oldv, secure))
                       beg
+                | None, Some (_, None, _) -> (* Should not happen *)
+                   beg
               )
           )
           (function
@@ -380,27 +489,27 @@ let compute_new_ri_cookies'
     ricookies
     cookies_set_by_page =
 
-  Ocsigen_cookies.Cookies.fold
+  Ocsigen_cookie_map.Map_path.fold
     (fun cpath t cookies ->
       if Url.is_prefix_skip_end_slash
           (Url.remove_slash_at_beginning cpath)
           (Url.remove_slash_at_beginning ripath)
       then
-        CookiesTable.fold
+        Ocsigen_cookie_map.Map_inner.fold
           (fun name v cookies ->
 (*VVV We always keep secure cookies, event if the protocol is not secure,
   because this function is for actions only. Is that right? *)
             match v with
-              | Ocsigen_cookies.OSet (Some exp, value, secure)
+              | OSet (Some exp, value, _secure)
                   when exp>now ->
-                CookiesTable.add name value cookies
-              | Ocsigen_cookies.OSet (None, value, secure) ->
-                CookiesTable.add name value cookies
-              | Ocsigen_cookies.OSet (Some exp, value, secure)
+                Ocsigen_cookie_map.Map_inner.add name value cookies
+              | OSet (None, value, _secure) ->
+                Ocsigen_cookie_map.Map_inner.add name value cookies
+              | OSet (Some exp, _value, _secure)
                   when exp<=now ->
-                CookiesTable.remove name cookies
-              | Ocsigen_cookies.OUnset ->
-                CookiesTable.remove name cookies
+                Ocsigen_cookie_map.Map_inner.remove name cookies
+              | OUnset ->
+                Ocsigen_cookie_map.Map_inner.remove name cookies
               | _ -> cookies)
           t
           cookies
@@ -415,20 +524,20 @@ let compute_new_ri_cookies'
 let compute_new_ri_cookies
     (now : float)
     (ripath : string list)
-    (ricookies : string CookiesTable.t)
+    (ricookies : string Ocsigen_cookie_map.Map_inner.t)
     ((ci, secure_ci) : Eliom_common.tables Eliom_common.cookie_info)
-    (cookies_set_by_page : Ocsigen_cookies.cookieset)
-    : string CookiesTable.t Lwt.t =
+    (cookies_set_by_page : Ocsigen_cookie_map.t)
+    : string Ocsigen_cookie_map.Map_inner.t Lwt.t =
 
   (* first we add cookies set by page: *)
   let ric =
     compute_new_ri_cookies' now ripath ricookies cookies_set_by_page
   in
   (* then session cookies: *)
-  let f secure (service_cookie_info, data_cookie_info, pers_cookie_info) ric =
+  let f _secure (service_cookie_info, data_cookie_info, pers_cookie_info) ric =
     let ric =
       Eliom_common.Full_state_name_table.fold
-        (fun ((sc, _, _) as full_st_name) (_, v) beg ->
+        (fun ({Eliom_common.user_scope = sc} as full_st_name) (_, v) beg ->
           let ct = Eliom_common.cookie_level_of_user_scope sc in
           if ct = `Client_process
           then beg
@@ -438,17 +547,18 @@ let compute_new_ri_cookies
             match !v with
               | Eliom_common.SCData_session_expired
               | Eliom_common.SCNo_data ->
-                CookiesTable.remove n beg
-              | Eliom_common.SC c ->
-                CookiesTable.add
-                  n c.Eliom_common.sc_value beg
+                Ocsigen_cookie_map.Map_inner.remove n beg
+              | Eliom_common.SC {Eliom_common.sc_set_value = Some v} ->
+                Ocsigen_cookie_map.Map_inner.add n v beg
+              | Eliom_common.SC {Eliom_common.sc_set_value = None} ->
+                beg
         )
         !service_cookie_info
         ric
     in
     let ric =
       Eliom_common.Full_state_name_table.fold
-        (fun ((sc, _, _) as full_st_name) v beg ->
+        (fun ({Eliom_common.user_scope = sc} as full_st_name) v beg ->
           let ct = Eliom_common.cookie_level_of_user_scope sc in
           if ct = `Client_process
           then beg
@@ -461,10 +571,11 @@ let compute_new_ri_cookies
               match !v with
                 | Eliom_common.SCData_session_expired
                 | Eliom_common.SCNo_data ->
-                  CookiesTable.remove n beg
-                | Eliom_common.SC c ->
-                  CookiesTable.add
-                    n c.Eliom_common.dc_value beg
+                  Ocsigen_cookie_map.Map_inner.remove n beg
+                | Eliom_common.SC {Eliom_common.dc_set_value = Some v} ->
+                  Ocsigen_cookie_map.Map_inner.add n v beg
+                | Eliom_common.SC {Eliom_common.dc_set_value = None} ->
+                  beg
             else beg
         )
         !data_cookie_info
@@ -472,7 +583,7 @@ let compute_new_ri_cookies
     in
     let ric =
       Eliom_common.Full_state_name_table.fold
-        (fun ((sc, _, _) as full_st_name) v beg ->
+        (fun ({Eliom_common.user_scope = sc} as full_st_name) v beg ->
           let ct = Eliom_common.cookie_level_of_user_scope sc in
           if ct = `Client_process
           then beg
@@ -486,10 +597,11 @@ let compute_new_ri_cookies
                match !v with
                  | Eliom_common.SCData_session_expired
                  | Eliom_common.SCNo_data ->
-                   Lwt.return (CookiesTable.remove n beg)
-                 | Eliom_common.SC c ->
-                   Lwt.return (CookiesTable.add
-                                 n c.Eliom_common.pc_value beg))
+                   Lwt.return (Ocsigen_cookie_map.Map_inner.remove n beg)
+                 | Eliom_common.SC {Eliom_common.pc_set_value = Some v} ->
+                   Lwt.return (Ocsigen_cookie_map.Map_inner.add n v beg)
+                 | Eliom_common.SC {Eliom_common.pc_set_value = None} ->
+                   Lwt.return beg)
             else return beg
         )
         !pers_cookie_info
