@@ -18,9 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-open Ocsigen_cookies
-
-val cookieset_to_json : Ocsigen_cookies.cookieset -> string
+val cookieset_to_json : Ocsigen_cookie_map.t -> string
 
 val make_new_session_id : unit -> string
 
@@ -37,21 +35,44 @@ val get_cookie_info :
   Eliom_common.Full_state_name_table.key list
 
 val new_service_cookie_table :
-  unit -> Eliom_common.tables Eliom_common.servicecookiestable
-val new_data_cookie_table : unit -> Eliom_common.datacookiestable
+  unit -> Eliom_common.tables Eliom_common.Service_cookie.table
+val new_data_cookie_table : unit -> Eliom_common.Data_cookie.table
 val compute_session_cookies_to_send :
   Eliom_common.sitedata ->
   Eliom_common.tables Eliom_common.cookie_info ->
-  Ocsigen_cookies.cookieset ->
-  Ocsigen_cookies.cookieset Lwt.t
+  Ocsigen_cookie_map.t ->
+  Ocsigen_cookie_map.t Lwt.t
 val compute_cookies_to_send :
   Eliom_common.sitedata ->
   Eliom_common.tables Eliom_common.cookie_info ->
-  Ocsigen_cookies.cookieset ->
-  Ocsigen_cookies.cookieset Lwt.t
+  Ocsigen_cookie_map.t ->
+  Ocsigen_cookie_map.t Lwt.t
 val compute_new_ri_cookies :
   float ->
   string list ->
-  string CookiesTable.t ->
+  string Ocsigen_cookie_map.Map_inner.t ->
   Eliom_common.tables Eliom_common.cookie_info ->
-  Ocsigen_cookies.cookieset -> string CookiesTable.t Lwt.t
+  Ocsigen_cookie_map.t -> string Ocsigen_cookie_map.Map_inner.t Lwt.t
+
+type date = float
+
+type cookie = {
+  full_state_name : Eliom_common.full_state_name;
+  expiry : date option;
+  timeout : Eliom_common.timeout;
+  session_group : Eliom_common.perssessgrp option
+}
+
+module Persistent_cookies : sig
+  module Cookies : Ocsipersist.TABLE
+    with type key = string and type value = cookie
+  module Expiry_dates : sig
+    include Ocsipersist.TABLE with type key = date and type value = string
+    val add_cookie : date -> string -> unit Lwt.t
+    val remove_cookie : date option -> string -> unit Lwt.t
+  end
+  val add : string -> cookie -> unit Lwt.t
+  val replace_if_exists : string -> cookie -> unit Lwt.t
+  val garbage_collect :
+    section:Lwt_log_core.Section.t -> (Cookies.key -> unit Lwt.t) -> unit Lwt.t
+end
