@@ -114,35 +114,30 @@ let rec find_or_create_data_cookie ?set_session_group
               ~sp
               ()
             in
-            Some v.Eliom_common.dc_value
+            Some Eliom_common.(Hashed_cookies.to_string v.dc_hvalue)
           end
         | _ -> set_session_group
     in
     let fullsessgrp = fullsessgrp ~cookie_level ~sp set_session_group in
 
-    let rec aux () =
-      let c = Eliommod_cookies.make_new_session_id () in
-      (* Just to be sure it is not already used.
-         Actually not needed for the cookies we use *)
-      if Eliom_common.SessionCookies.mem table c
-      then aux ()
-      else c
-    in
-    let c = aux () in
+    let c = Eliommod_cookies.make_new_session_id () in
+    let hc = Eliom_common.Hashed_cookies.hash c in
+    let hc_string = Eliom_common.Hashed_cookies.to_string hc in
     let usertimeout = ref Eliom_common.TGlobal (* See global table *) in
     let serverexp = ref None (* Some 0. *) (* None = never. We'll change it later. *) in
     let fullsessgrpref = ref fullsessgrp in
-    let node = Eliommod_sessiongroups.Data.add sitedata c fullsessgrp in
+    let node = Eliommod_sessiongroups.Data.add sitedata hc_string fullsessgrp in
     Eliom_common.SessionCookies.replace
       (* actually it will add the cookie *)
       table
-      c
-      (full_st_name,
-       serverexp (* exp on server *),
-       usertimeout,
-       fullsessgrpref,
-       node);
-    {Eliom_common.dc_value= c;
+      hc_string
+      {Eliom_common.Data_cookie.full_state_name = full_st_name;
+       expiry = serverexp;
+       timeout = usertimeout;
+       session_group = fullsessgrpref;
+       session_group_node = node};
+    {Eliom_common.dc_hvalue= hc;
+     Eliom_common.dc_set_value= Some c;
      Eliom_common.dc_timeout= usertimeout;
      Eliom_common.dc_exp= serverexp;
      Eliom_common.dc_cookie_exp=
@@ -163,7 +158,7 @@ let rec find_or_create_data_cookie ?set_session_group
   let full_st_name =
     Eliom_common.make_full_state_name ~sp ~secure ~scope:cookie_scope in
   try
-    let (old, ior) =
+    let (_old, ior) =
       Lazy.force
         (Eliom_common.Full_state_name_table.find full_st_name !cookie_info)
     in
@@ -182,7 +177,7 @@ let rec find_or_create_data_cookie ?set_session_group
     | Eliom_common.SC c ->
         (match set_session_group with
           | None -> ()
-          | Some session_group ->
+          | Some _session_group ->
             let fullsessgrp = fullsessgrp ~cookie_level ~sp set_session_group in
             let node = Eliommod_sessiongroups.Data.move
               sitedata
@@ -261,6 +256,9 @@ let create_volatile_table, create_volatile_table_during_session =
     in
     sitedata.Eliom_common.remove_session_data <-
       (fun cookie ->
+         (* cookie is actually either a cookie or a a group name *)
+         (* In session group tables, keys may be either group names,
+            or a cookie values when no group name has been set. *)
         old_remove_session_data cookie;
         Eliom_common.SessionCookies.remove t cookie
       );
