@@ -63,8 +63,7 @@ let gc_timeouted_services now tables =
           Eliom_common.Serv_Table.fold
 (*VVV not tail recursive: may be a problem if lots of coservices *)
             (fun ptk (`Ptc (nodeopt, l)) thr ->
-               thr >>= fun thr -> (* we wait for the previous one
-                                     to be completed *)
+               let%lwt _ = thr in (* we wait for the previous one to be completed *)
                (match nodeopt, l with
                   | Some node, {Eliom_common.s_expire = Some (_, e)} :: _
                     (* it is an anonymous coservice.  The list should
@@ -103,7 +102,7 @@ let gc_timeouted_services now tables =
                               Eliom_common.Serv_Table.add
                                 ptk (`Ptc (nodeopt, newl)) ll
                       with Not_found -> ());
-                 Lwt_unix.yield ())
+                 Lwt.pause ())
             !ptr
             return_unit >>= fun () ->
           if Eliom_common.Serv_Table.is_empty !ptr
@@ -165,7 +164,7 @@ let gc_timeouted_naservices now tr =
                              tr := Eliom_common.remove_naservice_table !tr k
                       )
                   | _ -> ());
-               Lwt_unix.yield ()
+               Lwt.pause ()
             )
             t
             return_unit
@@ -194,14 +193,10 @@ let service_session_gc sitedata =
 
         (* private continuation tables: *)
         Eliom_common.SessionCookies.fold
-          (fun k (sessname,
-                  tables,
-                  exp,
-                  _,
-                  session_group_ref,
-                  session_group_node) thr ->
+          (fun k {Eliom_common.Service_cookie.session_table = tables; expiry;
+                                            session_group; session_group_node} thr ->
             thr >>= fun () ->
-            (match !exp with
+            (match !expiry with
             | Some exp when exp < now ->
               Eliommod_sessiongroups.Serv.remove session_group_node;
               Lwt.return_unit
@@ -213,8 +208,8 @@ let service_session_gc sitedata =
                then gc_timeouted_naservices now
                   tables.Eliom_common.table_naservices
                else return_unit) >>= fun () ->
-              (match !session_group_ref with
-              | (_, scope, Right _) (* no group *)
+              (match !session_group with
+              | (_, _scope, Right _) (* no group *)
 (*VVV check this *)
                   when
                     (Eliommod_sessiongroups.Serv.group_size
@@ -233,12 +228,12 @@ let service_session_gc sitedata =
               | _ -> () (*VVV enough? *));
               return_unit
             )
-            >>= Lwt_unix.yield
+            >>= Lwt.pause
           )
           service_cookie_table
           return_unit
         >>= f
-      in ignore (f ())
+      in Lwt.async f
 
 (* This is a thread that will work for example every hour. *)
 let data_session_gc sitedata =
@@ -254,17 +249,14 @@ let data_session_gc sitedata =
         Lwt_log.ign_info ~section "GC of session data";
         (* private continuation tables: *)
         Eliom_common.SessionCookies.fold
-          (fun k (sessname,
-                  exp, _,
-                  session_group_ref,
-                  session_group_node) thr ->
+          (fun k {Eliom_common.Data_cookie.expiry; session_group; session_group_node} thr ->
             thr >>= fun () ->
-            (match !exp with
+            (match !expiry with
                | Some exp when exp < now ->
                    Eliommod_sessiongroups.Data.remove session_group_node;
                    return_unit
                | _ ->
-                   match !session_group_ref with
+                   match !session_group with
                      | (_, scope, Right _) (* no group *)
                          when
                            (Eliommod_sessiongroups.Data.group_size
@@ -290,38 +282,47 @@ let data_session_gc sitedata =
                                Lwt.return_unit
                      | _ -> Lwt.return_unit
             )
-            >>= Lwt_unix.yield
+            >>= Lwt.pause
           )
           data_cookie_table
           return_unit
           >>=
         f
-      in ignore (f ())
+      in Lwt.async f
 
 (* garbage collection of timeouted persistent sessions *)
 (* This is a thread that will work every hour/day *)
 let persistent_session_gc sitedata =
+  let gc () =
+    let now = Unix.time () in
+    let log_hash c = Eliom_common.Hashed_cookies.(sha256 c) in
+    let do_gc_cookie cookie {Eliommod_cookies.full_state_name; expiry; session_group} =
+      let scope = full_state_name.Eliom_common.user_scope in
+      match expiry with
+      | Some exp when exp <= now ->
+        Lwt_log.ign_info_f ~section "remove expired cookie %s" (log_hash cookie);
+        Eliommod_persess.close_persistent_state2
+          ~scope
+          sitedata
+          session_group cookie
+      (*WAS: remove_from_all_persistent_tables k *)
+      | _ ->
+          Lwt_log.ign_info_f ~section "cookie not expired: %s" (log_hash cookie);
+          return_unit
+    in
+    let gc_cookie c =
+      Lwt.try_bind
+        (fun () -> Eliommod_cookies.Persistent_cookies.Cookies.find c)
+        (do_gc_cookie c)
+        (function
+         | Not_found ->
+             Lwt_log.ign_info_f ~section "cookie does not exist: %s" (log_hash c);
+             Lwt.return_unit
+         | exn -> Lwt.fail exn)
+    in
+    Lwt_log.ign_info ~section "GC of persistent sessions";
+    Eliommod_cookies.Persistent_cookies.garbage_collect ~section gc_cookie
+  in
   match get_persistentsessiongcfrequency () with
   | None -> () (* No garbage collection *)
-  | Some t ->
-      let rec f () =
-        Lwt_unix.sleep t >>=
-        (fun () ->
-          let now = Unix.time () in
-          Lwt_log.ign_info ~section "GC of persistent sessions";
-          Lazy.force Eliommod_persess.persistent_cookies_table >>=
-          Ocsipersist.iter_table
-             (fun k ((scope, _, _), exp, _, session_group) ->
-               (match exp with
-               | Some exp when exp < now ->
-(*VVV ? *)
-                 Eliommod_persess.close_persistent_state2
-                   ~scope
-                   sitedata
-                   session_group k
-               (*WAS: remove_from_all_persistent_tables k *)
-               | _ -> return_unit)
-             ))
-          >>=
-        f
-      in ignore (f ())
+  | Some t -> let rec f () = Lwt_unix.sleep t >>= gc >>= f in Lwt.async f
