@@ -23,7 +23,7 @@ val close_service_state :
   ?sp:Eliom_common.server_params -> unit -> unit
 val find_or_create_service_cookie :
   ?set_session_group:string ->
-  cookie_scope:[< Eliom_common.cookie_scope > `Session ] ->
+  cookie_scope:[< Eliom_common.cookie_scope] ->
   secure_o: bool option ->
   ?sp:Eliom_common.server_params ->
   unit -> Eliom_common.tables Eliom_common.one_service_cookie_info
