@@ -30,7 +30,7 @@ let make_full_group_name ~cookie_level ri site_dir_string ipv4mask ipv6mask =
              cookie_level,
              Right
                (Eliom_common.network_of_ip
-                  (!!(Ocsigen_extensions.Ocsigen_request_info.remote_ip_parsed ri))
+                  (Ocsigen_request.remote_ip_parsed ri)
                   ipv4mask
                   ipv6mask
                ))
@@ -164,6 +164,7 @@ struct
       let cl = Ocsigen_cache.Dlist.create size in
       Ocsigen_cache.Dlist.set_finaliser_after
         (fun node ->
+           (* Finaliser of sessions and client processes *)
           let name = Ocsigen_cache.Dlist.value node in
           (* First we close all subsessions
              (that is, all sessions in the group associated to the session) *)
@@ -297,16 +298,16 @@ Besides, volatile sessions are (hopefully) going to disappear soon.
       match (sess_grp : GroupTable.key) with
         | (_, `Client_process, Left sess_id) ->
           (try
-             let (_, _, _, sgr, sgn) =
+             let {Eliom_common.Data_cookie.session_group; session_group_node} =
                Eliom_common.SessionCookies.find
                  sitedata.Eliom_common.session_data sess_id
              in
-             (match !sgr with
+             (match !session_group with
                | (_, `Session, Right _) (* no group *)
                    when sitedata.Eliom_common.not_bound_in_data_tables
                      sess_id
                      ->
-                 remove1 sgn
+                 remove1 session_group_node
                | _ -> ()
              )
            with Not_found -> ())
@@ -367,12 +368,13 @@ Besides, volatile sessions are (hopefully) going to disappear soon.
         | (_, `Client_process, Left sess_id) ->
           (try
 
-             let (_, tables, _, _, sgr, sgn) =
+             let {Eliom_common.Service_cookie.session_table = tables;
+                                              session_group_node} =
                Eliom_common.SessionCookies.find
                  sitedata.Eliom_common.session_services sess_id
              in
              if Eliom_common.service_tables_are_empty tables
-             then remove1 sgn
+             then remove1 session_group_node
 
            with Not_found -> ())
         | (_, `Session, _) ->
@@ -411,7 +413,9 @@ let cut n l =
 
 module Pers = struct
 (*VVV Verify this carefully! *)
-(*VVV VEOcsigen_request_infoFY concurrent access *)
+(*VVV VERIFY concurrent access *)
+
+  module Ocsipersist = Eliom_common.Ocsipersist.Polymorphic
 
   let grouptable : (nbmax * string list) Ocsipersist.table Lwt.t Lazy.t =
     lazy (Ocsipersist.open_table "__eliom_session_group_table")
@@ -471,7 +475,7 @@ module Pers = struct
   let rec remove_group ~cookie_level sitedata sess_grp =
     (* cookie_level is the scope of group members *)
 (*VVV NEW 201007 closing all sessions in the group and removing group data *)
-(*VVV VEOcsigen_request_infoFY concurrent access *)
+(*VVV VERIFY concurrent access *)
 (*VVV Check this carefully!!!! Verify the order of actions. *)
     Lwt.catch
       (fun () ->
@@ -486,27 +490,29 @@ module Pers = struct
         >>= fun () ->
 
         (* Then, we remove group data: *)
-        let group_name =
-          match sess_grp with
-            | Some sg ->
-              (match Eliom_common.getperssessgrp sg with
-                | (_, _, Left s) -> s
-                | _ -> Eliom_common.default_group_name)
-            | None -> Eliom_common.default_group_name
-        in
-        Eliom_common.remove_from_all_persistent_tables group_name >>= fun () ->
-
-        (* If it is associated to a session,
-           we remove the session from its group,
-           and we remove cookie info: *)
-        (match cookie_level with
-          | `Client_process grp -> (* We are closing a browser session,
-                                      belonging to the group grp *)
-            (* group_name is the cookie value *)
-            remove sitedata group_name grp >>= fun () ->
-            !!Eliom_common.persistent_cookies_table >>= fun table ->
-            Ocsipersist.remove table group_name
-          | _ -> Lwt.return_unit)
+        (match sess_grp with
+         | None -> Lwt.return_unit
+         | Some sg ->
+           (match Eliom_common.getperssessgrp sg with
+            | (_, _, Right _) ->
+              (* No group has been set. No group table.
+                 Data associated to default (automatic) groups
+                 is removed when closing associated sessions. *)
+              Lwt.return_unit
+            | (_, _, Left group_name) ->
+              Eliom_common.Persistent_tables.remove_key_from_all_tables group_name
+              >>= fun () ->
+              (* If it is associated to a session,
+                 we remove the session from its group,
+                 and we remove cookie info: *)
+              (match cookie_level with
+               | `Client_process grp -> (* We are closing a browser session,
+                                           belonging to the group grp *)
+                 (* group_name is the cookie value *)
+                 remove sitedata group_name grp
+               | _ -> Lwt.return_unit)
+           )
+        )
         >>= fun () ->
 
         (* Then, we remove group from group table: *)
@@ -528,16 +534,11 @@ module Pers = struct
       (fun () ->
         match cookie_level with
           | `Client_process -> begin
-            (* We remove cookie info from the table *)
-            !!Eliom_common.persistent_cookies_table >>= fun table ->
-            Ocsipersist.remove table cookie
-            >>= fun () ->
-
             (* We remove the session from its group: *)
             remove sitedata cookie fullsessgrp >>= fun () ->
 
             (* Then, we remove session data: *)
-            Eliom_common.remove_from_all_persistent_tables cookie
+            Eliom_common.Persistent_tables.remove_key_from_all_tables cookie
           end
           | `Session ->
             remove_group
@@ -553,7 +554,7 @@ module Pers = struct
         | e -> Lwt.fail e)
 
 
-  and remove sitedata sess_id sess_grp =
+  and remove _sitedata sess_id sess_grp =
     match sess_grp with
     | Some sg0 ->
       let sg = Eliom_common.string_of_perssessgrp sg0 in
