@@ -8,11 +8,15 @@ let usage () =
   Printf.eprintf "  -eliom-inc <dir>\tAdd <dir> to the list of eliom include directories (prepend eliom build directories)\n";
   Printf.eprintf "  -package <name>\tRefer to package when compiling\n";
   Printf.eprintf "  -no-autoload\t\tDo not load commonly used syntax extensions (deriving, lwt, js_of_ocaml, tyxml)\n";
-  Printf.eprintf "  -type-conv\t\tUse type_conv syntax extensions instead of deriving one\n";
-  Printf.eprintf "\t\t\tIt has no effect if used in conjunction with -no-autoload\n";
   Printf.eprintf "  -ppopt <p>\t\tAppend option <opt> to preprocessor invocation\n";
   Printf.eprintf "  -predicates <p>\tAdd predicate <p> when resolving package properties\n";
   Printf.eprintf "  -verbose\t\tPrint calls to external commands\n";
+  Printf.eprintf "  -ppx";
+  Printf.eprintf "\t\t\tUse the PPX Eliom syntax extension (default: Camlp4)\n";
+  Printf.eprintf "\t\t\tThe above description only applies to the first\n";
+  Printf.eprintf "\t\t\tappearance of -ppx in the list of flags. Subsequent\n";
+  Printf.eprintf "\t\t\tappearances require an argument and specify a\n";
+  Printf.eprintf "\t\t\tPPX preprocessor to use (see STANDARD OPTIONS).\n";
   create_filter !compiler ["-help"] (help_filter 2 "STANDARD OPTIONS:");
   exit 1
 
@@ -44,35 +48,42 @@ let in_an_eliom_inc_dir s =
 
 let compile_intf file =
   wait (
-  create_process
-    !compiler ( "-pp" :: get_pp [] !ppopt :: !args @
-          (get_default_args ())
-        @ (get_common_include ())
-        @ (map_include !eliom_inc_dirs)
-		@ ["-intf"; file] ))
+    create_process
+      !compiler (
+      preprocess_opt !ppopt
+      @ !args
+      @ (get_default_args ())
+      @ (get_common_include ())
+      @ (map_include !eliom_inc_dirs)
+      @ ["-intf"; file] ))
 
 let compile_impl file =
   wait (
-  create_process
-    !compiler ( "-pp" :: get_pp [] !ppopt :: !args @
-          (get_default_args ())
-        @ (get_common_include ())
-        @ (map_include !eliom_inc_dirs)
-		@ ["-impl"; file] ))
+    create_process
+      !compiler (
+      preprocess_opt !ppopt
+      @ !args
+      @ (get_default_args ())
+      @ (get_common_include ())
+      @ (map_include !eliom_inc_dirs)
+      @ ["-impl"; file] ))
 
 let server_pp_opt impl_intf =
-  ["-printer"; "o"; "-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
+  match !pp_mode with
+  | `Camlp4 ->
+    "-printer" :: "o" :: !ppopt @ [impl_intf_opt impl_intf]
+  | `Ppx ->
+    !ppopt
+
 let client_pp_opt impl_intf =
-  ["-printer"; "o"; "-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
+  match !pp_mode with
+  | `Camlp4 ->
+    "-printer" :: "o" :: !ppopt @ [impl_intf_opt impl_intf]
+  | `Ppx ->
+    !ppopt
 
 let generate_temp_file file =
-  let tmp_dir =
-#if ocaml_version < (4, 00)
-    Filename.temp_dir_name
-#else
-    (Filename.get_temp_dir_name ())
-#endif
-  in
+  let tmp_dir = Filename.get_temp_dir_name () in
   let temp_file =
     tmp_dir
     ^ (Filename.dir_sep)
@@ -85,8 +96,9 @@ let compile_server_eliom ~impl_intf file =
   wait (create_process ~out "eliompp" ["-server"; file]);
   wait (
   create_process !compiler
-    ( "-pp" :: get_pp ["eliom.syntax.server"] (server_pp_opt impl_intf) :: !args @
-        (get_default_args ())
+    ( preprocess_opt ~kind:`Server (server_pp_opt impl_intf)
+      @ !args
+      @ (get_default_args ())
       @ (get_common_include ())
       @ (map_include !eliom_inc_dirs)
       @ [impl_intf_opt impl_intf; file'] ))
@@ -96,8 +108,9 @@ let compile_client_eliom ~impl_intf file =
   wait (create_process ~out "eliompp" ["-client"; file]);
   wait (
   create_process !compiler
-    ( "-pp" :: get_pp ["eliom.syntax.client"] (client_pp_opt impl_intf) :: !args @
-        (get_default_args ())
+    ( preprocess_opt ~kind:`Client (client_pp_opt impl_intf)
+      @ !args
+      @ (get_default_args ())
       @ (get_common_include ())
       @ (map_include !eliom_inc_dirs)
       @ [impl_intf_opt impl_intf; file'] ))
@@ -117,12 +130,12 @@ let generate_doc () =
       wait (create_process !compiler ("-load" :: file :: !args))
 
 let process_option () =
-  let i = ref 2 in
+  let i = ref 2
+  and ppx = ref false in
   while !i < Array.length Sys.argv do
     match Sys.argv.(!i) with
     | "-verbose" -> verbose := true; incr i
     | "-no-autoload" -> autoload_predef := false; incr i
-    | "-type-conv" -> type_conv := true; incr i
     | "-eliom-inc" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       let dir = Sys.argv.(!i+1) in
@@ -141,6 +154,10 @@ let process_option () =
       if !i+1 >= Array.length Sys.argv then usage ();
       pp := Some Sys.argv.(!i+1);
       i := !i+2
+    | "-ppx" when not !ppx ->
+      ppx := true;
+      pp_mode := `Ppx;
+      i := !i+1
     | "-ppopt" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       ppopt := !ppopt @ [Sys.argv.(!i+1)];
