@@ -5,9 +5,9 @@ let remove_spaces s beg endd =
   let rec find_not_space s i step =
     if (i > endd) || (beg > i) then i
     else
-      if s.[i] = ' '
-      then find_not_space s (i+step) step
-      else i
+    if s.[i] = ' '
+    then find_not_space s (i+step) step
+    else i
   in
   let first = find_not_space s beg 1 in
   let last = find_not_space s endd (-1) in
@@ -20,11 +20,11 @@ let split c s =
   let rec aux deb =
     if deb >= longueur then []
     else try
-      let firstsep = String.index_from s deb c in
-      if firstsep = deb then
-        aux (deb + 1)
-      else
-        (remove_spaces s deb (firstsep-1))::
+        let firstsep = String.index_from s deb c in
+        if firstsep = deb then
+          aux (deb + 1)
+        else
+          (remove_spaces s deb (firstsep-1))::
           (aux (firstsep+1))
       with Not_found -> [remove_spaces s deb (longueur-1)]
   in
@@ -41,11 +41,15 @@ let args : string list ref = ref []
 let package : string list ref = ref []
 let predicates : string list ref = ref []
 let pp : string option ref = ref None
-let kind : [ `Server | `Client | `ServerOpt ] ref = ref `Server
+type kind = [ `Server | `Client | `ServerOpt ]
+let kind : kind ref = ref `Server
 let type_file : string option ref = ref None
 
 let autoload_predef = ref true
-let type_conv = ref false
+
+type pp_mode = [ `Camlp4 | `Ppx ]
+let pp_mode : pp_mode ref = ref `Camlp4
+
 
 let default_server_dir =
   try Sys.getenv "ELIOM_SERVER_DIR"
@@ -68,8 +72,8 @@ let type_dir : string ref = ref default_type_dir
 
 let get_kind k =
   match k with
-    | Some k -> k
-    | None -> !kind
+  | Some k -> k
+  | None -> (!kind : kind :> [> kind])
 
 (** Findlib *)
 
@@ -106,18 +110,15 @@ let with_autoload all_pkgs =
   if !autoload_predef
   then begin
     (* Format.eprintf "\nAUTOLOADING PREDEF PKGS\n%s\n@." (String.concat ", " all_pkgs); *)
-    let l = "eliom.syntax.predef"::all_pkgs in
-    if !type_conv
-    then "deriving.syntax.tc"::"type_conv"::l
-    else "deriving.syntax.std"::l
+    "eliom.syntax.predef"::all_pkgs
   end
   else all_pkgs
 
 let get_server_package ?kind:k ?package:p () =
   let package =
     match p with
-      | Some p -> p
-      | None -> !package
+    | Some p -> p
+    | None -> !package
   in
   let pkgs = "eliom.server" :: package in
   let pkg_predicates = get_pkg_predicates pkgs in
@@ -140,30 +141,6 @@ let get_client_package ?kind:k () =
     Printf.eprintf "Unknown package: %s\n%!" name;
     exit 1
 
-let explain_who_need ~find ~from =
-  let pkg_predicates = get_pkg_predicates from in
-  (* remove pkg_type_conv from predicates to exclude weak dependencies *)
-  (* Weak dependencies: pa_eliom_seed needs to be loaded after type_conv IF type_conv EXISTS *)
-  let pkg_predicates = List.filter ((<>) "pkg_type_conv") pkg_predicates in
-  let l = List.filter (fun pkg ->
-      if
-        try
-          begin
-	          let objs = Findlib.package_property (pkg_predicates @ Lazy.force syntax_predicates) pkg "archive"
-	          in List.concat (List.map (split ',') (split ' ' objs)) <> []
-          end
-	      with Not_found -> false
-      then
-        begin
-          let l = Findlib.package_deep_ancestors (pkg_predicates @ Lazy.force syntax_predicates) [pkg] in
-          List.mem find l
-        end
-      else false
-    ) from in
-  match l with
-  | [] -> ()
-  | _ -> Printf.eprintf "List of packages requiring %s: %s.\n%!" find (String.concat ", " l)
-
 let get_syntax_package pkg =
   let resolve_syntax_packages pkgs =
     (* Format.eprintf "pkgs: %s@." (String.concat ", " pkgs); *)
@@ -174,38 +151,68 @@ let get_syntax_package pkg =
            (fun p ->
               let all_predicates = pkg_predicates @ (Lazy.force syntax_predicates) in
               try
-	              let objs =
-	                Findlib.package_property all_predicates  p "archive" in
-	              List.concat (List.map (split ',') (split ' ' objs)) <> []
-	            with Not_found -> false)
+                let objs =
+                  Findlib.package_property all_predicates  p "archive" in
+                List.concat (List.map (split ',') (split ' ' objs)) <> []
+              with Not_found -> false)
            pkgs)
     with Findlib.No_such_package (name, _) ->
       Printf.eprintf "Unknown package: %s\n%!" name;
       exit 1 in
   let all_pkgs = pkg @ !package in
-  let resolved_pkgs = resolve_syntax_packages (with_autoload all_pkgs) in
-  if !autoload_predef
-  then
-    if !type_conv && List.mem "deriving.syntax.std" resolved_pkgs
-    then begin
-      Printf.eprintf "Error: '-type-conv' option enabled but some packages require the deriving syntax to be loaded.\n%!";
-      explain_who_need ~find:"deriving.syntax.std" ~from:all_pkgs;
-      exit 1
-    end
-    else if not !type_conv && List.mem "type_conv" resolved_pkgs
-    then begin
-      Printf.eprintf "Error: '-type-conv' option disabled but some packages require the type_conv syntax to be loaded.\n%!";
-      explain_who_need ~find:"type_conv" ~from:all_pkgs;
-      exit 1
-    end
-    else resolved_pkgs
-  else resolved_pkgs
+  resolve_syntax_packages (with_autoload all_pkgs)
 
 let has_package name =
   try
     ignore(Findlib.package_directory name);true
   with Findlib.No_such_package (_, _) -> false
 
+let get_ppxs l =
+  let meta_ppx_opts =
+    List.concat
+      (List.map
+         (fun pname ->
+            try
+              let opts = Findlib.package_property [] pname "ppxopt" in
+              List.concat
+                (List.map
+                   (fun opts ->
+                     match split ',' opts with
+                     | pkg :: opts ->
+                       [pkg, (pname, opts)]
+                     | [] ->
+                       [])
+                   (split ' ' opts))
+            with Not_found ->
+              [])
+         l)
+  in
+
+  let f p acc =
+    let d = Findlib.package_directory p in
+    try
+      let ppx = Findlib.package_property [] p "ppx" in
+      let ppx = Findlib.resolve_path ~base:d ~explicit:true ppx in
+      let options =
+        List.concat
+          (List.map
+             (fun (_, (pname, opts)) ->
+                let base = Findlib.package_directory pname in
+                List.map (Findlib.resolve_path ~base ~explicit:true) opts)
+             (List.filter (fun (p', _) -> p' = p) meta_ppx_opts))
+      in
+      "-ppx" :: String.concat " " (ppx :: options) :: acc
+    with Not_found -> acc
+  in
+  List.fold_right f l []
+
+let get_common_ppx ?kind ?package () =
+  get_ppxs @@ match get_kind kind with
+  | `Server | `ServerOpt ->
+    get_server_package ?kind ?package ()
+  | `Client ->
+    get_client_package ?kind ()
+
 let rec map_include xs = match xs with
   | [] -> []
   | x::xs -> "-I" :: x :: map_include xs
@@ -213,13 +220,15 @@ let rec map_include xs = match xs with
 let get_common_include ?kind:k ?build_dir:dir ?package:p () =
   let dir = match dir with Some d -> d | None -> !build_dir in
   (match get_kind k with
-  | `Server | `ServerOpt ->
-      map_include (List.map Findlib.package_directory (get_server_package ?kind:k ?package:p ()))
-  | `Client ->
-      map_include (List.map Findlib.package_directory (get_client_package ?kind:k ())))
+   | `Server | `ServerOpt ->
+     "js_of_ocaml" :: get_server_package ?kind:k ?package:p ()
+     |> List.map Findlib.package_directory
+     |> map_include
+   | `Client ->
+     map_include (List.map Findlib.package_directory (get_client_package ?kind:k ())))
   @ match dir with
-    | "" | "." -> []
-    | d -> ["-I"; d]
+  | "" | "." -> []
+  | d -> ["-I"; d]
 
 let get_common_syntax pkg =
   let syntax_pkg = get_syntax_package pkg in
@@ -239,35 +248,36 @@ let get_client_lib ?kind:k () =
   List.concat
     (List.map
        (fun p ->
-	 try
-	   split ' ' (Findlib.package_property (get_predicates ?kind:k ()) p "archive")
-	 with Not_found -> [])
+          try
+            split ' ' (Findlib.package_property (get_predicates ?kind:k ()) p "archive")
+          with Not_found -> [])
        (get_client_package ?kind:k ()))
 
 let get_client_js () =
   [
     "+eliom.client/eliom_client.js" ;
-    "+js_of_ocaml/weak.js";
-    "+js_of_ocaml/classlist.js" (* ie9 support *)
   ]
 
-(* Should be calld only with -dump... *)
+(* Should be called only with -dump... *)
 let get_pp_dump pkg opt = match !pp with
   | None -> (!camlp4, get_common_syntax pkg @ opt)
   | Some pp ->
-      try
-        ignore(String.index pp ' ');
-        Printf.eprintf "Incompatible option: -pp and -dump\n%!";
-        exit 1
-      with Not_found -> (pp, get_common_syntax pkg @ opt)
+    try
+      ignore(String.index pp ' ');
+      Printf.eprintf "Incompatible option: -pp and -dump\n%!";
+      exit 1
+    with Not_found -> (pp, get_common_syntax pkg @ opt)
 
-let get_pp pkg opt =
+let get_pp pkg =
   let s = match !pp with
-    | None -> String.concat " " (!camlp4 :: get_common_syntax pkg @ opt)
-    | Some pp -> pp ^ " " ^ String.concat " " (get_common_syntax pkg @ opt)
-  (* Format.eprintf "get_pp %S@." s *)
+    | None -> String.concat " " (!camlp4 :: get_common_syntax pkg)
+    | Some pp -> pp ^ " " ^ String.concat " " (get_common_syntax pkg)
+    (* Format.eprintf "get_pp %S@." s *)
   in s
 
+let get_ppx pkg =
+  Findlib.package_property [] pkg "ppx"
+
 let get_thread_opt () = match !kind with
   | `Client -> []
   | `Server | `ServerOpt -> ["-thread"]
@@ -283,7 +293,7 @@ let get_type_file file =
   match !type_file with
   | Some f -> f
   | None ->
-      prefix_type_dir (chop_extension_if_any file ^ !server_types_file_ext)
+    prefix_type_dir (chop_extension_if_any file ^ !server_types_file_ext)
 
 let impl_intf_opt = function
   | `Impl -> "-impl"
@@ -294,6 +304,33 @@ let type_opt impl_intf file =
   | `Impl -> ["-type"; get_type_file file]
   | `Intf -> ["-notype"]
 
+let simplify_kind ?kind () =
+  match get_kind kind with
+  | `Client -> `Client
+  | `Server | `ServerOpt -> `Server
+  | `Types -> `Types
+
+let get_ppopts ~impl_intf file =
+  match !pp_mode with
+  | `Camlp4 ->
+    type_opt impl_intf file @ !ppopt @ [impl_intf_opt impl_intf]
+  | `Ppx ->
+    type_opt impl_intf file @ !ppopt
+
+let preprocess_opt ?(ocaml = false) ?kind opts =
+  match !pp_mode with
+  | `Camlp4 ->
+    [ "-pp"; get_pp [] ^ " " ^ String.concat " " opts ]
+  | `Ppx when ocaml ->
+    []
+  | `Ppx ->
+    let pkg = match simplify_kind ?kind () with
+      | `Client -> "eliom.ppx.client"
+      | `Server -> "eliom.ppx.server"
+      | `Types  -> "eliom.ppx.type"
+    in
+    [ "-ppx"; get_ppx pkg ^ " " ^ String.concat " " opts ]
+
 (** Process *)
 
 let rec wait ?(on_error=fun _ -> ()) pid =
@@ -302,13 +339,13 @@ let rec wait ?(on_error=fun _ -> ()) pid =
   | Unix.WEXITED 0 -> ()
   | Unix.WEXITED i -> on_error e; exit i
   | Unix.WSIGNALED i ->
-      Printf.eprintf "Child killed with signal: %d" i;
-      on_error e;
-      exit 255
+    Printf.eprintf "Child killed with signal: %d" i;
+    on_error e;
+    exit 255
   | Unix.WSTOPPED i ->
-      Printf.eprintf "Child stoped with signal: %d" i;
-      on_error e;
-      wait pid
+    Printf.eprintf "Child stopped with signal: %d" i;
+    on_error e;
+    wait pid
 
 let create_process
     ?(in_= Unix.stdin) ?(out= Unix.stdout) ?(err= Unix.stderr)
@@ -335,8 +372,8 @@ let help_filter skip msg ch =
 let fail fmt =
   Printf.ksprintf
     (fun msg ->
-      prerr_endline msg;
-      exit 1)
+       prerr_endline msg;
+       exit 1)
     fmt
 
 (** *)
